"use strict";
var q = require('q');
var q_1 = require('q');
var path = require('path');
var cp = require('child_process');
var os = require('os');
var vscode_jsonrpc_1 = require('vscode-jsonrpc');
var common_1 = require('./common');
var config_1 = require('./config');
var exitCode_1 = require('./exitCode');
var logger_1 = require('./logger');
var connect_1 = require('./connect');
var ec = require('errno-codes');
var connectTimeoutMs = 5000;
var ControllerConnection = (function () {
    function ControllerConnection(options) {
        this.options = options;
        this.connectCts = new vscode_jsonrpc_1.CancellationTokenSource();
        this.options = options || {};
        this.options.clientName = this.options.clientName || 'hubControllerLookup';
        this.options.logger = this.options.logger || new logger_1.Logger(this.options.clientName);
        this.options.getConfig = this.options.getConfig || config_1.getConfig;
        this.options.getPipeName = this.options.getPipeName || common_1.getLocationServicePipeName;
        this.pipeName = this.options.getPipeName();
        this.deferredControllerExitCode = q_1.defer();
        this.controllerExitCode = this.deferredControllerExitCode.promise;
    }
    ControllerConnection.connect = function (options) {
        var connection = new ControllerConnection(options);
        return connection.connectImpl();
    };
    ControllerConnection.prototype.connectImpl = function () {
        var _this = this;
        var streamPromise;
        // If we're in ServiceHub host process, it must have environment variable 'ServiceHubLocationServicePipeName' set.
        // We use this environment to populate this.pipeName via getLocationServicePipeName().
        // If the pipe name is already known, this means we're running in context of service hub and there is no need to start another controller.
        if (this.pipeName) {
            // Since we don't start the controller, we may not know its exit code.
            this.deferredControllerExitCode.resolve(undefined);
            streamPromise = connect_1.connectWithRetries({ url: this.pipeName, timeoutMs: connectTimeoutMs });
        }
        else {
            streamPromise = this.startController();
        }
        return streamPromise
            .then(function (stream) {
            _this.stream = stream;
            _this.isConnected = true;
            return _this;
        });
    };
    ControllerConnection.prototype.startController = function () {
        var _this = this;
        return this.options.getConfig()
            .then(function (config) {
            _this.pipeName = config.controller.pipeName;
            // If we can connect immediately without starting new controller, don't bother to start it.
            var connectPromise = _this.options.dontTryConnectBeforeStartingController ? q.reject() : connect_1.connect(_this.pipeName);
            return connectPromise
                .then(function (stream) {
                // Since we could just connect and don't start the controller, we will not know its exit code.
                _this.deferredControllerExitCode.resolve(undefined);
                return stream;
            }, function (err) {
                // We could not connect. Start a new controller.
                var command = config.controller.executable;
                // If an empty set of quotes was passed as an argument, replace it with an empty space surrounded by quotes so that the arg isn't removed when passed to cmd.exe
                var args = config.controller.args.map(function (arg) { return arg.indexOf('\"\"') >= 0 ? '\" \"' : arg; });
                var processOptions = {
                    cwd: config.filePath ? path.dirname(config.filePath) : undefined,
                    detached: true,
                    windowsHide: true
                };
                if (config.controller.additionalEnvironmentVariables) {
                    processOptions.env = addControllerEnv(cloneProcessEnv(), config.controller.additionalEnvironmentVariables);
                }
                var isExeActuallyDll = command.substr((command.lastIndexOf('.') + 1)) === 'dll';
                if (isExeActuallyDll) {
                    if (os.platform() === 'win32') {
                        command = 'dotnet ' + config.controller.executable;
                    }
                    else {
                        command = '/bin/bash ';
                        args = ['dotnet', config.controller.executable].concat(args);
                    }
                }
                _this.options.logger.info("Starting Service Hub controller '" + command + " " + args.join(' ') + "'.");
                // Because the controller is now CLR based there would never be a scenario when there's a space in both the exe path and an arg 
                // because we are no longer passing a js file to it so the controller exe can be spawned directly instead of using cmd.exe and a batch script.
                _this.controllerProcess = cp.spawn(command, args, processOptions);
                _this.controllerProcess.once('error', function (err) { return _this.onControllerProcessExited(0, null, err); });
                _this.controllerProcess.once('exit', function (code, signal) { return _this.onControllerProcessExited(code, signal); });
                logger_1.traceProcessOutput(_this.options.logger, _this.controllerProcess, 'HubController');
                return connect_1.connectWithRetries({ url: _this.pipeName, timeoutMs: connectTimeoutMs, cancellationToken: _this.connectCts.token })
                    .catch(function (err) {
                    throw _this.controllerError || err; // If connect fails, controller process error may provide more insights than the pipe error.
                });
            });
        });
    };
    ControllerConnection.prototype.onControllerProcessExited = function (exitCode, signal, err) {
        this.deferredControllerExitCode.resolve(exitCode);
        // Controller may exit because there is another controller running already.
        // This is expectected, not an error.
        if (exitCode === exitCode_1.default.ErrorStartingServerPipeInUse) {
            return;
        }
        // Controller can eventually exit after we have connected, e.g. due to inactivity.
        // No point in cancelling the connection in this case.
        // However, we will log an error if the exit code was not 0, or if node couldn't start the process,
        // and a warning if the controller process was killed.
        if (this.isConnected) {
            if (err) {
                this.error("Hub Controller process could not start properly: " + err.message);
            }
            else if (signal) {
                this.options.logger.warn("Hub Controller process exited with signal '" + signal + "'.");
            }
            else if (exitCode !== exitCode_1.default.Success) {
                var exitCodeMessage = "" + exitCode + (exitCode_1.default[exitCode] ? " (" + exitCode_1.default[exitCode] + ")" : '');
                this.error("Hub controller process exited with exit code " + exitCodeMessage + ".");
            }
            return;
        }
        var error;
        if (err) {
            error = new Error("Hub Controller process could not start properly: " + err.message);
            error.inner = err;
        }
        else if (signal) {
            error = new Error("Hub Controller process exited prematurely with signal '" + signal + "'.");
            error.signal = signal;
        }
        else {
            var exitCodeMessage = "" + exitCode + (exitCode_1.default[exitCode] ? " (" + exitCode_1.default[exitCode] + ")" : '');
            error = new Error("Hub Controller process exited prematurely with exit code " + exitCodeMessage + ".");
            error.exitCode = exitCode;
        }
        this.error(error.message);
        this.controllerError = error;
        // No point in trying to connect to the Hub Controller pipe if the process has exited prematurely.
        this.connectCts.cancel();
    };
    ControllerConnection.prototype.error = function (message) {
        this.options.logger.error(message);
    };
    return ControllerConnection;
}());
exports.ControllerConnection = ControllerConnection;
function cloneProcessEnv() {
    var clonedEnv = {};
    for (var env in process.env) {
        clonedEnv[env] = process.env[env];
    }
    return clonedEnv;
}
function addControllerEnv(targetEnv, controllerEnv) {
    for (var env in controllerEnv) {
        targetEnv[env] = controllerEnv[env];
    }
    return targetEnv;
}

//# sourceMappingURL=controllerConnection.js.map
// SIG // Begin signature block
// SIG // MIIiRwYJKoZIhvcNAQcCoIIiODCCIjQCAQExDzANBglg
// SIG // hkgBZQMEAgEFADB3BgorBgEEAYI3AgEEoGkwZzAyBgor
// SIG // BgEEAYI3AgEeMCQCAQEEEBDgyQbOONQRoqMAEEvTUJAC
// SIG // AQACAQACAQACAQACAQAwMTANBglghkgBZQMEAgEFAAQg
// SIG // 0r975lYm9DAj9nn/1qt8VA0pPClkjkBNHpTqMuh32JOg
// SIG // ggtyMIIE+jCCA+KgAwIBAgITMwAAAs58ms59kF7StwAA
// SIG // AAACzjANBgkqhkiG9w0BAQsFADB+MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBT
// SIG // aWduaW5nIFBDQSAyMDEwMB4XDTE5MDUwMjIxMjU0MloX
// SIG // DTIwMDUwMjIxMjU0MlowdDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEeMBwGA1UEAxMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
// SIG // owhbo0atJEL/giHpwS2stH0eY6h2vghqYkLwa9LT5G5k
// SIG // q5QvKepTYRPV5tH0f5xTRTFTuZopGubS+mDvUkm5CF3C
// SIG // 0w4hucK3Mr8ad7l2yeEmZ7Tikgro6rzUApaNX3pEmKmt
// SIG // 2cPaAJ5XnM4DgC6Q5VV6gX3C/NO+h7ss51xKRb662Hjp
// SIG // y4s1p1weopn+gyWVpQwGNgGwTAcSguLN7XpFUl1esVAC
// SIG // GzvS4MIb8qsl1+aXWvgNUMmzSAwlIHDRnmd6HejS5HEM
// SIG // m4Ye2bYmRKb6Bc7I4hsi3pIm304LK4vSAdLH2HElS7VY
// SIG // mofTuiU2MZUNWaz+14kv5R4yRn+ojSNQUQIDAQABo4IB
// SIG // eTCCAXUwHwYDVR0lBBgwFgYKKwYBBAGCNz0GAQYIKwYB
// SIG // BQUHAwMwHQYDVR0OBBYEFHM/DO7M0NXkSmSFJIwE5oKt
// SIG // gMgPMFAGA1UdEQRJMEekRTBDMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEWMBQG
// SIG // A1UEBRMNMjMwODY1KzQ1NDI0MzAfBgNVHSMEGDAWgBTm
// SIG // /F97uyIAWORyTrX0IXQjMubvrDBWBgNVHR8ETzBNMEug
// SIG // SaBHhkVodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NybC9wcm9kdWN0cy9NaWNDb2RTaWdQQ0FfMjAxMC0w
// SIG // Ny0wNi5jcmwwWgYIKwYBBQUHAQEETjBMMEoGCCsGAQUF
// SIG // BzAChj5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NlcnRzL01pY0NvZFNpZ1BDQV8yMDEwLTA3LTA2LmNy
// SIG // dDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IB
// SIG // AQAcPuovja14/IUbUPK1RjGlfqowbC45KppnIgLxztfM
// SIG // +gbG8nz62UhMR8MjrPM2gyl+dwxqSm5kmAqQ0zeU/lTA
// SIG // z3wH6dvaa8gWwR5gCRYCu0z/s1sR7pjXlQpEILf6QP3t
// SIG // PTg+CEhYbIYxUH+Hr8SQisaxZWCC/rTywdBanMHIMQ7L
// SIG // LZbYaEjmkXmjSzIq4wGjaRRvoJ1cQUXLe0BHgaycOCgk
// SIG // TtJMON5OUqZB3C4rqcjlQmp8/THwHeSGwZgVPYQJCNZi
// SIG // j3V24F1nEKDwiazXleVkJ6smBpa4wBYbPFGR/VzTW49o
// SIG // gr51TH3e4IENba6DsZIhnrk2YP7s/xvsbKdGMIIGcDCC
// SIG // BFigAwIBAgIKYQxSTAAAAAAAAzANBgkqhkiG9w0BAQsF
// SIG // ADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMp
// SIG // TWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9y
// SIG // aXR5IDIwMTAwHhcNMTAwNzA2MjA0MDE3WhcNMjUwNzA2
// SIG // MjA1MDE3WjB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMK
// SIG // V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
// SIG // A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYD
// SIG // VQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBDQSAy
// SIG // MDEwMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
// SIG // AQEA6Q5kUHlntcTj/QkATJ6UrPdWaOpE2M/FWE+ppXZ8
// SIG // bUW60zmStKQe+fllguQX0o/9RJwI6GWTzixVhL99COMu
// SIG // K6hBKxi3oktuSUxrFQfe0dLCiR5xlM21f0u0rwjYzIjW
// SIG // axeUOpPOJj/s5v40mFfVHV1J9rIqLtWFu1k/+JC0K4N0
// SIG // yiuzO0bj8EZJwRdmVMkcvR3EVWJXcvhnuSUgNN5dpqWV
// SIG // XqsogM3Vsp7lA7Vj07IUyMHIiiYKWX8H7P8O7YASNUwS
// SIG // pr5SW/Wm2uCLC0h31oVH1RC5xuiq7otqLQVcYMa0Kluc
// SIG // IxxfReMaFB5vN8sZM4BqiU2jamZjeJPVMM+VHwIDAQAB
// SIG // o4IB4zCCAd8wEAYJKwYBBAGCNxUBBAMCAQAwHQYDVR0O
// SIG // BBYEFOb8X3u7IgBY5HJOtfQhdCMy5u+sMBkGCSsGAQQB
// SIG // gjcUAgQMHgoAUwB1AGIAQwBBMAsGA1UdDwQEAwIBhjAP
// SIG // BgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2VsuP
// SIG // 6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeG
// SIG // RWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
// SIG // L3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEwLTA2LTIz
// SIG // LmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKG
// SIG // Pmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2Vy
// SIG // dHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3J0MIGd
// SIG // BgNVHSAEgZUwgZIwgY8GCSsGAQQBgjcuAzCBgTA9Bggr
// SIG // BgEFBQcCARYxaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
// SIG // L1BLSS9kb2NzL0NQUy9kZWZhdWx0Lmh0bTBABggrBgEF
// SIG // BQcCAjA0HjIgHQBMAGUAZwBhAGwAXwBQAG8AbABpAGMA
// SIG // eQBfAFMAdABhAHQAZQBtAGUAbgB0AC4gHTANBgkqhkiG
// SIG // 9w0BAQsFAAOCAgEAGnTvV08pe8QWhXi4UNMi/AmdrIKX
// SIG // +DT/KiyXlRLl5L/Pv5PI4zSp24G43B4AvtI1b6/lf3mV
// SIG // d+UC1PHr2M1OHhthosJaIxrwjKhiUUVnCOM/PB6T+DCF
// SIG // F8g5QKbXDrMhKeWloWmMIpPMdJjnoUdD8lOswA8waX/+
// SIG // 0iUgbW9h098H1dlyACxphnY9UdumOUjJN2FtB91TGcun
// SIG // 1mHCv+KDqw/ga5uV1n0oUbCJSlGkmmzItx9KGg5pqdfc
// SIG // wX7RSXCqtq27ckdjF/qm1qKmhuyoEESbY7ayaYkGx0aG
// SIG // ehg/6MUdIdV7+QIjLcVBy78dTMgW77Gcf/wiS0mKbhXj
// SIG // pn92W9FTeZGFndXS2z1zNfM8rlSyUkdqwKoTldKOEdqZ
// SIG // Z14yjPs3hdHcdYWch8ZaV4XCv90Nj4ybLeu07s8n07Ve
// SIG // afqkFgQBpyRnc89NT7beBVaXevfpUk30dwVPhcbYC/GO
// SIG // 7UIJ0Q124yNWeCImNr7KsYxuqh3khdpHM2KPpMmRM19x
// SIG // HkCvmGXJIuhCISWKHC1g2TeJQYkqFg/XYTyUaGBS79ZH
// SIG // maCAQO4VgXc+nOBTGBpQHTiVmx5mMxMnORd4hzbOTsNf
// SIG // svU9R1O24OXbC2E9KteSLM43Wj5AQjGkHxAIwlacvyRd
// SIG // UQKdannSF9PawZSOB3slcUSrBmrm1MbfI5qWdcUxghYt
// SIG // MIIWKQIBATCBlTB+MQswCQYDVQQGEwJVUzETMBEGA1UE
// SIG // CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEe
// SIG // MBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgw
// SIG // JgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBD
// SIG // QSAyMDEwAhMzAAACznyazn2QXtK3AAAAAALOMA0GCWCG
// SIG // SAFlAwQCAQUAoIGuMBkGCSqGSIb3DQEJAzEMBgorBgEE
// SIG // AYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3
// SIG // AgEVMC8GCSqGSIb3DQEJBDEiBCCrA7YweOwRwEN56nrZ
// SIG // s4RsB0T0JH7KVhnsGFCLiwYkoTBCBgorBgEEAYI3AgEM
// SIG // MTQwMqAUgBIATQBpAGMAcgBvAHMAbwBmAHShGoAYaHR0
// SIG // cDovL3d3dy5taWNyb3NvZnQuY29tMA0GCSqGSIb3DQEB
// SIG // AQUABIIBAI1d2kfJ0CnmFRyMAECRMo/o4Dhca/TAr4eC
// SIG // oEPccWNkurjC/c4hQHWtFL23pLl6Wx4m5nX7rlyJ5+7p
// SIG // DFLGmN9kOSWMbeLZuWTDxQWIuS5FVBQPuajtvwExPojg
// SIG // b5c3Xlv7HUht+eZCR+60z4F3vNVXsctxzudmIicZFhKT
// SIG // sxPEtT387yGkVHn0QYG0B8037VKwEWIE+2miAxA9jXqF
// SIG // YaANXIuMUPyP4VQ3qdowkd6N1n6FmkBE3v0goB25Eue/
// SIG // zL2x03AmqkNZUO0OpeSTgF7Av6i3D2I50ICKe4zqMEHM
// SIG // EgD2w3Kf+xQ7ENrTInkhLQjduY2MeDVIXW1uSdQu6zWh
// SIG // ghO3MIITswYKKwYBBAGCNwMDATGCE6MwghOfBgkqhkiG
// SIG // 9w0BBwKgghOQMIITjAIBAzEPMA0GCWCGSAFlAwQCAQUA
// SIG // MIIBWAYLKoZIhvcNAQkQAQSgggFHBIIBQzCCAT8CAQEG
// SIG // CisGAQQBhFkKAwEwMTANBglghkgBZQMEAgEFAAQgrqFV
// SIG // r6iq/9t1w9dO49ZA4oUTYm2vHm2ibyfbKm82ta4CBl0W
// SIG // Jo0kQBgTMjAxOTA3MDEyMTEyNTQuNDM1WjAHAgEBgAIB
// SIG // 9KCB1KSB0TCBzjELMAkGA1UEBhMCVVMxEzARBgNVBAgT
// SIG // Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAc
// SIG // BgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcG
// SIG // A1UECxMgTWljcm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRv
// SIG // IFJpY28xJjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOjU4
// SIG // NDctRjc2MS00RjcwMSUwIwYDVQQDExxNaWNyb3NvZnQg
// SIG // VGltZS1TdGFtcCBTZXJ2aWNloIIPHzCCBPUwggPdoAMC
// SIG // AQICEzMAAADUTxnD2ITLRWMAAAAAANQwDQYJKoZIhvcN
// SIG // AQELBQAwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldh
// SIG // c2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNV
// SIG // BAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UE
// SIG // AxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAw
// SIG // HhcNMTgwODIzMjAyNjQwWhcNMTkxMTIzMjAyNjQwWjCB
// SIG // zjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0
// SIG // b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1p
// SIG // Y3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMgTWlj
// SIG // cm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28xJjAk
// SIG // BgNVBAsTHVRoYWxlcyBUU1MgRVNOOjU4NDctRjc2MS00
// SIG // RjcwMSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFt
// SIG // cCBTZXJ2aWNlMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
// SIG // MIIBCgKCAQEAv0i7sMQTtT74OGpouWATfTY+WnmWenhQ
// SIG // yL9pJAFK/a7LZOF32Fmgf00gimwjJckiYY72FBQx0UqD
// SIG // b3aTsusoplv47VDf27/0klDF49EJ8gBIwhEWLYTHtFNR
// SIG // g98M5wOTzVKfuhjWXK1nzPsW5/Qx6NGoQjfKDhPjMEsR
// SIG // pEzYPH8v0ef742MSvrRI4ydNswRZRX0mcdrx2hRFmIRo
// SIG // Ku4m4jadyHxkwdYBWE4mA4V1vZfS0MxFHcjbCPaW1Ufu
// SIG // Hw4NnbHsDwRv8L0QOESupwbQA7wledA0Xj+fQchPb69P
// SIG // 0lk17eyRGKneUgB+fHAL0JNbY+qqMuwGh1R0Uda3yQEb
// SIG // JQIDAQABo4IBGzCCARcwHQYDVR0OBBYEFP5k/7/EzmU2
// SIG // uNamuNJbShVglJcHMB8GA1UdIwQYMBaAFNVjOlyKMZDz
// SIG // Q3t8RhvFM2hahW1VMFYGA1UdHwRPME0wS6BJoEeGRWh0
// SIG // dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3JsL3By
// SIG // b2R1Y3RzL01pY1RpbVN0YVBDQV8yMDEwLTA3LTAxLmNy
// SIG // bDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKGPmh0
// SIG // dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2VydHMv
// SIG // TWljVGltU3RhUENBXzIwMTAtMDctMDEuY3J0MAwGA1Ud
// SIG // EwEB/wQCMAAwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDQYJ
// SIG // KoZIhvcNAQELBQADggEBAEnI5BKJf8YlLqfOo0xeUA8d
// SIG // kzKYWUUpF1roTtBH5fmvsXLsVQx84IT8WKvAcqU2/2dn
// SIG // rMP/YFtz3qSrucNPZXIdjhEOWME9ELHrBKEWzHRmfm/D
// SIG // ZuqorEPnQfaNboPXbJuyfxWc6tJn3h18adwiTJW7AIde
// SIG // Gf2e+D1v2qehOcrAtOC2l+rNRzCULRvKRgTS6o77geVG
// SIG // 4V97yLKxLMF5ZdI89jN+q+8EHIsIB+ggteUYAGOX+WXZ
// SIG // xR4/Ib/odk3ze9AD2FR+X1WlF1EzMrg9QUm4Aszmkld5
// SIG // wiR9HCbn7ji5EsAA8H6Irp8csHrRmXUl2WeVvysXxNja
// SIG // kP23cgQvEn8wggZxMIIEWaADAgECAgphCYEqAAAAAAAC
// SIG // MA0GCSqGSIb3DQEBCwUAMIGIMQswCQYDVQQGEwJVUzET
// SIG // MBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVk
// SIG // bW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0
// SIG // aW9uMTIwMAYDVQQDEylNaWNyb3NvZnQgUm9vdCBDZXJ0
// SIG // aWZpY2F0ZSBBdXRob3JpdHkgMjAxMDAeFw0xMDA3MDEy
// SIG // MTM2NTVaFw0yNTA3MDEyMTQ2NTVaMHwxCzAJBgNVBAYT
// SIG // AlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQH
// SIG // EwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29y
// SIG // cG9yYXRpb24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1l
// SIG // LVN0YW1wIFBDQSAyMDEwMIIBIjANBgkqhkiG9w0BAQEF
// SIG // AAOCAQ8AMIIBCgKCAQEAqR0NvHcRijog7PwTl/X6f2mU
// SIG // a3RUENWlCgCChfvtfGhLLF/Fw+Vhwna3PmYrW/AVUycE
// SIG // MR9BGxqVHc4JE458YTBZsTBED/FgiIRUQwzXTbg4CLNC
// SIG // 3ZOs1nMwVyaCo0UN0Or1R4HNvyRgMlhgRvJYR4YyhB50
// SIG // YWeRX4FUsc+TTJLBxKZd0WETbijGGvmGgLvfYfxGwScd
// SIG // JGcSchohiq9LZIlQYrFd/XcfPfBXday9ikJNQFHRD5wG
// SIG // Pmd/9WbAA5ZEfu/QS/1u5ZrKsajyeioKMfDaTgaRtogI
// SIG // Neh4HLDpmc085y9Euqf03GS9pAHBIAmTeM38vMDJRF1e
// SIG // FpwBBU8iTQIDAQABo4IB5jCCAeIwEAYJKwYBBAGCNxUB
// SIG // BAMCAQAwHQYDVR0OBBYEFNVjOlyKMZDzQ3t8RhvFM2ha
// SIG // hW1VMBkGCSsGAQQBgjcUAgQMHgoAUwB1AGIAQwBBMAsG
// SIG // A1UdDwQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB8GA1Ud
// SIG // IwQYMBaAFNX2VsuP6KJcYmjRPZSQW9fOmhjEMFYGA1Ud
// SIG // HwRPME0wS6BJoEeGRWh0dHA6Ly9jcmwubWljcm9zb2Z0
// SIG // LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL01pY1Jvb0NlckF1
// SIG // dF8yMDEwLTA2LTIzLmNybDBaBggrBgEFBQcBAQROMEww
// SIG // SgYIKwYBBQUHMAKGPmh0dHA6Ly93d3cubWljcm9zb2Z0
// SIG // LmNvbS9wa2kvY2VydHMvTWljUm9vQ2VyQXV0XzIwMTAt
// SIG // MDYtMjMuY3J0MIGgBgNVHSABAf8EgZUwgZIwgY8GCSsG
// SIG // AQQBgjcuAzCBgTA9BggrBgEFBQcCARYxaHR0cDovL3d3
// SIG // dy5taWNyb3NvZnQuY29tL1BLSS9kb2NzL0NQUy9kZWZh
// SIG // dWx0Lmh0bTBABggrBgEFBQcCAjA0HjIgHQBMAGUAZwBh
// SIG // AGwAXwBQAG8AbABpAGMAeQBfAFMAdABhAHQAZQBtAGUA
// SIG // bgB0AC4gHTANBgkqhkiG9w0BAQsFAAOCAgEAB+aIUQ3i
// SIG // xuCYP4FxAz2do6Ehb7Prpsz1Mb7PBeKp/vpXbRkws8LF
// SIG // Zslq3/Xn8Hi9x6ieJeP5vO1rVFcIK1GCRBL7uVOMzPRg
// SIG // Eop2zEBAQZvcXBf/XPleFzWYJFZLdO9CEMivv3/Gf/I3
// SIG // fVo/HPKZeUqRUgCvOA8X9S95gWXZqbVr5MfO9sp6AG9L
// SIG // MEQkIjzP7QOllo9ZKby2/QThcJ8ySif9Va8v/rbljjO7
// SIG // Yl+a21dA6fHOmWaQjP9qYn/dxUoLkSbiOewZSnFjnXsh
// SIG // bcOco6I8+n99lmqQeKZt0uGc+R38ONiU9MalCpaGpL2e
// SIG // Gq4EQoO4tYCbIjggtSXlZOz39L9+Y1klD3ouOVd2onGq
// SIG // BooPiRa6YacRy5rYDkeagMXQzafQ732D8OE7cQnfXXSY
// SIG // Ighh2rBQHm+98eEA3+cxB6STOvdlR3jo+KhIq/fecn5h
// SIG // a293qYHLpwmsObvsxsvYgrRyzR30uIUBHoD7G4kqVDmy
// SIG // W9rIDVWZeodzOwjmmC3qjeAzLhIp9cAvVCch98isTtoo
// SIG // uLGp25ayp0Kiyc8ZQU3ghvkqmqMRZjDTu3QyS99je/WZ
// SIG // ii8bxyGvWbWu3EQ8l1Bx16HSxVXjad5XwdHeMMD9zOZN
// SIG // +w2/XU/pnR4ZOC+8z1gFLu8NoFA12u8JJxzVs341Hgi6
// SIG // 2jbb01+P3nSISRKhggOtMIIClQIBATCB/qGB1KSB0TCB
// SIG // zjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0
// SIG // b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1p
// SIG // Y3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMgTWlj
// SIG // cm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28xJjAk
// SIG // BgNVBAsTHVRoYWxlcyBUU1MgRVNOOjU4NDctRjc2MS00
// SIG // RjcwMSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFt
// SIG // cCBTZXJ2aWNloiUKAQEwCQYFKw4DAhoFAAMVAO0ICv9f
// SIG // 7xxmgMPCjIj5XADfKo9DoIHeMIHbpIHYMIHVMQswCQYD
// SIG // VQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4G
// SIG // A1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
// SIG // IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQg
// SIG // T3BlcmF0aW9ucyBQdWVydG8gUmljbzEnMCUGA1UECxMe
// SIG // bkNpcGhlciBOVFMgRVNOOjRERTktMEM1RS0zRTA5MSsw
// SIG // KQYDVQQDEyJNaWNyb3NvZnQgVGltZSBTb3VyY2UgTWFz
// SIG // dGVyIENsb2NrMA0GCSqGSIb3DQEBBQUAAgUA4MQRFjAi
// SIG // GA8yMDE5MDcwMTEyNTY1NFoYDzIwMTkwNzAyMTI1NjU0
// SIG // WjB0MDoGCisGAQQBhFkKBAExLDAqMAoCBQDgxBEWAgEA
// SIG // MAcCAQACAgO0MAcCAQACAhnjMAoCBQDgxWKWAgEAMDYG
// SIG // CisGAQQBhFkKBAIxKDAmMAwGCisGAQQBhFkKAwGgCjAI
// SIG // AgEAAgMW42ChCjAIAgEAAgMHoSAwDQYJKoZIhvcNAQEF
// SIG // BQADggEBAIv/F6Td2a3/FyBtCTMs+tMJOMAs/IU9G7VD
// SIG // DCGetUAFKeu77w1CO6NpfR0ZPL8x4QNy6ABeKMArCBQW
// SIG // 2yIXJ6UlO9N3GwVawH0cNLzhDFx4v2f9pVRV1pMNyfPl
// SIG // D8/vYTCLthavLHse2rrfrWGQVMiKrrrQbCXiaeDz1qvU
// SIG // qBYUK8RDXiQTUuy+XhrvcUGJCbGh+p/F0eNamclzddKr
// SIG // YL7XY8SXHcWeSUan4h/mHYl22YY2goAwHHoeqOwrVGri
// SIG // Q6am7ZfDkq1ofpp2ohVPsIdf/Rwg8XwYJDNMHe0HeVUd
// SIG // ttvDIK8/Nkqy2wAOjdjhIKrM8jyQPdP01ZZjLnCPgNQx
// SIG // ggL1MIIC8QIBATCBkzB8MQswCQYDVQQGEwJVUzETMBEG
// SIG // A1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
// SIG // ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MSYwJAYDVQQDEx1NaWNyb3NvZnQgVGltZS1TdGFtcCBQ
// SIG // Q0EgMjAxMAITMwAAANRPGcPYhMtFYwAAAAAA1DANBglg
// SIG // hkgBZQMEAgEFAKCCATIwGgYJKoZIhvcNAQkDMQ0GCyqG
// SIG // SIb3DQEJEAEEMC8GCSqGSIb3DQEJBDEiBCCEmxnKdi1D
// SIG // VxxPqAN0Nr+nnsXvUUUtsuR9pSF38hkRtTCB4gYLKoZI
// SIG // hvcNAQkQAgwxgdIwgc8wgcwwgbEEFO0ICv9f7xxmgMPC
// SIG // jIj5XADfKo9DMIGYMIGApH4wfDELMAkGA1UEBhMCVVMx
// SIG // EzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1Jl
// SIG // ZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3Jh
// SIG // dGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3Rh
// SIG // bXAgUENBIDIwMTACEzMAAADUTxnD2ITLRWMAAAAAANQw
// SIG // FgQUhPIorMqr0/MceQxGbyYQdSQKY0YwDQYJKoZIhvcN
// SIG // AQELBQAEggEADUAszwBg5GYaoisUit3w3LErqihPbDUm
// SIG // R3VpBPhmtNm8umHXWEH0qcSDYuT/FW/fPFETbo6vw0hK
// SIG // zV457mq4rOC2vvv0j4/oowbhUWScS2krEDXk5xgRG+f7
// SIG // +FOGRf2OnKEZBGWyJSZ2rKvqRVQjHZHc/rhE0flfEfbl
// SIG // MaNZ4I6zO515egHQtsZ272o8Lym7MkYACGgju573vHVm
// SIG // czBYup/dDuIIFZn37U6wooBLFCAiT2wnXVSq4xDGLmoy
// SIG // zdSg42fwNoANPrUlkrCeo2Grif3FmtQHnZqyLzxd2j2a
// SIG // iGxGq21GXfDt10w6WeMxXD0UDi8EKKsjlX7jSpklZDO5fA==
// SIG // End signature block
