"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var path = require('path');
var fs = require('fs');
var q_1 = require('q');
var logger_1 = require('./logger');
var connect_1 = require('./connect');
var hubLookup_1 = require('./hubLookup');
var config_1 = require('./config');
var common_1 = require('./common');
var libraryResourceStrings_1 = require('./libraryResourceStrings');
var ec = require('errno-codes');
var requestServiceMaxRetries = 50;
var requestServiceRetryDelayMs = 100;
var HubClient = (function () {
    function HubClient(clientName, logger, serviceHubConfigFilePath) {
        var _this = this;
        this.clientName = clientName || 'client';
        this._logger = logger || new logger_1.Logger(this.clientName);
        if (!serviceHubConfigFilePath) {
            this.hubLookup = new hubLookup_1.HubLookup();
        }
        else {
            if (!path.isAbsolute(serviceHubConfigFilePath)) {
                throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.serviceHubConfigFilePathIsNotAbsolute(serviceHubConfigFilePath));
            }
            if (path.basename(serviceHubConfigFilePath) !== config_1.serviceHubConfigFileName) {
                throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.serviceHubConfigFileNameIsIncorrect(serviceHubConfigFilePath));
            }
            if (!fs.existsSync(serviceHubConfigFilePath)) {
                throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.serviceHubConfigFileDoesNotExist(serviceHubConfigFilePath));
            }
            this.hubLookup = new hubLookup_1.HubLookup({
                getConfig: function () { return config_1.loadConfig(path.dirname(serviceHubConfigFilePath), _this._logger); }
            });
        }
        this.locationServiceFactory = this.hubLookup.getLocationService.bind(this.hubLookup);
    }
    Object.defineProperty(HubClient.prototype, "logger", {
        get: function () {
            return this._logger;
        },
        enumerable: true,
        configurable: true
    });
    ;
    HubClient.prototype.getSessionId = function () {
        this.throwIfDisposed();
        return this.locationServiceFactory()
            .then(function (ls) { return ls.getSessionId(); });
    };
    HubClient.prototype.requestService = function (service) {
        var _this = this;
        if (!service) {
            throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('service'));
        }
        this.throwIfDisposed();
        var serviceDescriptor = typeof service === 'string' ? { name: service } : service;
        return this.requestServiceWithRetries(serviceDescriptor, requestServiceMaxRetries)
            .then(function (serviceConnection) {
            _this.startClientWatch(serviceConnection.clientWatch);
            return serviceConnection.stream;
        });
    };
    HubClient.prototype.disposeAsync = function () {
        this.disposed = true;
        return this.hubLookup.disposeAsync();
    };
    HubClient.prototype.requestServiceWithRetries = function (serviceDescriptor, retries) {
        var _this = this;
        var loc = null;
        return this.locationServiceFactory()
            .then(function (ls) { return ls.locate(serviceDescriptor, { clientName: _this.clientName, pid: process.pid }); })
            .then(function (location) {
            loc = location;
            return connect_1.connect(location.location)
                .then(function (stream) { return { stream: stream, clientWatch: location.clientWatch }; });
        })
            .catch(function (err) {
            // If the service host has shut down when we're requesting a service there,
            // request the service again.
            if (err.code === ec.ENOENT.code && retries > 1) {
                // Delay the next attempt to give host enough time to start the service.
                var attemptNum = requestServiceMaxRetries - retries;
                _this.logger.error("Retrying attempted connection to service '" + _this.clientName + "' - attempt '" + attemptNum + "'");
                return q_1.delay(requestServiceRetryDelayMs)
                    .then(function () { return _this.requestServiceWithRetries(serviceDescriptor, retries - 1); });
            }
            if (loc !== null) {
                _this.logger.error("Attempted connection to service '" + _this.clientName + "' at '" + loc.location + "'");
            }
            else {
                _this.logger.error("Attempted connection to service '" + _this.clientName + "': location is null");
            }
            throw err;
        });
    };
    HubClient.prototype.throwIfDisposed = function () {
        if (this.disposed) {
            throw new common_1.ServiceHubError(common_1.ErrorKind.ObjectDisposed);
        }
    };
    HubClient.prototype.startClientWatch = function (clientWatch) {
        var _this = this;
        if (clientWatch && !HubClient.clientWatchStreams[clientWatch]) {
            HubClient.clientWatchStreams[clientWatch] = connect_1.connectWithRetries({ url: clientWatch, timeoutMs: 1000 })
                .then(function (stream) {
                _this.logger.info("Connected to Hub Controller's client watch '" + clientWatch + "'");
                stream.once('end', function () { delete HubClient.clientWatchStreams[clientWatch]; });
                return stream;
            }, function (err) {
                _this.logger.error("Error connecting to Hub Controller's client watch '" + clientWatch + "': " + err.message);
                delete HubClient.clientWatchStreams[clientWatch];
                return null;
            });
        }
    };
    HubClient.clientWatchStreams = {};
    return HubClient;
}());
exports.HubClient = HubClient;
var HubClientInternal = (function (_super) {
    __extends(HubClientInternal, _super);
    function HubClientInternal(clientName, logger, locationServiceFactory) {
        _super.call(this, clientName, logger);
        this.locationServiceFactory = locationServiceFactory;
    }
    return HubClientInternal;
}(HubClient));
exports.HubClientInternal = HubClientInternal;
var HubClientInternal2 = (function (_super) {
    __extends(HubClientInternal2, _super);
    function HubClientInternal2(clientName, logger, serviceHubConfigFilePath) {
        _super.call(this, clientName, logger, serviceHubConfigFilePath);
    }
    HubClientInternal2.prototype.getConfig = function () {
        return this.hubLookup.getConfig();
    };
    HubClientInternal2.prototype.stopHubController = function () {
        return this.hubLookup.stopHubController();
    };
    HubClientInternal2.prototype.exit = function () {
        var _this = this;
        var exitCode;
        return this.stopHubController()
            .then(function (code) {
            exitCode = code;
            return _this.disposeAsync();
        })
            .catch(function (err) {
            if (err.code !== ec.ENOENT.code) {
                throw err;
            }
        })
            .then(function () { return exitCode; });
    };
    HubClientInternal2.prototype.startHeapDiff = function () {
        return this.locationServiceFactory()
            .then(function (ls) { return ls.startHeapDiff(); });
    };
    HubClientInternal2.prototype.endHeapDiff = function () {
        return this.locationServiceFactory()
            .then(function (ls) { return ls.endHeapDiff(); });
    };
    HubClientInternal2.prototype.runGC = function () {
        return this.locationServiceFactory()
            .then(function (ls) { return ls.runGC(); });
    };
    HubClientInternal2.prototype.captureHeapDump = function (filePath) {
        return this.locationServiceFactory()
            .then(function (ls) { return ls.captureHeapDump(filePath); });
    };
    HubClientInternal2.prototype.getHubControllerPid = function () {
        return this.locationServiceFactory()
            .then(function (ls) { return ls.getHubControllerPid(); });
    };
    return HubClientInternal2;
}(HubClient));
exports.HubClientInternal2 = HubClientInternal2;

//# sourceMappingURL=hubClient.js.map
// SIG // Begin signature block
// SIG // MIIiRAYJKoZIhvcNAQcCoIIiNTCCIjECAQExDzANBglg
// SIG // hkgBZQMEAgEFADB3BgorBgEEAYI3AgEEoGkwZzAyBgor
// SIG // BgEEAYI3AgEeMCQCAQEEEBDgyQbOONQRoqMAEEvTUJAC
// SIG // AQACAQACAQACAQACAQAwMTANBglghkgBZQMEAgEFAAQg
// SIG // grR88NqOtZ6djZSusvRWquTW34Vn4c5ulCbUXNJuVVSg
// SIG // ggtyMIIE+jCCA+KgAwIBAgITMwAAAs58ms59kF7StwAA
// SIG // AAACzjANBgkqhkiG9w0BAQsFADB+MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBT
// SIG // aWduaW5nIFBDQSAyMDEwMB4XDTE5MDUwMjIxMjU0MloX
// SIG // DTIwMDUwMjIxMjU0MlowdDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEeMBwGA1UEAxMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
// SIG // owhbo0atJEL/giHpwS2stH0eY6h2vghqYkLwa9LT5G5k
// SIG // q5QvKepTYRPV5tH0f5xTRTFTuZopGubS+mDvUkm5CF3C
// SIG // 0w4hucK3Mr8ad7l2yeEmZ7Tikgro6rzUApaNX3pEmKmt
// SIG // 2cPaAJ5XnM4DgC6Q5VV6gX3C/NO+h7ss51xKRb662Hjp
// SIG // y4s1p1weopn+gyWVpQwGNgGwTAcSguLN7XpFUl1esVAC
// SIG // GzvS4MIb8qsl1+aXWvgNUMmzSAwlIHDRnmd6HejS5HEM
// SIG // m4Ye2bYmRKb6Bc7I4hsi3pIm304LK4vSAdLH2HElS7VY
// SIG // mofTuiU2MZUNWaz+14kv5R4yRn+ojSNQUQIDAQABo4IB
// SIG // eTCCAXUwHwYDVR0lBBgwFgYKKwYBBAGCNz0GAQYIKwYB
// SIG // BQUHAwMwHQYDVR0OBBYEFHM/DO7M0NXkSmSFJIwE5oKt
// SIG // gMgPMFAGA1UdEQRJMEekRTBDMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEWMBQG
// SIG // A1UEBRMNMjMwODY1KzQ1NDI0MzAfBgNVHSMEGDAWgBTm
// SIG // /F97uyIAWORyTrX0IXQjMubvrDBWBgNVHR8ETzBNMEug
// SIG // SaBHhkVodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NybC9wcm9kdWN0cy9NaWNDb2RTaWdQQ0FfMjAxMC0w
// SIG // Ny0wNi5jcmwwWgYIKwYBBQUHAQEETjBMMEoGCCsGAQUF
// SIG // BzAChj5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NlcnRzL01pY0NvZFNpZ1BDQV8yMDEwLTA3LTA2LmNy
// SIG // dDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IB
// SIG // AQAcPuovja14/IUbUPK1RjGlfqowbC45KppnIgLxztfM
// SIG // +gbG8nz62UhMR8MjrPM2gyl+dwxqSm5kmAqQ0zeU/lTA
// SIG // z3wH6dvaa8gWwR5gCRYCu0z/s1sR7pjXlQpEILf6QP3t
// SIG // PTg+CEhYbIYxUH+Hr8SQisaxZWCC/rTywdBanMHIMQ7L
// SIG // LZbYaEjmkXmjSzIq4wGjaRRvoJ1cQUXLe0BHgaycOCgk
// SIG // TtJMON5OUqZB3C4rqcjlQmp8/THwHeSGwZgVPYQJCNZi
// SIG // j3V24F1nEKDwiazXleVkJ6smBpa4wBYbPFGR/VzTW49o
// SIG // gr51TH3e4IENba6DsZIhnrk2YP7s/xvsbKdGMIIGcDCC
// SIG // BFigAwIBAgIKYQxSTAAAAAAAAzANBgkqhkiG9w0BAQsF
// SIG // ADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMp
// SIG // TWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9y
// SIG // aXR5IDIwMTAwHhcNMTAwNzA2MjA0MDE3WhcNMjUwNzA2
// SIG // MjA1MDE3WjB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMK
// SIG // V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
// SIG // A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYD
// SIG // VQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBDQSAy
// SIG // MDEwMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
// SIG // AQEA6Q5kUHlntcTj/QkATJ6UrPdWaOpE2M/FWE+ppXZ8
// SIG // bUW60zmStKQe+fllguQX0o/9RJwI6GWTzixVhL99COMu
// SIG // K6hBKxi3oktuSUxrFQfe0dLCiR5xlM21f0u0rwjYzIjW
// SIG // axeUOpPOJj/s5v40mFfVHV1J9rIqLtWFu1k/+JC0K4N0
// SIG // yiuzO0bj8EZJwRdmVMkcvR3EVWJXcvhnuSUgNN5dpqWV
// SIG // XqsogM3Vsp7lA7Vj07IUyMHIiiYKWX8H7P8O7YASNUwS
// SIG // pr5SW/Wm2uCLC0h31oVH1RC5xuiq7otqLQVcYMa0Kluc
// SIG // IxxfReMaFB5vN8sZM4BqiU2jamZjeJPVMM+VHwIDAQAB
// SIG // o4IB4zCCAd8wEAYJKwYBBAGCNxUBBAMCAQAwHQYDVR0O
// SIG // BBYEFOb8X3u7IgBY5HJOtfQhdCMy5u+sMBkGCSsGAQQB
// SIG // gjcUAgQMHgoAUwB1AGIAQwBBMAsGA1UdDwQEAwIBhjAP
// SIG // BgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2VsuP
// SIG // 6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeG
// SIG // RWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
// SIG // L3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEwLTA2LTIz
// SIG // LmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKG
// SIG // Pmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2Vy
// SIG // dHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3J0MIGd
// SIG // BgNVHSAEgZUwgZIwgY8GCSsGAQQBgjcuAzCBgTA9Bggr
// SIG // BgEFBQcCARYxaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
// SIG // L1BLSS9kb2NzL0NQUy9kZWZhdWx0Lmh0bTBABggrBgEF
// SIG // BQcCAjA0HjIgHQBMAGUAZwBhAGwAXwBQAG8AbABpAGMA
// SIG // eQBfAFMAdABhAHQAZQBtAGUAbgB0AC4gHTANBgkqhkiG
// SIG // 9w0BAQsFAAOCAgEAGnTvV08pe8QWhXi4UNMi/AmdrIKX
// SIG // +DT/KiyXlRLl5L/Pv5PI4zSp24G43B4AvtI1b6/lf3mV
// SIG // d+UC1PHr2M1OHhthosJaIxrwjKhiUUVnCOM/PB6T+DCF
// SIG // F8g5QKbXDrMhKeWloWmMIpPMdJjnoUdD8lOswA8waX/+
// SIG // 0iUgbW9h098H1dlyACxphnY9UdumOUjJN2FtB91TGcun
// SIG // 1mHCv+KDqw/ga5uV1n0oUbCJSlGkmmzItx9KGg5pqdfc
// SIG // wX7RSXCqtq27ckdjF/qm1qKmhuyoEESbY7ayaYkGx0aG
// SIG // ehg/6MUdIdV7+QIjLcVBy78dTMgW77Gcf/wiS0mKbhXj
// SIG // pn92W9FTeZGFndXS2z1zNfM8rlSyUkdqwKoTldKOEdqZ
// SIG // Z14yjPs3hdHcdYWch8ZaV4XCv90Nj4ybLeu07s8n07Ve
// SIG // afqkFgQBpyRnc89NT7beBVaXevfpUk30dwVPhcbYC/GO
// SIG // 7UIJ0Q124yNWeCImNr7KsYxuqh3khdpHM2KPpMmRM19x
// SIG // HkCvmGXJIuhCISWKHC1g2TeJQYkqFg/XYTyUaGBS79ZH
// SIG // maCAQO4VgXc+nOBTGBpQHTiVmx5mMxMnORd4hzbOTsNf
// SIG // svU9R1O24OXbC2E9KteSLM43Wj5AQjGkHxAIwlacvyRd
// SIG // UQKdannSF9PawZSOB3slcUSrBmrm1MbfI5qWdcUxghYq
// SIG // MIIWJgIBATCBlTB+MQswCQYDVQQGEwJVUzETMBEGA1UE
// SIG // CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEe
// SIG // MBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgw
// SIG // JgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBD
// SIG // QSAyMDEwAhMzAAACznyazn2QXtK3AAAAAALOMA0GCWCG
// SIG // SAFlAwQCAQUAoIGuMBkGCSqGSIb3DQEJAzEMBgorBgEE
// SIG // AYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3
// SIG // AgEVMC8GCSqGSIb3DQEJBDEiBCBBiWAobYgkzFty3Lpe
// SIG // PA/3KrmhUhjiJTpuQZPvHPisrzBCBgorBgEEAYI3AgEM
// SIG // MTQwMqAUgBIATQBpAGMAcgBvAHMAbwBmAHShGoAYaHR0
// SIG // cDovL3d3dy5taWNyb3NvZnQuY29tMA0GCSqGSIb3DQEB
// SIG // AQUABIIBAAva3jJkL/9pXJ3x9Cyjv4GokOyq1D4189zv
// SIG // c4Noiwbgau1jvZ9zIuBezC4XJcZFa6kIjzuT6STSWQ6h
// SIG // V6vRwAOZpgKhaoDGMKmguWghDMsVjKgABoDzOVN0+Lyo
// SIG // wTVOIFvDwV0TZMKh1Istq60WjA947J/gdpqXKGVtg/L3
// SIG // p6DvubH0tX/UPcSwULXTr6GRWvIKQYUxEbwXJxDqK+u3
// SIG // Jfkg68mSmdRUFVDhM25npGivWKbVuhD0sMGhnYUgVhXD
// SIG // vBi5jYfIYs9xUbyN/jnCUuZxHmPqqwZATRZifkz+ScNH
// SIG // iB9qQsq6V3Q0ZIY1phlA9mwvImYBRGXh2OhV9sgokJah
// SIG // ghO0MIITsAYKKwYBBAGCNwMDATGCE6AwghOcBgkqhkiG
// SIG // 9w0BBwKgghONMIITiQIBAzEPMA0GCWCGSAFlAwQCAQUA
// SIG // MIIBVQYLKoZIhvcNAQkQAQSgggFEBIIBQDCCATwCAQEG
// SIG // CisGAQQBhFkKAwEwMTANBglghkgBZQMEAgEFAAQgKYl1
// SIG // qeIOtM8i1f4dl4234uqJD+65Bvt17ScT0tQzktECBl0W
// SIG // SRQrFBgTMjAxOTA3MDEyMTE2MjYuOTg1WjAEgAIB9KCB
// SIG // 1KSB0TCBzjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldh
// SIG // c2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNV
// SIG // BAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UE
// SIG // CxMgTWljcm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJp
// SIG // Y28xJjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOjE0OEMt
// SIG // QzRCOS0yMDY2MSUwIwYDVQQDExxNaWNyb3NvZnQgVGlt
// SIG // ZS1TdGFtcCBTZXJ2aWNloIIPHzCCBPUwggPdoAMCAQIC
// SIG // EzMAAADVpyPleJVqCf8AAAAAANUwDQYJKoZIhvcNAQEL
// SIG // BQAwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMd
// SIG // TWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwHhcN
// SIG // MTgwODIzMjAyNjQ1WhcNMTkxMTIzMjAyNjQ1WjCBzjEL
// SIG // MAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24x
// SIG // EDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jv
// SIG // c29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMgTWljcm9z
// SIG // b2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28xJjAkBgNV
// SIG // BAsTHVRoYWxlcyBUU1MgRVNOOjE0OEMtQzRCOS0yMDY2
// SIG // MSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFtcCBT
// SIG // ZXJ2aWNlMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
// SIG // CgKCAQEAwhQJK64X8TSCMvzuifB27nQzUHBTFDC2tlYR
// SIG // BgKYsyEaE1H/d5DEEP2WpjgVjGFFnVvF+wiOztIq9wXS
// SIG // bVh+GArmP0F1m/O5552QRjwxVS7jXMHXx2f42Zp6XPm8
// SIG // XP0EurmEk8AR1mDdlH7h0/vX21Vck/+JWSyUtNMof9ip
// SIG // PxVyJ6Zc5RoKwr9F7vF0F7yoFKGw7BuAPvFVjCCZ6/P5
// SIG // tJndSxYKVxwEH2QyE0eGeIgqS4yjgE0nm177KnwoFXLO
// SIG // ie0qa06FLVco1IjLARYDNEYpm2U/foNUY7L1IZqPB5XR
// SIG // GMeSpkxQPW0SIhoOgGmJldpwc+d7UVAbBKMrg0yUSwID
// SIG // AQABo4IBGzCCARcwHQYDVR0OBBYEFCloozGUdkPZX10N
// SIG // m5/TsoT2YjeEMB8GA1UdIwQYMBaAFNVjOlyKMZDzQ3t8
// SIG // RhvFM2hahW1VMFYGA1UdHwRPME0wS6BJoEeGRWh0dHA6
// SIG // Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3JsL3Byb2R1
// SIG // Y3RzL01pY1RpbVN0YVBDQV8yMDEwLTA3LTAxLmNybDBa
// SIG // BggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKGPmh0dHA6
// SIG // Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2VydHMvTWlj
// SIG // VGltU3RhUENBXzIwMTAtMDctMDEuY3J0MAwGA1UdEwEB
// SIG // /wQCMAAwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDQYJKoZI
// SIG // hvcNAQELBQADggEBAHnZy8LpxNHAqy0O0ia/2megC/32
// SIG // wLbBAvLiuYHNYvaQj+yV4E1ejvhyp0Bahh8tEJCj4O5b
// SIG // abn/SYx7gOm88MtgJBJImQC27+FQvzTCKTlmpo28h+nu
// SIG // HX8In8oy4iB3xZuE1nEvIblLhbsCzrrtA2J/B4KuMWpk
// SIG // xX1e0E7bCPP1C2LVGN04iaFftXiR0JgpK11CDbEmFRT1
// SIG // m0+oamu2Fo+q2j0WF6qzfsLDa87e0CNioxdNkK/JvP7h
// SIG // tQfEha+q8khcum48/PXMK8Ki0t8HPMGJX2aIgQFeye/a
// SIG // ogiR0OchKqWshnWdd1Ed1eXhmEF/fABlH8TAgu8pqSwo
// SIG // VjOuTgswggZxMIIEWaADAgECAgphCYEqAAAAAAACMA0G
// SIG // CSqGSIb3DQEBCwUAMIGIMQswCQYDVQQGEwJVUzETMBEG
// SIG // A1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
// SIG // ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MTIwMAYDVQQDEylNaWNyb3NvZnQgUm9vdCBDZXJ0aWZp
// SIG // Y2F0ZSBBdXRob3JpdHkgMjAxMDAeFw0xMDA3MDEyMTM2
// SIG // NTVaFw0yNTA3MDEyMTQ2NTVaMHwxCzAJBgNVBAYTAlVT
// SIG // MRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdS
// SIG // ZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9y
// SIG // YXRpb24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0
// SIG // YW1wIFBDQSAyMDEwMIIBIjANBgkqhkiG9w0BAQEFAAOC
// SIG // AQ8AMIIBCgKCAQEAqR0NvHcRijog7PwTl/X6f2mUa3RU
// SIG // ENWlCgCChfvtfGhLLF/Fw+Vhwna3PmYrW/AVUycEMR9B
// SIG // GxqVHc4JE458YTBZsTBED/FgiIRUQwzXTbg4CLNC3ZOs
// SIG // 1nMwVyaCo0UN0Or1R4HNvyRgMlhgRvJYR4YyhB50YWeR
// SIG // X4FUsc+TTJLBxKZd0WETbijGGvmGgLvfYfxGwScdJGcS
// SIG // chohiq9LZIlQYrFd/XcfPfBXday9ikJNQFHRD5wGPmd/
// SIG // 9WbAA5ZEfu/QS/1u5ZrKsajyeioKMfDaTgaRtogINeh4
// SIG // HLDpmc085y9Euqf03GS9pAHBIAmTeM38vMDJRF1eFpwB
// SIG // BU8iTQIDAQABo4IB5jCCAeIwEAYJKwYBBAGCNxUBBAMC
// SIG // AQAwHQYDVR0OBBYEFNVjOlyKMZDzQ3t8RhvFM2hahW1V
// SIG // MBkGCSsGAQQBgjcUAgQMHgoAUwB1AGIAQwBBMAsGA1Ud
// SIG // DwQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB8GA1UdIwQY
// SIG // MBaAFNX2VsuP6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRP
// SIG // ME0wS6BJoEeGRWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNv
// SIG // bS9wa2kvY3JsL3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8y
// SIG // MDEwLTA2LTIzLmNybDBaBggrBgEFBQcBAQROMEwwSgYI
// SIG // KwYBBQUHMAKGPmh0dHA6Ly93d3cubWljcm9zb2Z0LmNv
// SIG // bS9wa2kvY2VydHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYt
// SIG // MjMuY3J0MIGgBgNVHSABAf8EgZUwgZIwgY8GCSsGAQQB
// SIG // gjcuAzCBgTA9BggrBgEFBQcCARYxaHR0cDovL3d3dy5t
// SIG // aWNyb3NvZnQuY29tL1BLSS9kb2NzL0NQUy9kZWZhdWx0
// SIG // Lmh0bTBABggrBgEFBQcCAjA0HjIgHQBMAGUAZwBhAGwA
// SIG // XwBQAG8AbABpAGMAeQBfAFMAdABhAHQAZQBtAGUAbgB0
// SIG // AC4gHTANBgkqhkiG9w0BAQsFAAOCAgEAB+aIUQ3ixuCY
// SIG // P4FxAz2do6Ehb7Prpsz1Mb7PBeKp/vpXbRkws8LFZslq
// SIG // 3/Xn8Hi9x6ieJeP5vO1rVFcIK1GCRBL7uVOMzPRgEop2
// SIG // zEBAQZvcXBf/XPleFzWYJFZLdO9CEMivv3/Gf/I3fVo/
// SIG // HPKZeUqRUgCvOA8X9S95gWXZqbVr5MfO9sp6AG9LMEQk
// SIG // IjzP7QOllo9ZKby2/QThcJ8ySif9Va8v/rbljjO7Yl+a
// SIG // 21dA6fHOmWaQjP9qYn/dxUoLkSbiOewZSnFjnXshbcOc
// SIG // o6I8+n99lmqQeKZt0uGc+R38ONiU9MalCpaGpL2eGq4E
// SIG // QoO4tYCbIjggtSXlZOz39L9+Y1klD3ouOVd2onGqBooP
// SIG // iRa6YacRy5rYDkeagMXQzafQ732D8OE7cQnfXXSYIghh
// SIG // 2rBQHm+98eEA3+cxB6STOvdlR3jo+KhIq/fecn5ha293
// SIG // qYHLpwmsObvsxsvYgrRyzR30uIUBHoD7G4kqVDmyW9rI
// SIG // DVWZeodzOwjmmC3qjeAzLhIp9cAvVCch98isTtoouLGp
// SIG // 25ayp0Kiyc8ZQU3ghvkqmqMRZjDTu3QyS99je/WZii8b
// SIG // xyGvWbWu3EQ8l1Bx16HSxVXjad5XwdHeMMD9zOZN+w2/
// SIG // XU/pnR4ZOC+8z1gFLu8NoFA12u8JJxzVs341Hgi62jbb
// SIG // 01+P3nSISRKhggOtMIIClQIBATCB/qGB1KSB0TCBzjEL
// SIG // MAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24x
// SIG // EDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jv
// SIG // c29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMgTWljcm9z
// SIG // b2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28xJjAkBgNV
// SIG // BAsTHVRoYWxlcyBUU1MgRVNOOjE0OEMtQzRCOS0yMDY2
// SIG // MSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFtcCBT
// SIG // ZXJ2aWNloiUKAQEwCQYFKw4DAhoFAAMVAK3DJLzR2XXw
// SIG // XdDdRR85MZ5u1p6xoIHeMIHbpIHYMIHVMQswCQYDVQQG
// SIG // EwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UE
// SIG // BxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENv
// SIG // cnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQgT3Bl
// SIG // cmF0aW9ucyBQdWVydG8gUmljbzEnMCUGA1UECxMebkNp
// SIG // cGhlciBOVFMgRVNOOjRERTktMEM1RS0zRTA5MSswKQYD
// SIG // VQQDEyJNaWNyb3NvZnQgVGltZSBTb3VyY2UgTWFzdGVy
// SIG // IENsb2NrMA0GCSqGSIb3DQEBBQUAAgUA4MS5zDAiGA8y
// SIG // MDE5MDcwMjAwNTY0NFoYDzIwMTkwNzAzMDA1NjQ0WjB0
// SIG // MDoGCisGAQQBhFkKBAExLDAqMAoCBQDgxLnMAgEAMAcC
// SIG // AQACAgXfMAcCAQACAhhpMAoCBQDgxgtMAgEAMDYGCisG
// SIG // AQQBhFkKBAIxKDAmMAwGCisGAQQBhFkKAwGgCjAIAgEA
// SIG // AgMHoSChCjAIAgEAAgMHoSAwDQYJKoZIhvcNAQEFBQAD
// SIG // ggEBAFHSsPH8cgkNyyaV2yfWPWWJp9RRGWW5rlniktDx
// SIG // kfk95iWX90EuHFPGk8BP8nTbE9NdxnSicQwvdefNQ2CE
// SIG // RCeJwHA8JTc242OJm+1MnCUp4oUT3zCr/Eu7wOt8yueL
// SIG // 43m9xqArPRYC7XFsfpAb1YMV0DKgpGbMRI52UIYPhJvh
// SIG // W2g5OM0N28JZoz5MQESxTLUEjnm0Igpf7FtNcotEWc4G
// SIG // GtprqfAGIss5OhaH38qO5HkVAU5rqbP7O40+KF08kNym
// SIG // aU0sBF8HQ/neYwIRXsLXtCPsYwNUKvZOE/QZotP6n1wc
// SIG // iyJGic0maG1dT6BtEGHZ1D3f6aGGqp3I4JG+LOgxggL1
// SIG // MIIC8QIBATCBkzB8MQswCQYDVQQGEwJVUzETMBEGA1UE
// SIG // CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEe
// SIG // MBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYw
// SIG // JAYDVQQDEx1NaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0Eg
// SIG // MjAxMAITMwAAANWnI+V4lWoJ/wAAAAAA1TANBglghkgB
// SIG // ZQMEAgEFAKCCATIwGgYJKoZIhvcNAQkDMQ0GCyqGSIb3
// SIG // DQEJEAEEMC8GCSqGSIb3DQEJBDEiBCCtIP4pxXmsqonT
// SIG // bhX9e9eJiXKi4dTExDuIC6MR2O/mkTCB4gYLKoZIhvcN
// SIG // AQkQAgwxgdIwgc8wgcwwgbEEFK3DJLzR2XXwXdDdRR85
// SIG // MZ5u1p6xMIGYMIGApH4wfDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAg
// SIG // UENBIDIwMTACEzMAAADVpyPleJVqCf8AAAAAANUwFgQU
// SIG // rEir3XchsWfbaO27lINmtlfq9h8wDQYJKoZIhvcNAQEL
// SIG // BQAEggEAOP/Cut6wu43B2YmRLThnuVZr2/djhu4Z/vhi
// SIG // z1HPSJle2lxd18CS5vn/lE4vvR2FZh1T2g92DxmAFx6a
// SIG // tuNnFxpDru8qQ9ZA1x9zSaQKMOGy90IlerfNBqIp1adV
// SIG // 1AW7JBuUBZ8MojI8fZHtExAODAgIOwzsXC0HAtnQ7dh+
// SIG // YRu0BPo67XwPb+4mMVvTKt/0v1Ky2/yO351okF76uySl
// SIG // gimfS2KZNP/WYc4Cqw+Hwu7emRNUQwLFO80+lpiHctm+
// SIG // KVW9m4HAHpBzWc8xOCkM8niA7tDq/7c5VUIhLk92kF7c
// SIG // OjoUd6E1b4wYKp8ZhPePaVZC4luAV7h9PZgy0HfIng==
// SIG // End signature block
