"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var child_process_1 = require('child_process');
var q = require('q');
var path = require('path');
var ec = require('errno-codes');
var discoveryServiceProxy_1 = require('./discoveryServiceProxy');
var jsonRpc_1 = require('./jsonRpc');
var findFile_1 = require('./findFile');
var relativePath_1 = require('./relativePath');
var serviceDiscovery_1 = require('./serviceDiscovery');
var hostInfo_1 = require('./hostInfo');
var exitCode_1 = require('./exitCode');
var serviceDescriptor_1 = require('./serviceDescriptor');
var common_1 = require('./common');
var randomHexString_1 = require('./randomHexString');
var logger_1 = require('./logger');
var libraryResourceStrings_1 = require('./libraryResourceStrings');
var controllerEngineToken = '$CONTROLLERENGINE';
/** Debug host on startup environment variable. Must be in sync with EnvUtils.DebugHostOnStartupEnvVarName in src\clr\utility\Microsoft.ServiceHub.Utility.Shared\EnvUtils.cs */
var DebugHostOnStartupEnvVarName = 'SERVICEHUBDEBUGHOSTONSTARTUP';
var HostIdentity = (function () {
    function HostIdentity(host) {
        if (!host) {
            throw new Error(libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('host'));
        }
        if (!host.host) {
            throw new Error(libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('host.host'));
        }
        this.hostId = host.hostId;
        this.host = host.host;
        this.hostGroup = host.hostGroup;
    }
    HostIdentity.prototype.getId = function () {
        return this.host + (this.hostId ? '$' + this.hostId : '') + (this.hostGroup ? '$$' + this.hostGroup : '');
    };
    HostIdentity.prototype.getHostInfoFileGlobPattern = function () {
        // Glob allows searching for multiple files if {option1,option2} set notation is used.
        return this.hostId ? "{" + this.host + "." + this.hostId + "," + this.host + "}" : this.host;
    };
    HostIdentity.create = function (smi, serviceDescriptor) {
        var hostGroup = serviceDescriptor && serviceDescriptor.hasHostGroup() ? serviceDescriptor.hostGroup.id : null;
        return new HostIdentity({ hostId: smi.hostId, host: smi.host, hostGroup: hostGroup });
    };
    return HostIdentity;
}());
exports.HostIdentity = HostIdentity;
var HostInfo = (function (_super) {
    __extends(HostInfo, _super);
    function HostInfo(host, hostProcess) {
        _super.call(this, host);
        this.hostProcess = hostProcess;
        this.endPoint = q.defer();
        this.services = [];
        this.hasActiveServices = false;
        this.pendingStartServiceRequestCount = 0;
        this.touch();
    }
    HostInfo.prototype.setCooldownTimeout = function (callback, ms) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        this.clearCooldownTimeout();
        this.cooldownTimeout = global.setTimeout.apply(global, [callback, ms].concat(args));
    };
    HostInfo.prototype.clearCooldownTimeout = function () {
        if (this.cooldownTimeout) {
            global.clearTimeout(this.cooldownTimeout);
            this.cooldownTimeout = null;
        }
    };
    HostInfo.prototype.touch = function () {
        this.lastAccessTimeMs = new Date().getTime();
    };
    return HostInfo;
}(HostIdentity));
var LocationService = (function () {
    function LocationService(logger, config, options) {
        this.hosts = {};
        this.hostPids = [];
        this.locatedServices = {};
        this.hostInfoMap = {};
        this.externalClientCount = 0;
        if (!logger) {
            throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('logger'));
        }
        if (!config) {
            throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('config'));
        }
        options = options || {};
        this.hostCooldownMs = options.hostCooldownMs || 10 * 1000;
        this.fastShutdownHostCooldownMs = options.fastShutdownHostCooldownMs || 500;
        this.logger = logger;
        this.config = config;
        this.callbackPipeName = randomHexString_1.default().then(function (value) { return common_1.formatPipeName(value); });
        this.sessionId = randomHexString_1.default();
    }
    LocationService.prototype.locate = function (serviceDescriptor, clientInfo) {
        var _this = this;
        var service = new serviceDescriptor_1.ServiceDescriptor(serviceDescriptor);
        clientInfo = clientInfo || {};
        var clientId = "" + (clientInfo.clientName || 'client') + (clientInfo.pid ? " (pid: " + clientInfo.pid + ")" : '');
        var serviceKey = service.getServiceKey();
        this.logger.info(libraryResourceStrings_1.LibraryResourceStrings.locatingService(serviceKey, clientId));
        if (this.isExternalClient(clientInfo)) {
            this.stopFastShutdown();
        }
        return this.getHostedServiceLocation(service)
            .then(function (serviceLocation) {
            if (serviceLocation.host.shutdown) {
                // The host is shutting down; We need to locate the service again.
                return _this.locate(serviceDescriptor, clientInfo);
            }
            // Increase the host lifetime, to allow the client connect to the service,
            // and the host get back to us notifying about the started service.
            serviceLocation.host.touch();
            // Update the cached service location.
            // We don't cache IServiceLocation.clientWatch because it depends on whether the client is external or not.
            _this.locatedServices[serviceKey] = q.resolve(serviceLocation);
            var locationAndHost = serviceLocation.location + " @ " + serviceLocation.host.getId();
            _this.logger.info(libraryResourceStrings_1.LibraryResourceStrings.serviceLocated(serviceLocation.serviceId, locationAndHost, clientId));
            // Check if we want to watch for the client lifetime.
            return q.resolve(_this.isExternalClient(clientInfo) ? _this.getClientWatchServerName() : null)
                .then(function (clientWatch) { return { location: serviceLocation.location, serviceId: serviceLocation.serviceId, clientWatch: clientWatch }; });
        })
            .catch(function (reason) {
            _this.logger.error(libraryResourceStrings_1.LibraryResourceStrings.startingServiceFailed(serviceKey, clientId, "" + (reason.message || reason)));
            return q.reject(reason);
        });
    };
    LocationService.prototype.stop = function () {
        var _this = this;
        var hosts = common_1.getPropertiesArray(this.hosts).map(function (hostPromise) { return hostPromise.then(function (host) { return _this.stopHost(host, libraryResourceStrings_1.LibraryResourceStrings.forceControllerShutDown); }); });
        return q.all(hosts);
    };
    /**
    * Gets session ID
    *
    * Session ID is a unique string that stays the same during dev hub session (life time of hub controller)
    * and is different for different sessions.
    */
    LocationService.prototype.getSessionId = function () {
        return this.sessionId;
    };
    LocationService.prototype.getServiceModuleInfo = function (service) {
        var _this = this;
        var serviceDiscoveryConfig = {
            servicesFolderPath: this.config.services.rootDir,
            discoveryHelper: function (s) { return _this.discoverService(s); },
        };
        return serviceDiscovery_1.getServiceModuleInfo(this.logger, service, serviceDiscoveryConfig);
    };
    LocationService.prototype.discoverService = function (service, discoveryServices, discoveryServiceIndex) {
        var _this = this;
        if (!discoveryServices) {
            discoveryServices = this.config.services.discoveryServices;
            if (!discoveryServices || discoveryServices.length == 0 || discoveryServices.indexOf(service) >= 0) {
                return q.resolve(null);
            }
            return this.discoverService(service, discoveryServices, 0 /* discoveryServiceIndex */);
        }
        if (discoveryServices.length <= discoveryServiceIndex) {
            // We tried all known discovery services and couldn't find the service.
            return q.resolve(null);
        }
        var discoveryServiceName = discoveryServices[discoveryServiceIndex];
        var proxy = new discoveryServiceProxy_1.default(this.logger, this, discoveryServiceName);
        return proxy.discoverService(service)
            .then(function (filePath) { return filePath ? { filePath: filePath } : null; }, function (error) {
            _this.logger.error(libraryResourceStrings_1.LibraryResourceStrings.locatingServcieError(discoveryServiceName, service, error.message, error.stack));
            return { error: new Error(discoveryServiceName + ": " + error.message) };
        })
            .finally(function () {
            proxy.end()
                .catch(function (err) { return _this.logger.error(libraryResourceStrings_1.LibraryResourceStrings.disconnectingDiscoveryServiceFailed(discoveryServiceName, err.message, err.stack)); })
                .done();
        })
            .then(function (result) { return result || _this.discoverService(service, discoveryServices, discoveryServiceIndex + 1); });
    };
    /**
     * Find hub host info for the given host type by locating and parsing <host>.servicehub.host.json
     */
    LocationService.prototype.getHubHostInfo = function (hostIdentity) {
        var _this = this;
        if (!hostIdentity || !hostIdentity.host) {
            throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined(!hostIdentity ? 'hostIdentity' : 'hostIdentity.host'));
        }
        var hostId = hostIdentity.getId();
        if (this.hostInfoMap.hasOwnProperty(hostId)) {
            return this.hostInfoMap[hostId];
        }
        var globPattern = hostIdentity.getHostInfoFileGlobPattern();
        return this.hostInfoMap[hostId] = this.findHostConfigFile(globPattern)
            .then(function (filePath) {
            _this.logger.info(libraryResourceStrings_1.LibraryResourceStrings.loadingHostInfo(hostId, filePath));
            return common_1.parseFileAsJson(filePath)
                .then(function (result) {
                LocationService.verifyHostInfo(result, filePath);
                result.filePath = filePath;
                result.hostExecutable = result.hostExecutable.replace(controllerEngineToken, function () { return process.execPath; });
                return relativePath_1.expandDotRelativePaths(result, path.dirname(filePath), ['host', 'hostExecutable']);
            })
                .catch(function (err) {
                throw new common_1.ServiceHubError(common_1.ErrorKind.HubHostInfoLoadError, libraryResourceStrings_1.LibraryResourceStrings.cannotResolveHost(hostId, filePath, err.message));
            });
        });
    };
    LocationService.prototype.findHostConfigFile = function (fileNameGlobPattern) {
        var filePattern = '*/' + fileNameGlobPattern + '.servicehub.host.json';
        var localHostDir = path.join(__dirname, common_1.Constants.HostInfoFolderName);
        var configHostDir = this.config.hosts ? this.config.hosts.rootDir : null;
        var hubHostFile = configHostDir ? findFile_1.default(filePattern, { cwd: configHostDir, nodir: true, fileSelector: fileSelector, nosort: true }) : q.resolve(null);
        return hubHostFile
            .then(function (filePath) { return filePath || findFile_1.default(filePattern, { cwd: localHostDir, nodir: true, fileSelector: fileSelector, nosort: true }); })
            .then(function (filePath) {
            if (filePath) {
                return filePath;
            }
            var message = configHostDir ?
                libraryResourceStrings_1.LibraryResourceStrings.hostTypeNotFoundInLocation(fileNameGlobPattern, configHostDir, localHostDir) :
                libraryResourceStrings_1.LibraryResourceStrings.hostTypeNotFound(fileNameGlobPattern, localHostDir);
            throw new common_1.ServiceHubError(common_1.ErrorKind.HubHostInfoLoadError, message);
        });
        // There may be multiple config files found by the glob pattern.
        // The first one is the most specific, always pick it.
        function fileSelector(files) {
            return files[0];
        }
    };
    /**
     * A host must report that it has at least one active client
     * by invoking this method on the host callback pipe.
     */
    LocationService.prototype.hostServicesStarted = function (hostId) {
        if (!hostId) {
            throw new Error(libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('hostId'));
        }
        if (this.hosts.hasOwnProperty(hostId)) {
            this.hosts[hostId].then(function (h) { return h.hasActiveServices = true; });
        }
    };
    /**
     * A host must report that it has no more active clients
     * by invoking this method on the host callback pipe.
     */
    LocationService.prototype.hostServicesEnded = function (hostId) {
        var _this = this;
        if (!hostId) {
            throw new Error(libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('hostId'));
        }
        if (this.hosts.hasOwnProperty(hostId)) {
            this.hosts[hostId].then(function (host) { return _this.startHostCooldown(host); });
        }
    };
    /** A new host has connected to the location service */
    LocationService.prototype.onHostConnected = function () { };
    /** Last host has disconnected from the location service */
    LocationService.prototype.onLastHostDisconnected = function () { };
    ;
    /**
     * Verify hub host info.
     *
     * This method checks if the hub host info has mandatory fields but does not have reserved fields.
     * It throws if these conditions are not met.
     */
    LocationService.verifyHostInfo = function (hostInfo, path) {
        if (!hostInfo) {
            throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.hostInfoNotDefined(path));
        }
        if (!hostInfo.hostExecutable || typeof hostInfo.hostExecutable !== 'string'
            || !hostInfo.hostArgs || typeof hostInfo.hostArgs !== 'object' || !Array.isArray(hostInfo.hostArgs)) {
            throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.hostInfoInvalid(path));
        }
        if (hostInfo.filePath) {
            var message = libraryResourceStrings_1.LibraryResourceStrings.hostInfoContainsReservedProperty(path);
            throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, message);
        }
    };
    LocationService.isHostDebuggingOnStartupRequested = function () {
        return process.env[DebugHostOnStartupEnvVarName] ? true : false;
    };
    LocationService.isHostDebuggingOnStartupRequestedForHost = function (hostInfo) {
        var debugServiceHostOnStartupEnvVar = process.env[DebugHostOnStartupEnvVarName];
        if (!debugServiceHostOnStartupEnvVar) {
            return false;
        }
        debugServiceHostOnStartupEnvVar = debugServiceHostOnStartupEnvVar.toLowerCase();
        if (debugServiceHostOnStartupEnvVar === 'all' || debugServiceHostOnStartupEnvVar === path.basename(hostInfo.hostExecutable).toLowerCase()) {
            return true;
        }
        return false;
    };
    /**
     * Start service module process if it is not running yet.
     * @param: service - service module name, resolvable to a .json or .js file that either conforms to c.IServiceModuleInfo
     *                   or provides 'serviceModule' export that conforms to c.IServiceModuleInfo.
     */
    LocationService.prototype.getHostedServiceLocation = function (service) {
        var _this = this;
        var serviceKey = service.getServiceKey();
        // If somebody else has asked for the service before, use that promise
        if (this.locatedServices.hasOwnProperty(serviceKey)) {
            return this.locatedServices[serviceKey];
        }
        return this.getServiceModuleInfo(service.name).then(function (smi) {
            // If somebody else has already started the service, use it instead.
            if (_this.locatedServices.hasOwnProperty(serviceKey)) {
                return _this.locatedServices[serviceKey];
            }
            var result = _this.startService(smi, service);
            // Save the service promise so the next calls for the same service name can use that service.
            _this.locatedServices[serviceKey] = result;
            return result;
        });
    };
    LocationService.prototype.startService = function (smi, serviceDescriptor) {
        var _this = this;
        if (serviceDescriptor.hasHostGroup() && !smi.hostGroupAllowed) {
            var message = libraryResourceStrings_1.LibraryResourceStrings.HostGroupNotSupported(serviceDescriptor.name, serviceDescriptor.hostGroup.id, 'hostGroupAllowed');
            throw new common_1.ServiceHubError(common_1.ErrorKind.HostGroupsNotSupported, message);
        }
        var serviceKey = serviceDescriptor.getServiceKey();
        this.logger.info(libraryResourceStrings_1.LibraryResourceStrings.firstTimeLoadingService(serviceKey));
        var hostIdentity = HostIdentity.create(smi, serviceDescriptor);
        var hostId = hostIdentity.getId();
        var hostPromise;
        if (this.hosts.hasOwnProperty(hostId)) {
            hostPromise = this.hosts[hostId];
        }
        else {
            hostPromise = this.hosts[hostId] = this.startNewHost(hostIdentity);
        }
        // Sign up our continuation that we will run after the host has launched and is ready for a service request. The given channel is used to communicate
        // our launch request with the host.
        return hostPromise.then(function (host) {
            if (host.shutdown) {
                _this.logger.info("The host '" + host.getId() + "' for service '" + serviceDescriptor.name + "' is shutting down. We'll start a new host then.");
                return _this.startService(smi, serviceDescriptor);
            }
            // Track serivce running by the host.
            host.services.push(serviceKey);
            // Starting a new service may take some time.
            // Increase the pending request count to prevent the host from shutting down while the new service is starting.
            host.pendingStartServiceRequestCount++;
            return host.endPoint.promise
                .then(function (endPoint) { return endPoint.sendRequest(hostInfo_1.HostMethod.StartService, [smi]); })
                .then(function (location) {
                // Delay host shutdown after it started a new service to allow the client to connect to it.
                host.touch();
                return { location: location, host: host, serviceId: serviceDescriptor.name };
            })
                .finally(function () { return host.pendingStartServiceRequestCount--; });
        });
    };
    LocationService.prototype.startNewHost = function (hostIdentity) {
        var _this = this;
        var hostId = hostIdentity.getId();
        var hostCallbackPipeName = null;
        return this.getHostCallbackPipe()
            .then(function (pipeName) {
            hostCallbackPipeName = pipeName;
            return _this.getHubHostInfo(hostIdentity);
        })
            .then(function (hostInfo) {
            var hostDir = path.dirname(hostInfo.filePath);
            var debugHostOnStartup = LocationService.isHostDebuggingOnStartupRequestedForHost(hostInfo);
            // if NodeDebugArgs exists in the host args then if debugging is requested replace it with '--inspect-brk --nolazy' otherwise remove it
            var nodeDebugArgsIndex = hostInfo.hostArgs.indexOf(hostInfo_1.HostArgsToken.NodeDebugArgs);
            if (nodeDebugArgsIndex >= 0) {
                if (debugHostOnStartup) {
                    hostInfo.hostArgs.splice(nodeDebugArgsIndex, 1, '--inspect-brk', '--nolazy');
                }
                else {
                    hostInfo.hostArgs.splice(nodeDebugArgsIndex, 1);
                }
            }
            var args = []
                .concat(hostInfo.hostArgs.map(function (arg) { return arg
                .replace(hostInfo_1.HostArgsToken.HostId, function () { return hostId; })
                .replace(hostInfo_1.HostArgsToken.Pipe, function () { return hostCallbackPipeName; }); }));
            _this.logger.info(libraryResourceStrings_1.LibraryResourceStrings.launchingHostWithCmd(hostId, hostInfo.hostExecutable + " " + args.map(function (arg) { return ("\"" + arg + "\""); }).join(' ')));
            var process = child_process_1.spawn(hostInfo.hostExecutable, args, { cwd: hostDir });
            logger_1.traceProcessOutput(_this.logger, process, "HubHost '" + hostId + "'");
            var result = new HostInfo(hostIdentity, process);
            process.once('exit', function (code, signal) { return _this.onHostExit(result, code, signal); });
            process.once('error', function (err) { return _this.onHostError(result, err); });
            if (!debugHostOnStartup) {
                // When the host just starts, it doesn't have any service on it yet.
                // Even though the hub controller will immediately ask it to start a service,
                // the client may still not connect to the service for a long time, so we'll shut down
                // the host due to inactivity.
                _this.startHostCooldown(result);
            }
            else {
                // startHostCooldown ensures that when a host is started but no client connects to it within 10s
                // then the host will be shutdown. We disable that behavior when user has requested host debugging.
                _this.logger.info("Skipping startHostCooldown since debugging on startup has been requested for host " + hostInfo.hostExecutable);
            }
            _this.logger.info(libraryResourceStrings_1.LibraryResourceStrings.hostLaunched(hostId, result.hostProcess.pid.toString()));
            return result;
        });
    };
    LocationService.prototype.onHostExit = function (host, code, signal) {
        var withCode = code !== undefined && code !== null ? libraryResourceStrings_1.LibraryResourceStrings.hostExitedWithCode(code, exitCode_1.default[code] ? " (" + exitCode_1.default[code] + ")" : '') : '';
        var withSignal = signal !== undefined && signal !== null ? libraryResourceStrings_1.LibraryResourceStrings.hostExitedWithSignal(signal) : '';
        var message = libraryResourceStrings_1.LibraryResourceStrings.hostExited(host.host, host.hostProcess.pid.toString()) + withCode + withSignal;
        // Remove the host pid
        var hostPid = host.hostProcess.pid;
        var pidIndex = this.hostPids.indexOf(hostPid);
        if (pidIndex >= 0) {
            this.hostPids.splice(pidIndex, 1);
        }
        try {
            process.kill(hostPid, signal);
        }
        catch (e) {
            this.logger.warn(libraryResourceStrings_1.LibraryResourceStrings.couldNotKillHost(host.host, e.message));
        }
        this.forgetHost(host, new common_1.ServiceHubError(common_1.ErrorKind.InvalidOperation, message));
    };
    LocationService.prototype.onHostError = function (host, error) {
        this.forgetHost(host, error);
    };
    LocationService.prototype.forgetHost = function (host, rejectHostEndpointReason) {
        if (!host.shutdown) {
            if (rejectHostEndpointReason) {
                host.endPoint.reject(rejectHostEndpointReason);
            }
            host.shutdown = true;
            delete this.hosts[host.getId()];
            for (var _i = 0, _a = host.services; _i < _a.length; _i++) {
                var serviceKey = _a[_i];
                delete this.locatedServices[serviceKey];
            }
            if (Object.getOwnPropertyNames(this.hosts).length === 0) {
                this.onLastHostDisconnected();
            }
        }
    };
    LocationService.prototype.onHostConnecting = function (endPoint, id) {
        // The host has gotten back to us on the host callback pipe we passed as a command line arg. This means we now have a duplex
        // connection with the host that we can use to send startService requests. Since we only launch hosts in response to a service request from a
        // client, this callback means we have a pending launch, we need to complete our promise so the launch code can then communicate the service launch
        // request over the given endpoint to the host.
        var _this = this;
        if (this.hosts.hasOwnProperty(id)) {
            this.hosts[id].done(function (host) {
                // Save the host PID.
                _this.hostPids.push(host.hostProcess.pid);
                // Stash away the endpoint in the host info.
                host.endPoint.resolve(endPoint);
                // If the host end point ends, consider the host gone.
                // Another case when the host is gone is when its process exits.
                endPoint.onClose(function () { return _this.forgetHost(host); }, _this);
                _this.onHostConnected();
            });
        }
        else {
            this.logger.error(libraryResourceStrings_1.LibraryResourceStrings.hostInfoNotFound(id));
        }
    };
    LocationService.prototype.onGetHostIdFailed = function (error) {
        this.logger.error(libraryResourceStrings_1.LibraryResourceStrings.getHostIdFailed("" + (error.message || error)));
        // See if we have only one pending host and reject it if so.
        var singleHostEndpoint = null;
        for (var id in this.hosts) {
            if (!this.hosts.hasOwnProperty(id)) {
                continue;
            }
            var state = this.hosts[id].inspect();
            if (state.state !== 'fulfilled') {
                continue;
            }
            var endPoint = state.value.endPoint;
            if (endPoint.promise.isPending) {
                if (singleHostEndpoint !== null) {
                    // If there is more than one host pending, we do not know which one failed, so just log an error.
                    this.logger.error(libraryResourceStrings_1.LibraryResourceStrings.moreThanOneHostPending);
                    return;
                }
                singleHostEndpoint = endPoint;
            }
        }
        if (singleHostEndpoint !== null) {
            singleHostEndpoint.reject(error);
        }
    };
    LocationService.prototype.getHostCallbackPipe = function () {
        var _this = this;
        return this.callbackPipeName
            .then(function (pipeName) {
            if (_this.callbackPipeCreated) {
                return pipeName;
            }
            var options = {
                name: 'hostCallback',
                logger: _this.logger,
                pipeName: pipeName
            };
            return common_1.serverManager.startService(options, function (stream) {
                var endPoint = jsonRpc_1.JsonRpcConnection.attach(stream, _this.logger, _this, [
                    'hostServicesStarted',
                    'hostServicesEnded'
                ]);
                q(endPoint.sendRequest('getHostId'))
                    .then(function (id) { return _this.onHostConnecting(endPoint, id); })
                    .catch(function (error) { return _this.onGetHostIdFailed(error); })
                    .done();
            })
                .then(function () {
                _this.callbackPipeCreated = true;
                return pipeName;
            });
        });
    };
    LocationService.prototype.getClientWatchServerName = function () {
        var _this = this;
        if (!this.clientWatchServerName) {
            var serverOptions = {
                name: 'clientWatch',
                logger: this.logger,
                onErrorCb: function (e) {
                    var message = "Error in Client Watch server: " + e.message;
                    _this.logger.error(message);
                },
            };
            this.clientWatchServerName = common_1.serverManager.startService(serverOptions, this.onClientWatchConnection.bind(this));
        }
        return this.clientWatchServerName;
    };
    LocationService.prototype.onClientWatchConnection = function (stream) {
        var _this = this;
        this.logger.info('External client connected to client watch.');
        this.externalClientCount++;
        this.stopFastShutdown();
        stream.once('end', function () {
            _this.logger.info('External client disconnected from client watch.');
            _this.externalClientCount--;
            if (_this.externalClientCount === 0) {
                _this.startFastShutdown();
            }
        });
    };
    Object.defineProperty(LocationService.prototype, "currentHostCooldownMs", {
        get: function () {
            return this.isInFastShutdown ? this.fastShutdownHostCooldownMs : this.hostCooldownMs;
        },
        enumerable: true,
        configurable: true
    });
    LocationService.prototype.startFastShutdown = function () {
        if (!this.isInFastShutdown) {
            this.isInFastShutdown = true;
            this.logger.info('Starting fast shutdown');
            this.updateHostCooldownTimeouts();
        }
    };
    LocationService.prototype.stopFastShutdown = function () {
        if (this.isInFastShutdown) {
            this.isInFastShutdown = false;
            this.logger.info('Stopping fast shutdown');
        }
    };
    LocationService.prototype.updateHostCooldownTimeouts = function () {
        var _this = this;
        for (var hostId in this.hosts) {
            this.hosts[hostId].done(function (host) {
                if (!host.hasActiveServices) {
                    _this.startHostCooldown(host);
                }
            }, function (err) {
                _this.logger.error("Error getting host: " + err.message);
            });
        }
    };
    LocationService.prototype.getHost = function (hostId) {
        if (!hostId) {
            throw new Error(libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('hostid'));
        }
        if (!this.hosts.hasOwnProperty(hostId)) {
            throw new Error(libraryResourceStrings_1.LibraryResourceStrings.unknownHost(hostId));
        }
        return this.hosts[hostId];
    };
    LocationService.prototype.startHostCooldown = function (host) {
        host.hasActiveServices = false;
        // Start cooldown unless the host is already shutting down
        if (!host.shutdown) {
            host.setCooldownTimeout(this.shutdownInactiveHost.bind(this), this.currentHostCooldownMs, host);
        }
    };
    LocationService.prototype.shutdownInactiveHost = function (host) {
        if (host.hasActiveServices || host.shutdown) {
            // The host become active again or it has already shutdown
            return;
        }
        var currentTime = new Date().getTime();
        if (host.pendingStartServiceRequestCount === 0
            && host.lastAccessTimeMs < currentTime - this.currentHostCooldownMs) {
            // The host is inactive and no service has started there since the cooldown began.
            this.stopHost(host, 'inactivity').done();
        }
        else {
            // The host is inactive but there was some service started there during the cooldown period.
            // Start another cooldown period to account for the new service.
            this.startHostCooldown(host);
        }
    };
    LocationService.prototype.stopHost = function (host, reason) {
        this.logger.info(libraryResourceStrings_1.LibraryResourceStrings.stoppingHost(host.getId(), host.hostProcess.pid.toString(), reason));
        this.forgetHost(host);
        // No need to fire host cooldown timer if the host is exiting right now.
        host.clearCooldownTimeout();
        var hostProcessExit = q.defer();
        host.hostProcess.once('exit', function (code) { return hostProcessExit.resolve(code); });
        return host.endPoint.promise
            .then(function (endPoint) { return endPoint.sendNotification(hostInfo_1.HostMethod.Exit); })
            .then(function () { return hostProcessExit.promise; });
    };
    LocationService.prototype.isExternalClient = function (clientInfo) {
        // External client is the one that doesn't run in any of Service Hub processes.
        return clientInfo && clientInfo.pid && clientInfo.pid !== process.pid && this.hostPids.indexOf(clientInfo.pid) === -1;
    };
    return LocationService;
}());
exports.LocationService = LocationService;
var LocationServiceProxy = (function () {
    function LocationServiceProxy(endPoint) {
        this.endPoint = endPoint;
        if (!endPoint) {
            throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variablesAreNotDefined('endPoint'));
        }
    }
    LocationServiceProxy.prototype.locate = function (serviceDescriptor, clientInfo) {
        // To support backward compat between client and hub controller,
        // only use service descriptor if the service name alone is not enough.
        var isServiceNameOnly = (new serviceDescriptor_1.ServiceDescriptor(serviceDescriptor)).isServiceNameOnly();
        var serviceNameOrDescriptor = isServiceNameOnly ? serviceDescriptor.name : serviceDescriptor;
        return q(this.endPoint.sendRequest('locate', [serviceNameOrDescriptor, clientInfo]));
    };
    LocationServiceProxy.prototype.stop = function () {
        return q(this.endPoint.sendNotification('exit', ['explicit request from the client']))
            .catch(function (err) {
            // If the controller has already closed, ignore the json rpc error.
            if (err.code !== ec.ENOENT.code) {
                throw err;
            }
        });
    };
    LocationServiceProxy.prototype.getSessionId = function () {
        return q(this.endPoint.sendRequest('getSessionId'));
    };
    LocationServiceProxy.prototype.dispose = function () {
        this.endPoint.dispose();
    };
    LocationServiceProxy.prototype.startHeapDiff = function () {
        return q(this.endPoint.sendRequest('startHeapDiff'));
    };
    LocationServiceProxy.prototype.endHeapDiff = function () {
        return q(this.endPoint.sendRequest('endHeapDiff'));
    };
    LocationServiceProxy.prototype.runGC = function () {
        return q(this.endPoint.sendRequest('runGC'));
    };
    LocationServiceProxy.prototype.captureHeapDump = function (filePath) {
        return q(this.endPoint.sendRequest('captureHeapDump', [filePath]));
    };
    LocationServiceProxy.prototype.getHubControllerPid = function () {
        return q(this.endPoint.sendRequest('getHubControllerPid'));
    };
    return LocationServiceProxy;
}());
exports.LocationServiceProxy = LocationServiceProxy;

//# sourceMappingURL=locationService.js.map
// SIG // Begin signature block
// SIG // MIIiSQYJKoZIhvcNAQcCoIIiOjCCIjYCAQExDzANBglg
// SIG // hkgBZQMEAgEFADB3BgorBgEEAYI3AgEEoGkwZzAyBgor
// SIG // BgEEAYI3AgEeMCQCAQEEEBDgyQbOONQRoqMAEEvTUJAC
// SIG // AQACAQACAQACAQACAQAwMTANBglghkgBZQMEAgEFAAQg
// SIG // 5WYNtjUoP4VO8DMiJWlsts/CNSJbM3jnRAg6e9UFLyWg
// SIG // ggtyMIIE+jCCA+KgAwIBAgITMwAAAs58ms59kF7StwAA
// SIG // AAACzjANBgkqhkiG9w0BAQsFADB+MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBT
// SIG // aWduaW5nIFBDQSAyMDEwMB4XDTE5MDUwMjIxMjU0MloX
// SIG // DTIwMDUwMjIxMjU0MlowdDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEeMBwGA1UEAxMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
// SIG // owhbo0atJEL/giHpwS2stH0eY6h2vghqYkLwa9LT5G5k
// SIG // q5QvKepTYRPV5tH0f5xTRTFTuZopGubS+mDvUkm5CF3C
// SIG // 0w4hucK3Mr8ad7l2yeEmZ7Tikgro6rzUApaNX3pEmKmt
// SIG // 2cPaAJ5XnM4DgC6Q5VV6gX3C/NO+h7ss51xKRb662Hjp
// SIG // y4s1p1weopn+gyWVpQwGNgGwTAcSguLN7XpFUl1esVAC
// SIG // GzvS4MIb8qsl1+aXWvgNUMmzSAwlIHDRnmd6HejS5HEM
// SIG // m4Ye2bYmRKb6Bc7I4hsi3pIm304LK4vSAdLH2HElS7VY
// SIG // mofTuiU2MZUNWaz+14kv5R4yRn+ojSNQUQIDAQABo4IB
// SIG // eTCCAXUwHwYDVR0lBBgwFgYKKwYBBAGCNz0GAQYIKwYB
// SIG // BQUHAwMwHQYDVR0OBBYEFHM/DO7M0NXkSmSFJIwE5oKt
// SIG // gMgPMFAGA1UdEQRJMEekRTBDMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEWMBQG
// SIG // A1UEBRMNMjMwODY1KzQ1NDI0MzAfBgNVHSMEGDAWgBTm
// SIG // /F97uyIAWORyTrX0IXQjMubvrDBWBgNVHR8ETzBNMEug
// SIG // SaBHhkVodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NybC9wcm9kdWN0cy9NaWNDb2RTaWdQQ0FfMjAxMC0w
// SIG // Ny0wNi5jcmwwWgYIKwYBBQUHAQEETjBMMEoGCCsGAQUF
// SIG // BzAChj5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NlcnRzL01pY0NvZFNpZ1BDQV8yMDEwLTA3LTA2LmNy
// SIG // dDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IB
// SIG // AQAcPuovja14/IUbUPK1RjGlfqowbC45KppnIgLxztfM
// SIG // +gbG8nz62UhMR8MjrPM2gyl+dwxqSm5kmAqQ0zeU/lTA
// SIG // z3wH6dvaa8gWwR5gCRYCu0z/s1sR7pjXlQpEILf6QP3t
// SIG // PTg+CEhYbIYxUH+Hr8SQisaxZWCC/rTywdBanMHIMQ7L
// SIG // LZbYaEjmkXmjSzIq4wGjaRRvoJ1cQUXLe0BHgaycOCgk
// SIG // TtJMON5OUqZB3C4rqcjlQmp8/THwHeSGwZgVPYQJCNZi
// SIG // j3V24F1nEKDwiazXleVkJ6smBpa4wBYbPFGR/VzTW49o
// SIG // gr51TH3e4IENba6DsZIhnrk2YP7s/xvsbKdGMIIGcDCC
// SIG // BFigAwIBAgIKYQxSTAAAAAAAAzANBgkqhkiG9w0BAQsF
// SIG // ADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMp
// SIG // TWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9y
// SIG // aXR5IDIwMTAwHhcNMTAwNzA2MjA0MDE3WhcNMjUwNzA2
// SIG // MjA1MDE3WjB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMK
// SIG // V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
// SIG // A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYD
// SIG // VQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBDQSAy
// SIG // MDEwMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
// SIG // AQEA6Q5kUHlntcTj/QkATJ6UrPdWaOpE2M/FWE+ppXZ8
// SIG // bUW60zmStKQe+fllguQX0o/9RJwI6GWTzixVhL99COMu
// SIG // K6hBKxi3oktuSUxrFQfe0dLCiR5xlM21f0u0rwjYzIjW
// SIG // axeUOpPOJj/s5v40mFfVHV1J9rIqLtWFu1k/+JC0K4N0
// SIG // yiuzO0bj8EZJwRdmVMkcvR3EVWJXcvhnuSUgNN5dpqWV
// SIG // XqsogM3Vsp7lA7Vj07IUyMHIiiYKWX8H7P8O7YASNUwS
// SIG // pr5SW/Wm2uCLC0h31oVH1RC5xuiq7otqLQVcYMa0Kluc
// SIG // IxxfReMaFB5vN8sZM4BqiU2jamZjeJPVMM+VHwIDAQAB
// SIG // o4IB4zCCAd8wEAYJKwYBBAGCNxUBBAMCAQAwHQYDVR0O
// SIG // BBYEFOb8X3u7IgBY5HJOtfQhdCMy5u+sMBkGCSsGAQQB
// SIG // gjcUAgQMHgoAUwB1AGIAQwBBMAsGA1UdDwQEAwIBhjAP
// SIG // BgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2VsuP
// SIG // 6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeG
// SIG // RWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
// SIG // L3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEwLTA2LTIz
// SIG // LmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKG
// SIG // Pmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2Vy
// SIG // dHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3J0MIGd
// SIG // BgNVHSAEgZUwgZIwgY8GCSsGAQQBgjcuAzCBgTA9Bggr
// SIG // BgEFBQcCARYxaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
// SIG // L1BLSS9kb2NzL0NQUy9kZWZhdWx0Lmh0bTBABggrBgEF
// SIG // BQcCAjA0HjIgHQBMAGUAZwBhAGwAXwBQAG8AbABpAGMA
// SIG // eQBfAFMAdABhAHQAZQBtAGUAbgB0AC4gHTANBgkqhkiG
// SIG // 9w0BAQsFAAOCAgEAGnTvV08pe8QWhXi4UNMi/AmdrIKX
// SIG // +DT/KiyXlRLl5L/Pv5PI4zSp24G43B4AvtI1b6/lf3mV
// SIG // d+UC1PHr2M1OHhthosJaIxrwjKhiUUVnCOM/PB6T+DCF
// SIG // F8g5QKbXDrMhKeWloWmMIpPMdJjnoUdD8lOswA8waX/+
// SIG // 0iUgbW9h098H1dlyACxphnY9UdumOUjJN2FtB91TGcun
// SIG // 1mHCv+KDqw/ga5uV1n0oUbCJSlGkmmzItx9KGg5pqdfc
// SIG // wX7RSXCqtq27ckdjF/qm1qKmhuyoEESbY7ayaYkGx0aG
// SIG // ehg/6MUdIdV7+QIjLcVBy78dTMgW77Gcf/wiS0mKbhXj
// SIG // pn92W9FTeZGFndXS2z1zNfM8rlSyUkdqwKoTldKOEdqZ
// SIG // Z14yjPs3hdHcdYWch8ZaV4XCv90Nj4ybLeu07s8n07Ve
// SIG // afqkFgQBpyRnc89NT7beBVaXevfpUk30dwVPhcbYC/GO
// SIG // 7UIJ0Q124yNWeCImNr7KsYxuqh3khdpHM2KPpMmRM19x
// SIG // HkCvmGXJIuhCISWKHC1g2TeJQYkqFg/XYTyUaGBS79ZH
// SIG // maCAQO4VgXc+nOBTGBpQHTiVmx5mMxMnORd4hzbOTsNf
// SIG // svU9R1O24OXbC2E9KteSLM43Wj5AQjGkHxAIwlacvyRd
// SIG // UQKdannSF9PawZSOB3slcUSrBmrm1MbfI5qWdcUxghYv
// SIG // MIIWKwIBATCBlTB+MQswCQYDVQQGEwJVUzETMBEGA1UE
// SIG // CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEe
// SIG // MBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgw
// SIG // JgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBD
// SIG // QSAyMDEwAhMzAAACznyazn2QXtK3AAAAAALOMA0GCWCG
// SIG // SAFlAwQCAQUAoIGuMBkGCSqGSIb3DQEJAzEMBgorBgEE
// SIG // AYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3
// SIG // AgEVMC8GCSqGSIb3DQEJBDEiBCBdXr7J4lFiNK2KHVJH
// SIG // 1D6IL+WaHS9TzpC2w7GtC56YkDBCBgorBgEEAYI3AgEM
// SIG // MTQwMqAUgBIATQBpAGMAcgBvAHMAbwBmAHShGoAYaHR0
// SIG // cDovL3d3dy5taWNyb3NvZnQuY29tMA0GCSqGSIb3DQEB
// SIG // AQUABIIBAHVuB/UAt35GOFeUe9Y8wKi3L8lHPc+9V8ut
// SIG // HMN1yYIEtaP8Tc2++Mvj4wzTsixQ2aOxrqG8VxHsdRlH
// SIG // MTRqrx3E0WJU2BXOMb3PcD7UBKMaWDMgtx91cNYmLEOy
// SIG // 7Es9qrCRDObe0oGooRd3sPETszhJa9kGFzjWCJW+Tkik
// SIG // e/AvZLFrPzD9S4wFl/CorN3tXg5Qfbi2A2wnzM9X0oSy
// SIG // ceJe0VrWvkB46sCQBrz9109W2G0oc1Cm/AxQnyJ0AAwl
// SIG // EfoC8ctYB+1Bf7vOr8PYvvan+Z1r8g8cRNXY8kaJfJmS
// SIG // gVhcubnQpEnNQoP2DQ5IfuPUIod2/4ae+bREDzujIOmh
// SIG // ghO5MIITtQYKKwYBBAGCNwMDATGCE6UwghOhBgkqhkiG
// SIG // 9w0BBwKgghOSMIITjgIBAzEPMA0GCWCGSAFlAwQCAQUA
// SIG // MIIBWAYLKoZIhvcNAQkQAQSgggFHBIIBQzCCAT8CAQEG
// SIG // CisGAQQBhFkKAwEwMTANBglghkgBZQMEAgEFAAQgcRSa
// SIG // LtxST5swAtgLSjmRPIw+Mag7Vb4/69DGR/U/XCUCBl0a
// SIG // RbFqNxgTMjAxOTA3MDEyMTEyNTIuNzA3WjAHAgEBgAIB
// SIG // 9KCB1KSB0TCBzjELMAkGA1UEBhMCVVMxEzARBgNVBAgT
// SIG // Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAc
// SIG // BgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcG
// SIG // A1UECxMgTWljcm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRv
// SIG // IFJpY28xJjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOkI4
// SIG // RUMtMzBBNC03MTQ0MSUwIwYDVQQDExxNaWNyb3NvZnQg
// SIG // VGltZS1TdGFtcCBTZXJ2aWNloIIPITCCBnEwggRZoAMC
// SIG // AQICCmEJgSoAAAAAAAIwDQYJKoZIhvcNAQELBQAwgYgx
// SIG // CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9u
// SIG // MRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
// SIG // b3NvZnQgQ29ycG9yYXRpb24xMjAwBgNVBAMTKU1pY3Jv
// SIG // c29mdCBSb290IENlcnRpZmljYXRlIEF1dGhvcml0eSAy
// SIG // MDEwMB4XDTEwMDcwMTIxMzY1NVoXDTI1MDcwMTIxNDY1
// SIG // NVowfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMd
// SIG // TWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwggEi
// SIG // MA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCpHQ28
// SIG // dxGKOiDs/BOX9fp/aZRrdFQQ1aUKAIKF++18aEssX8XD
// SIG // 5WHCdrc+Zitb8BVTJwQxH0EbGpUdzgkTjnxhMFmxMEQP
// SIG // 8WCIhFRDDNdNuDgIs0Ldk6zWczBXJoKjRQ3Q6vVHgc2/
// SIG // JGAyWGBG8lhHhjKEHnRhZ5FfgVSxz5NMksHEpl3RYRNu
// SIG // KMYa+YaAu99h/EbBJx0kZxJyGiGKr0tkiVBisV39dx89
// SIG // 8Fd1rL2KQk1AUdEPnAY+Z3/1ZsADlkR+79BL/W7lmsqx
// SIG // qPJ6Kgox8NpOBpG2iAg16HgcsOmZzTznL0S6p/TcZL2k
// SIG // AcEgCZN4zfy8wMlEXV4WnAEFTyJNAgMBAAGjggHmMIIB
// SIG // 4jAQBgkrBgEEAYI3FQEEAwIBADAdBgNVHQ4EFgQU1WM6
// SIG // XIoxkPNDe3xGG8UzaFqFbVUwGQYJKwYBBAGCNxQCBAwe
// SIG // CgBTAHUAYgBDAEEwCwYDVR0PBAQDAgGGMA8GA1UdEwEB
// SIG // /wQFMAMBAf8wHwYDVR0jBBgwFoAU1fZWy4/oolxiaNE9
// SIG // lJBb186aGMQwVgYDVR0fBE8wTTBLoEmgR4ZFaHR0cDov
// SIG // L2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwvcHJvZHVj
// SIG // dHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3JsMFoG
// SIG // CCsGAQUFBwEBBE4wTDBKBggrBgEFBQcwAoY+aHR0cDov
// SIG // L3d3dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0cy9NaWNS
// SIG // b29DZXJBdXRfMjAxMC0wNi0yMy5jcnQwgaAGA1UdIAEB
// SIG // /wSBlTCBkjCBjwYJKwYBBAGCNy4DMIGBMD0GCCsGAQUF
// SIG // BwIBFjFodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vUEtJ
// SIG // L2RvY3MvQ1BTL2RlZmF1bHQuaHRtMEAGCCsGAQUFBwIC
// SIG // MDQeMiAdAEwAZQBnAGEAbABfAFAAbwBsAGkAYwB5AF8A
// SIG // UwB0AGEAdABlAG0AZQBuAHQALiAdMA0GCSqGSIb3DQEB
// SIG // CwUAA4ICAQAH5ohRDeLG4Jg/gXEDPZ2joSFvs+umzPUx
// SIG // vs8F4qn++ldtGTCzwsVmyWrf9efweL3HqJ4l4/m87WtU
// SIG // VwgrUYJEEvu5U4zM9GASinbMQEBBm9xcF/9c+V4XNZgk
// SIG // Vkt070IQyK+/f8Z/8jd9Wj8c8pl5SpFSAK84Dxf1L3mB
// SIG // ZdmptWvkx872ynoAb0swRCQiPM/tA6WWj1kpvLb9BOFw
// SIG // nzJKJ/1Vry/+tuWOM7tiX5rbV0Dp8c6ZZpCM/2pif93F
// SIG // SguRJuI57BlKcWOdeyFtw5yjojz6f32WapB4pm3S4Zz5
// SIG // Hfw42JT0xqUKloakvZ4argRCg7i1gJsiOCC1JeVk7Pf0
// SIG // v35jWSUPei45V3aicaoGig+JFrphpxHLmtgOR5qAxdDN
// SIG // p9DvfYPw4TtxCd9ddJgiCGHasFAeb73x4QDf5zEHpJM6
// SIG // 92VHeOj4qEir995yfmFrb3epgcunCaw5u+zGy9iCtHLN
// SIG // HfS4hQEegPsbiSpUObJb2sgNVZl6h3M7COaYLeqN4DMu
// SIG // Ein1wC9UJyH3yKxO2ii4sanblrKnQqLJzxlBTeCG+Sqa
// SIG // oxFmMNO7dDJL32N79ZmKLxvHIa9Zta7cRDyXUHHXodLF
// SIG // VeNp3lfB0d4wwP3M5k37Db9dT+mdHhk4L7zPWAUu7w2g
// SIG // UDXa7wknHNWzfjUeCLraNtvTX4/edIhJEjCCBPUwggPd
// SIG // oAMCAQICEzMAAADMOr07cjgRBboAAAAAAMwwDQYJKoZI
// SIG // hvcNAQELBQAwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgT
// SIG // Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAc
// SIG // BgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQG
// SIG // A1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIw
// SIG // MTAwHhcNMTgwODIzMjAyNjI1WhcNMTkxMTIzMjAyNjI1
// SIG // WjCBzjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMg
// SIG // TWljcm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28x
// SIG // JjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOkI4RUMtMzBB
// SIG // NC03MTQ0MSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1T
// SIG // dGFtcCBTZXJ2aWNlMIIBIjANBgkqhkiG9w0BAQEFAAOC
// SIG // AQ8AMIIBCgKCAQEAx9BVpYomFWsV3fB3mLu1+cxTSwZx
// SIG // MScSZOo9LyP6eVEotWi1ut8dW9O6C4MKVYhzX6WezDqC
// SIG // fP6aAlPljpXMganBpFhMx7dwn1f/9c5kFrvG5e70Djoe
// SIG // I7ZPmugkGQn2Ui3bry+hK9eaMZAD3Xc3WdaGxzpQThW4
// SIG // 6w/nqIe62IdK0nYCAxisOukkQEjYpRWT9cIRumIhpnIS
// SIG // fn2yHuIwEiTU4mnchKNBe34BiPGBKhZqrNK83+iPVFa7
// SIG // 6/SAV4tEGV2sWKEU70+9ncepInqzty2Y5mRid+Y2M7CF
// SIG // 5WZ0ePdCe55tt5jSE4MW1VdiW9BTsPDZIFDpcRyeXrFg
// SIG // cuUiiwIDAQABo4IBGzCCARcwHQYDVR0OBBYEFK2pTaZK
// SIG // wmUQT9frex4hmjEzSOupMB8GA1UdIwQYMBaAFNVjOlyK
// SIG // MZDzQ3t8RhvFM2hahW1VMFYGA1UdHwRPME0wS6BJoEeG
// SIG // RWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
// SIG // L3Byb2R1Y3RzL01pY1RpbVN0YVBDQV8yMDEwLTA3LTAx
// SIG // LmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKG
// SIG // Pmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2Vy
// SIG // dHMvTWljVGltU3RhUENBXzIwMTAtMDctMDEuY3J0MAwG
// SIG // A1UdEwEB/wQCMAAwEwYDVR0lBAwwCgYIKwYBBQUHAwgw
// SIG // DQYJKoZIhvcNAQELBQADggEBAD1dXWqC6eJlUTVSw8kc
// SIG // v+ARYFYQp5hy5kO6eCEIwbOrcCnbSTyqxGMoTCYunCNA
// SIG // rgSseGBku6NgIJGTGkBwdypSvN8FCXqExsuonAk1GYW7
// SIG // /sfgwkBySX7iUiY5SwlECOKF4eXrt+rlX6LwswGDwtg0
// SIG // 8rLiQKhNS9L+qBSE6tWdAKq0Q0lMMzGOY557M72S+o+2
// SIG // 8jHvoNJ0q2D49egT9SBdzfEvpEF42LHvox1TyUs6qDhL
// SIG // Uaz0HkrLFm/xO19/XJNT8TaQzA5/xCfNaerk19oNnXxN
// SIG // TOXJpwvjsYdbcrndNZ1bK37F+FgauTQpVweKnPWn3YB7
// SIG // UF7lfulKImsu2N+hggOvMIIClwIBATCB/qGB1KSB0TCB
// SIG // zjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0
// SIG // b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1p
// SIG // Y3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMgTWlj
// SIG // cm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28xJjAk
// SIG // BgNVBAsTHVRoYWxlcyBUU1MgRVNOOkI4RUMtMzBBNC03
// SIG // MTQ0MSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFt
// SIG // cCBTZXJ2aWNloiUKAQEwCQYFKw4DAhoFAAMVAHPahzH9
// SIG // Lvs8aKzn6EaUc0SHv5ZNoIHeMIHbpIHYMIHVMQswCQYD
// SIG // VQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4G
// SIG // A1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
// SIG // IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQg
// SIG // T3BlcmF0aW9ucyBQdWVydG8gUmljbzEnMCUGA1UECxMe
// SIG // bkNpcGhlciBOVFMgRVNOOjRERTktMEM1RS0zRTA5MSsw
// SIG // KQYDVQQDEyJNaWNyb3NvZnQgVGltZSBTb3VyY2UgTWFz
// SIG // dGVyIENsb2NrMA0GCSqGSIb3DQEBBQUAAgUA4MTEQzAi
// SIG // GA8yMDE5MDcwMTE3NDEyM1oYDzIwMTkwNzAyMTc0MTIz
// SIG // WjB2MDwGCisGAQQBhFkKBAExLjAsMAoCBQDgxMRDAgEA
// SIG // MAkCAQACARsCAf8wBwIBAAICGgEwCgIFAODGFcMCAQAw
// SIG // NgYKKwYBBAGEWQoEAjEoMCYwDAYKKwYBBAGEWQoDAaAK
// SIG // MAgCAQACAxbjYKEKMAgCAQACAwehIDANBgkqhkiG9w0B
// SIG // AQUFAAOCAQEABc25eD3DvvwHAM95vq5zhFiM10zC16og
// SIG // glyy0h0dsD6X3mVTrxeDxw/juYXXwy6opHws2Wnq2frN
// SIG // snIaXQHBsE6LotG8PDmmb6zS7Z6lrGmgV7nlztwZ+dwm
// SIG // zgqJ3dDj/V1yYzAxPCh6fWcd9wWbMHvlW+WgrlEMY9Xn
// SIG // iTHvWcmRtOTB7UgCZod2A8uEQ+owGy6YviucVeSF9u0s
// SIG // I1g/fw8K2CWwMzsJC+23idPBzNIk0j1zHJBwaRjIt2Ks
// SIG // c0bgKrxRUUzt3IP0KnyJmpAFWgYz3Z7/NmGi32bsMt28
// SIG // ZQM0AFWE615oF/AZZJW1TySYVTfgb/4/la6yFtS81N9O
// SIG // zTGCAvUwggLxAgEBMIGTMHwxCzAJBgNVBAYTAlVTMRMw
// SIG // EQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRt
// SIG // b25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRp
// SIG // b24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0YW1w
// SIG // IFBDQSAyMDEwAhMzAAAAzDq9O3I4EQW6AAAAAADMMA0G
// SIG // CWCGSAFlAwQCAQUAoIIBMjAaBgkqhkiG9w0BCQMxDQYL
// SIG // KoZIhvcNAQkQAQQwLwYJKoZIhvcNAQkEMSIEIBJv7NwP
// SIG // AlIm8cFYkBffDD1BaexLq3iP+ftHwnesUneAMIHiBgsq
// SIG // hkiG9w0BCRACDDGB0jCBzzCBzDCBsQQUc9qHMf0u+zxo
// SIG // rOfoRpRzRIe/lk0wgZgwgYCkfjB8MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQgVGltZS1T
// SIG // dGFtcCBQQ0EgMjAxMAITMwAAAMw6vTtyOBEFugAAAAAA
// SIG // zDAWBBSYosAtOhmXkYE4tXGsOM68pjRkWDANBgkqhkiG
// SIG // 9w0BAQsFAASCAQAKPmWyESePHpNIGIerlmmv8Vqbl+kw
// SIG // YTQAu5h2lDPvDEcyeTWN+J5Wr1hj1+JVZLUPm80ZYQ1F
// SIG // C9AtrrN8BhfoybUfc3XHXoRKxH5z4Zlv+ZN/3SL+ZXbv
// SIG // luYz4gSCQRA6vrn5mx6TbYfaVmxdFwc9JdGO9HMK5lBD
// SIG // PmMuXKu+GSpW5Fz9GCpHnmTwZpbOQd+IYfyabnSFBtzK
// SIG // hP5g782N+gCnG5QmlFg7OHA465HSv1xXYtA4tcoVj+kN
// SIG // YSGVu7o7Oo/V2tFWpvgO9CGvve7ueKHhX0ls4VTA7GB1
// SIG // Uan3gvkfDtYp8mmbSM9GcrV4F10Clb36rxBU/TdJw53C
// SIG // 4Tyi
// SIG // End signature block
