"use strict";
var fs = require('fs');
var path = require('path');
var os = require('os');
var util = require('util');
var c = require('./common');
var mkdirp = require('mkdir-parents');
var ec = require('errno-codes');
var tmp = require('tmp');
var exitCode_1 = require('./exitCode');
var libraryResourceStrings_1 = require('./libraryResourceStrings');
/**
 * Log levels
 */
(function (LogLevel) {
    LogLevel[LogLevel["Critical"] = 1] = "Critical";
    LogLevel[LogLevel["Error"] = 2] = "Error";
    LogLevel[LogLevel["Warning"] = 4] = "Warning";
    LogLevel[LogLevel["Information"] = 8] = "Information";
    LogLevel[LogLevel["Verbose"] = 16] = "Verbose";
})(exports.LogLevel || (exports.LogLevel = {}));
var LogLevel = exports.LogLevel;
exports.logFileDirectory = c.getLogFilesDir();
function traceProcessOutput(logger, process, processName) {
    if (!logger) {
        throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, 'logger is not defined');
    }
    if (!process) {
        throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, 'process is not defined');
    }
    if (!processName) {
        throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, 'processName is not defined');
    }
    process.once('exit', function (code, signal) {
        var codeOrSignal = '';
        if (typeof code === 'number') {
            codeOrSignal = " with code " + code;
            if (exitCode_1.default[code]) {
                codeOrSignal += " (" + exitCode_1.default[code] + ")";
            }
        }
        if (signal) {
            codeOrSignal += " with signal '" + signal + "''";
        }
        logger.info("" + processName + getPid(process) + " exited" + codeOrSignal + ".");
    });
    process.once('error', function (err) {
        logger.error("" + processName + getPid(process) + " error " + err.message + ".");
    });
    process.stderr.on('data', function (data) {
        logger.error("" + processName + getPid(process) + " stderr: " + data.toString());
    });
    if (logger.isEnabled(LogLevel.Verbose)) {
        process.stdout.on('data', function (data) {
            logger.verbose("" + processName + getPid(process) + " stdout: " + data.toString());
        });
    }
}
exports.traceProcessOutput = traceProcessOutput;
function getPid(process) {
    return process.pid ? ' PID ' + process.pid : '';
}
/** Trace level, must be in sync with System.Diagnostic.SourceLevels */
var SourceLevels;
(function (SourceLevels) {
    /** Allows all events through. */
    SourceLevels[SourceLevels["All"] = -1] = "All";
    /** Does not allow any events through. */
    SourceLevels[SourceLevels["Off"] = 0] = "Off";
    /** Allows only Critical events through. */
    SourceLevels[SourceLevels["Critical"] = LogLevel.Critical] = "Critical";
    /** Allows Critical and Error events through. */
    SourceLevels[SourceLevels["Error"] = 3] = "Error";
    /** Allows Critical, Error, and Warning events through. */
    SourceLevels[SourceLevels["Warning"] = 7] = "Warning";
    /** Allows Critical, Error, Warning, and Information events through. */
    SourceLevels[SourceLevels["Information"] = 15] = "Information";
    /** Allows Critical, Error, Warning, Information, and Verbose events through. */
    SourceLevels[SourceLevels["Verbose"] = 31] = "Verbose";
})(SourceLevels || (SourceLevels = {}));
/** Trace level. Must be in sync with EnvUtils.TraceLevelEnvVarName in src\clr\utility\Microsoft.ServiceHub.Utility.Shared\EnvUtils.cs */
exports.SourceLevelEnvironmentVariable = 'SERVICEHUBTRACELEVEL';
function cleanUpOldLogs(delayInMsecBeforeStartingLogCleanup, deleteLogFilesOlderThanDate) {
    return setTimeout(function () {
        fs.exists(exports.logFileDirectory, function (exists) {
            if (!exists) {
                return;
            }
            var logger = new Logger('oldLogsCleaner');
            // get list of log files
            fs.readdir(exports.logFileDirectory, function (err, files) {
                if (err) {
                    logger.error(libraryResourceStrings_1.LibraryResourceStrings.failedToEnumerateFilesInDir(exports.logFileDirectory, err.code, err.message));
                    return;
                }
                for (var i = 0; i < files.length; i++) {
                    // get "Change Time" for each log file
                    (function (file) {
                        fs.stat(file, function (err, stats) {
                            if (err) {
                                logger.error(libraryResourceStrings_1.LibraryResourceStrings.failedToGetFileStats(file, err.code, err.message));
                                return;
                            }
                            // check if the log file is old enough to be deleted based on its "Change Time"
                            if (stats.ctime.getTime() < deleteLogFilesOlderThanDate.getTime()) {
                                // delete file as it hasn't been used since the specified number of days
                                fs.unlink(file, function (err) {
                                    if (err) {
                                        logger.error(libraryResourceStrings_1.LibraryResourceStrings.failedToDeleteFile(file, err.code, err.message));
                                    }
                                });
                            }
                        });
                    })(path.join(exports.logFileDirectory, files[i]));
                }
            });
        });
    }, delayInMsecBeforeStartingLogCleanup);
}
exports.cleanUpOldLogs = cleanUpOldLogs;
var Logger = (function () {
    function Logger(logFilePrefix, options) {
        this.maxFileSizeInBytes = 500 * 1024; // 500 KB
        this.nextLogFileNumber = 1;
        this.currentFileSizeInBytes = 0;
        this.encoding = 'utf8';
        if (!logFilePrefix) {
            throw new Error(libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('LogFilePrefix'));
        }
        options = options || {};
        this.defaultLevel = options.defaultLevel || LogLevel.Verbose;
        var sourceLevelsEnvironment = options.logSourceLevel || process.env[exports.SourceLevelEnvironmentVariable];
        this.sourceLevels = parseInt(sourceLevelsEnvironment);
        if (isNaN(this.sourceLevels)) {
            var enumValue = c.getPropertyNoCase(SourceLevels, sourceLevelsEnvironment);
            this.sourceLevels = typeof enumValue === 'number' ? enumValue : SourceLevels.Error;
        }
        var dir = options.logFileDirectory || exports.logFileDirectory;
        this.logFilePath = path.join(dir, tmp.tmpNameSync({ template: logFilePrefix + "-" + process.pid + "-XXXXXX", postfix: '.log' }) + '.log');
        this.logFilePrefix = tmp.tmpNameSync({ template: logFilePrefix + "-" + process.pid + "-XXXXXX", postfix: '.log' });
        this.logFilePath = this.getLogFilePath();
        this.rollingEnabled = !this.isEnabled(LogLevel.Verbose);
    }
    Logger.prototype.critical = function (message) {
        this.LogInternal(LogLevel.Critical, message);
    };
    Logger.prototype.error = function (message) {
        this.LogInternal(LogLevel.Error, message);
    };
    Logger.prototype.warn = function (message) {
        this.LogInternal(LogLevel.Warning, message);
    };
    Logger.prototype.info = function (message) {
        this.LogInternal(LogLevel.Information, message);
    };
    Logger.prototype.verbose = function (message) {
        this.LogInternal(LogLevel.Verbose, message);
    };
    Logger.prototype.log = function (message) {
        this.LogInternal(this.defaultLevel, message);
    };
    Logger.prototype.isEnabled = function (level) {
        return (this.sourceLevels & level) != 0;
    };
    Logger.prototype.LogInternal = function (level, message) {
        try {
            if (this.isEnabled(level)) {
                if (!this.started) {
                    this.started = true;
                    this.startLogging();
                }
                this.logMessage(level, message);
            }
        }
        catch (e) {
        }
    };
    Logger.prototype.startLogging = function () {
        var logDirectory = path.dirname(this.logFilePath);
        if (!fs.existsSync(logDirectory)) {
            try {
                mkdirp.sync(logDirectory, c.unixOwnerOnlyAccessMode);
            }
            catch (e) {
                if (e.code !== ec.EEXIST.code) {
                    throw e;
                }
            }
        }
        this.logMessage(LogLevel.Information, libraryResourceStrings_1.LibraryResourceStrings.logConfig("$" + exports.SourceLevelEnvironmentVariable + "=\"" + (process.env[exports.SourceLevelEnvironmentVariable] || '') + "\""));
        // don't count this message for our roll over file size calculation
        this.currentFileSizeInBytes = 0;
    };
    Logger.prototype.logMessage = function (level, message) {
        if (!message) {
            return;
        }
        var text;
        if (typeof message === 'string') {
            text = message;
        }
        else if (message.message) {
            text = message;
            if (message.stack) {
                text += os.EOL + libraryResourceStrings_1.LibraryResourceStrings.messageAt + ("" + os.EOL + message.stack);
            }
        }
        else {
            text = util.inspect(message);
        }
        var finalMessage = formatMessage(text, level);
        // Since message header and footer are short and of constant length managed ServiceHub logger ignores them to simplify file size calculation.
        // To be consistent we do the same here for JS logger. This is ok because we don't need an accurate file size for rolling over to a new file.
        // This also helps in keeping the tests simple.
        var buffer = new Buffer(text, this.encoding);
        if (this.rollingEnabled && this.currentFileSizeInBytes > 0 && (this.currentFileSizeInBytes + buffer.length > this.maxFileSizeInBytes)) {
            // delete lastLogFilePath
            if (this.lastLogFilePath) {
                fs.unlink(this.lastLogFilePath, function (err) { });
            }
            this.lastLogFilePath = this.logFilePath;
            this.logFilePath = this.getLogFilePath();
            fs.appendFileSync(this.logFilePath, formatMessage(libraryResourceStrings_1.LibraryResourceStrings.logConfig("$" + exports.SourceLevelEnvironmentVariable + "=\"" + (process.env[exports.SourceLevelEnvironmentVariable] || '') + "\""), LogLevel.Information), { encoding: this.encoding });
            this.currentFileSizeInBytes = 0;
        }
        fs.appendFileSync(this.logFilePath, finalMessage, { encoding: this.encoding });
        this.currentFileSizeInBytes += buffer.length;
    };
    Logger.prototype.getLogFilePath = function () {
        return path.join(exports.logFileDirectory, this.logFilePrefix + "-" + this.nextLogFileNumber++ + ".log");
    };
    return Logger;
}());
exports.Logger = Logger;
function formatMessage(message, level) {
    return formatDateTime() + " : " + LogLevel[level] + " : " + message + os.EOL;
}
function twoDigits(v) {
    return "" + (v < 10 ? '0' : '') + v;
}
function formatTime(date) {
    return twoDigits(date.getHours()) + ":" + twoDigits(date.getMinutes()) + ":" + twoDigits(date.getSeconds());
}
function formatDateTime() {
    var now = new Date();
    return twoDigits(now.getMonth() + 1) + "/" + twoDigits(now.getDate()) + "/" + now.getFullYear() + " " + formatTime(now);
}

//# sourceMappingURL=logger.js.map
// SIG // Begin signature block
// SIG // MIIiSAYJKoZIhvcNAQcCoIIiOTCCIjUCAQExDzANBglg
// SIG // hkgBZQMEAgEFADB3BgorBgEEAYI3AgEEoGkwZzAyBgor
// SIG // BgEEAYI3AgEeMCQCAQEEEBDgyQbOONQRoqMAEEvTUJAC
// SIG // AQACAQACAQACAQACAQAwMTANBglghkgBZQMEAgEFAAQg
// SIG // Sz+N8IUrGn//bLRAv1w4xiFgHWBhx8kw9Q2n9uMUv+mg
// SIG // ggtyMIIE+jCCA+KgAwIBAgITMwAAAs58ms59kF7StwAA
// SIG // AAACzjANBgkqhkiG9w0BAQsFADB+MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBT
// SIG // aWduaW5nIFBDQSAyMDEwMB4XDTE5MDUwMjIxMjU0MloX
// SIG // DTIwMDUwMjIxMjU0MlowdDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEeMBwGA1UEAxMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
// SIG // owhbo0atJEL/giHpwS2stH0eY6h2vghqYkLwa9LT5G5k
// SIG // q5QvKepTYRPV5tH0f5xTRTFTuZopGubS+mDvUkm5CF3C
// SIG // 0w4hucK3Mr8ad7l2yeEmZ7Tikgro6rzUApaNX3pEmKmt
// SIG // 2cPaAJ5XnM4DgC6Q5VV6gX3C/NO+h7ss51xKRb662Hjp
// SIG // y4s1p1weopn+gyWVpQwGNgGwTAcSguLN7XpFUl1esVAC
// SIG // GzvS4MIb8qsl1+aXWvgNUMmzSAwlIHDRnmd6HejS5HEM
// SIG // m4Ye2bYmRKb6Bc7I4hsi3pIm304LK4vSAdLH2HElS7VY
// SIG // mofTuiU2MZUNWaz+14kv5R4yRn+ojSNQUQIDAQABo4IB
// SIG // eTCCAXUwHwYDVR0lBBgwFgYKKwYBBAGCNz0GAQYIKwYB
// SIG // BQUHAwMwHQYDVR0OBBYEFHM/DO7M0NXkSmSFJIwE5oKt
// SIG // gMgPMFAGA1UdEQRJMEekRTBDMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEWMBQG
// SIG // A1UEBRMNMjMwODY1KzQ1NDI0MzAfBgNVHSMEGDAWgBTm
// SIG // /F97uyIAWORyTrX0IXQjMubvrDBWBgNVHR8ETzBNMEug
// SIG // SaBHhkVodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NybC9wcm9kdWN0cy9NaWNDb2RTaWdQQ0FfMjAxMC0w
// SIG // Ny0wNi5jcmwwWgYIKwYBBQUHAQEETjBMMEoGCCsGAQUF
// SIG // BzAChj5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NlcnRzL01pY0NvZFNpZ1BDQV8yMDEwLTA3LTA2LmNy
// SIG // dDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IB
// SIG // AQAcPuovja14/IUbUPK1RjGlfqowbC45KppnIgLxztfM
// SIG // +gbG8nz62UhMR8MjrPM2gyl+dwxqSm5kmAqQ0zeU/lTA
// SIG // z3wH6dvaa8gWwR5gCRYCu0z/s1sR7pjXlQpEILf6QP3t
// SIG // PTg+CEhYbIYxUH+Hr8SQisaxZWCC/rTywdBanMHIMQ7L
// SIG // LZbYaEjmkXmjSzIq4wGjaRRvoJ1cQUXLe0BHgaycOCgk
// SIG // TtJMON5OUqZB3C4rqcjlQmp8/THwHeSGwZgVPYQJCNZi
// SIG // j3V24F1nEKDwiazXleVkJ6smBpa4wBYbPFGR/VzTW49o
// SIG // gr51TH3e4IENba6DsZIhnrk2YP7s/xvsbKdGMIIGcDCC
// SIG // BFigAwIBAgIKYQxSTAAAAAAAAzANBgkqhkiG9w0BAQsF
// SIG // ADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMp
// SIG // TWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9y
// SIG // aXR5IDIwMTAwHhcNMTAwNzA2MjA0MDE3WhcNMjUwNzA2
// SIG // MjA1MDE3WjB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMK
// SIG // V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
// SIG // A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYD
// SIG // VQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBDQSAy
// SIG // MDEwMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
// SIG // AQEA6Q5kUHlntcTj/QkATJ6UrPdWaOpE2M/FWE+ppXZ8
// SIG // bUW60zmStKQe+fllguQX0o/9RJwI6GWTzixVhL99COMu
// SIG // K6hBKxi3oktuSUxrFQfe0dLCiR5xlM21f0u0rwjYzIjW
// SIG // axeUOpPOJj/s5v40mFfVHV1J9rIqLtWFu1k/+JC0K4N0
// SIG // yiuzO0bj8EZJwRdmVMkcvR3EVWJXcvhnuSUgNN5dpqWV
// SIG // XqsogM3Vsp7lA7Vj07IUyMHIiiYKWX8H7P8O7YASNUwS
// SIG // pr5SW/Wm2uCLC0h31oVH1RC5xuiq7otqLQVcYMa0Kluc
// SIG // IxxfReMaFB5vN8sZM4BqiU2jamZjeJPVMM+VHwIDAQAB
// SIG // o4IB4zCCAd8wEAYJKwYBBAGCNxUBBAMCAQAwHQYDVR0O
// SIG // BBYEFOb8X3u7IgBY5HJOtfQhdCMy5u+sMBkGCSsGAQQB
// SIG // gjcUAgQMHgoAUwB1AGIAQwBBMAsGA1UdDwQEAwIBhjAP
// SIG // BgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2VsuP
// SIG // 6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeG
// SIG // RWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
// SIG // L3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEwLTA2LTIz
// SIG // LmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKG
// SIG // Pmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2Vy
// SIG // dHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3J0MIGd
// SIG // BgNVHSAEgZUwgZIwgY8GCSsGAQQBgjcuAzCBgTA9Bggr
// SIG // BgEFBQcCARYxaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
// SIG // L1BLSS9kb2NzL0NQUy9kZWZhdWx0Lmh0bTBABggrBgEF
// SIG // BQcCAjA0HjIgHQBMAGUAZwBhAGwAXwBQAG8AbABpAGMA
// SIG // eQBfAFMAdABhAHQAZQBtAGUAbgB0AC4gHTANBgkqhkiG
// SIG // 9w0BAQsFAAOCAgEAGnTvV08pe8QWhXi4UNMi/AmdrIKX
// SIG // +DT/KiyXlRLl5L/Pv5PI4zSp24G43B4AvtI1b6/lf3mV
// SIG // d+UC1PHr2M1OHhthosJaIxrwjKhiUUVnCOM/PB6T+DCF
// SIG // F8g5QKbXDrMhKeWloWmMIpPMdJjnoUdD8lOswA8waX/+
// SIG // 0iUgbW9h098H1dlyACxphnY9UdumOUjJN2FtB91TGcun
// SIG // 1mHCv+KDqw/ga5uV1n0oUbCJSlGkmmzItx9KGg5pqdfc
// SIG // wX7RSXCqtq27ckdjF/qm1qKmhuyoEESbY7ayaYkGx0aG
// SIG // ehg/6MUdIdV7+QIjLcVBy78dTMgW77Gcf/wiS0mKbhXj
// SIG // pn92W9FTeZGFndXS2z1zNfM8rlSyUkdqwKoTldKOEdqZ
// SIG // Z14yjPs3hdHcdYWch8ZaV4XCv90Nj4ybLeu07s8n07Ve
// SIG // afqkFgQBpyRnc89NT7beBVaXevfpUk30dwVPhcbYC/GO
// SIG // 7UIJ0Q124yNWeCImNr7KsYxuqh3khdpHM2KPpMmRM19x
// SIG // HkCvmGXJIuhCISWKHC1g2TeJQYkqFg/XYTyUaGBS79ZH
// SIG // maCAQO4VgXc+nOBTGBpQHTiVmx5mMxMnORd4hzbOTsNf
// SIG // svU9R1O24OXbC2E9KteSLM43Wj5AQjGkHxAIwlacvyRd
// SIG // UQKdannSF9PawZSOB3slcUSrBmrm1MbfI5qWdcUxghYu
// SIG // MIIWKgIBATCBlTB+MQswCQYDVQQGEwJVUzETMBEGA1UE
// SIG // CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEe
// SIG // MBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgw
// SIG // JgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBD
// SIG // QSAyMDEwAhMzAAACznyazn2QXtK3AAAAAALOMA0GCWCG
// SIG // SAFlAwQCAQUAoIGuMBkGCSqGSIb3DQEJAzEMBgorBgEE
// SIG // AYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3
// SIG // AgEVMC8GCSqGSIb3DQEJBDEiBCB5QrpVmbB3/mHqGn2X
// SIG // nVCGN5ZJSdaC9tZJRTSCOU3XMjBCBgorBgEEAYI3AgEM
// SIG // MTQwMqAUgBIATQBpAGMAcgBvAHMAbwBmAHShGoAYaHR0
// SIG // cDovL3d3dy5taWNyb3NvZnQuY29tMA0GCSqGSIb3DQEB
// SIG // AQUABIIBAER7BMWnmDTiVFAYkl/U6ePevm1kTuZLu/aD
// SIG // 8MOMhz7HOoDcXoZ+qqgloHs9DRAZB7rhmomZusTn8YWK
// SIG // YYq35mN3Y+QaC1dzwIMsbhTv65zl8NrhNAA9Zhro9NR8
// SIG // y3URHbjRQKoh0vPZRBXjEkq7gHdiZafxpCMJDeR/Fx7q
// SIG // xiJI+nkeFZzHH1EXzdDtLwI33zf6M3Zq//E1swFE8Uqv
// SIG // PkYeTysSJlevCgo/JTToeTOAotQW5XUvbFCVqtB0/W0j
// SIG // UWbOH7YzQSAnTRsfzNxKYIq3IGpRErpBSgZjiY3fMvun
// SIG // ZJr6DJswFUyvQTH15QNXDF8/wfqfQkh4xjS8z4SJmDOh
// SIG // ghO4MIITtAYKKwYBBAGCNwMDATGCE6QwghOgBgkqhkiG
// SIG // 9w0BBwKgghORMIITjQIBAzEPMA0GCWCGSAFlAwQCAQUA
// SIG // MIIBVwYLKoZIhvcNAQkQAQSgggFGBIIBQjCCAT4CAQEG
// SIG // CisGAQQBhFkKAwEwMTANBglghkgBZQMEAgEFAAQgtgPz
// SIG // caGJKz/BiPHfk92tjDiTqkOJEJJFUqwrMiFKjf8CBl0a
// SIG // L9FrzBgSMjAxOTA3MDEyMTEyNTIuNzdaMAcCAQGAAgH0
// SIG // oIHUpIHRMIHOMQswCQYDVQQGEwJVUzETMBEGA1UECBMK
// SIG // V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
// SIG // A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSkwJwYD
// SIG // VQQLEyBNaWNyb3NvZnQgT3BlcmF0aW9ucyBQdWVydG8g
// SIG // UmljbzEmMCQGA1UECxMdVGhhbGVzIFRTUyBFU046MzFD
// SIG // NS0zMEJBLTdDOTExJTAjBgNVBAMTHE1pY3Jvc29mdCBU
// SIG // aW1lLVN0YW1wIFNlcnZpY2Wggg8hMIIE9TCCA92gAwIB
// SIG // AgITMwAAAM2m2zivvS1BAAAAAAAAzTANBgkqhkiG9w0B
// SIG // AQsFADB8MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2Fz
// SIG // aGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UE
// SIG // ChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSYwJAYDVQQD
// SIG // Ex1NaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0EgMjAxMDAe
// SIG // Fw0xODA4MjMyMDI2MjZaFw0xOTExMjMyMDI2MjZaMIHO
// SIG // MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3Rv
// SIG // bjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWlj
// SIG // cm9zb2Z0IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEmMCQG
// SIG // A1UECxMdVGhhbGVzIFRTUyBFU046MzFDNS0zMEJBLTdD
// SIG // OTExJTAjBgNVBAMTHE1pY3Jvc29mdCBUaW1lLVN0YW1w
// SIG // IFNlcnZpY2UwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAw
// SIG // ggEKAoIBAQCyGo7FFLRXglYKrtD3s5lk8Uq9UgaJo2HO
// SIG // V3qNRbolSSs2UpJbkDoPjldqGRTfZQBH4IPyDLNA4Cs3
// SIG // Arjjpko+aKeveM+26PJJ6gQuVR3qDGJKo7ZHnmL6EZwq
// SIG // /mckGaeCk2ZrIrMkEIJKsgFnI0reD92vo2eAwU184uqd
// SIG // rFAolclNJUDNJKqwWoVzCPmhB3VaBnIhQlR+juIkyfTb
// SIG // kKspDrTl28iZW7FLa5SMJL36VR0FsWdNb1vZd1BtwjCP
// SIG // hIC3SLRE507JAJLSTxD1L7PfF341cYFUarl24UKVyouB
// SIG // AhWQs28pux9Wsbz7aB5yxnP/mr17T3F73ECEOcgW27Hb
// SIG // AgMBAAGjggEbMIIBFzAdBgNVHQ4EFgQUuB30YuL6CpIq
// SIG // VyAxBNnQvsnyAg4wHwYDVR0jBBgwFoAU1WM6XIoxkPND
// SIG // e3xGG8UzaFqFbVUwVgYDVR0fBE8wTTBLoEmgR4ZFaHR0
// SIG // cDovL2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwvcHJv
// SIG // ZHVjdHMvTWljVGltU3RhUENBXzIwMTAtMDctMDEuY3Js
// SIG // MFoGCCsGAQUFBwEBBE4wTDBKBggrBgEFBQcwAoY+aHR0
// SIG // cDovL3d3dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0cy9N
// SIG // aWNUaW1TdGFQQ0FfMjAxMC0wNy0wMS5jcnQwDAYDVR0T
// SIG // AQH/BAIwADATBgNVHSUEDDAKBggrBgEFBQcDCDANBgkq
// SIG // hkiG9w0BAQsFAAOCAQEAQkIAKiiPfcD4uSDnHnY1PJ6w
// SIG // WkIF0GH48CxDPewPCO5eQIuiBcsG15Lt/ckBazjdJwQq
// SIG // 20k1C6y8vwrjSVO0hr8q+nz6d1BWoC8XiymTQKaoGUfG
// SIG // BPZkzvYy0qpj7gho6yR0eKIjRr6pjyKxbLV9pdIHHfsR
// SIG // RtZ78kLfgZpR0wMC6+8Ph2hk7LbEE/nJKWqZZDk1dHV3
// SIG // 0WKPokS12+WimwopoKoodBtkyvdGdQ3HWzYQFw9a3k73
// SIG // tTeU1K058gXlaXmhJhp/LsEUTA6XSTFpGH5+ViTgWTwy
// SIG // cXAHud68rUioXkah74TDTO2/385SdTZa20fWnfu78CBs
// SIG // Och9qtQfnTCCBnEwggRZoAMCAQICCmEJgSoAAAAAAAIw
// SIG // DQYJKoZIhvcNAQELBQAwgYgxCzAJBgNVBAYTAlVTMRMw
// SIG // EQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRt
// SIG // b25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRp
// SIG // b24xMjAwBgNVBAMTKU1pY3Jvc29mdCBSb290IENlcnRp
// SIG // ZmljYXRlIEF1dGhvcml0eSAyMDEwMB4XDTEwMDcwMTIx
// SIG // MzY1NVoXDTI1MDcwMTIxNDY1NVowfDELMAkGA1UEBhMC
// SIG // VVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcT
// SIG // B1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jw
// SIG // b3JhdGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUt
// SIG // U3RhbXAgUENBIDIwMTAwggEiMA0GCSqGSIb3DQEBAQUA
// SIG // A4IBDwAwggEKAoIBAQCpHQ28dxGKOiDs/BOX9fp/aZRr
// SIG // dFQQ1aUKAIKF++18aEssX8XD5WHCdrc+Zitb8BVTJwQx
// SIG // H0EbGpUdzgkTjnxhMFmxMEQP8WCIhFRDDNdNuDgIs0Ld
// SIG // k6zWczBXJoKjRQ3Q6vVHgc2/JGAyWGBG8lhHhjKEHnRh
// SIG // Z5FfgVSxz5NMksHEpl3RYRNuKMYa+YaAu99h/EbBJx0k
// SIG // ZxJyGiGKr0tkiVBisV39dx898Fd1rL2KQk1AUdEPnAY+
// SIG // Z3/1ZsADlkR+79BL/W7lmsqxqPJ6Kgox8NpOBpG2iAg1
// SIG // 6HgcsOmZzTznL0S6p/TcZL2kAcEgCZN4zfy8wMlEXV4W
// SIG // nAEFTyJNAgMBAAGjggHmMIIB4jAQBgkrBgEEAYI3FQEE
// SIG // AwIBADAdBgNVHQ4EFgQU1WM6XIoxkPNDe3xGG8UzaFqF
// SIG // bVUwGQYJKwYBBAGCNxQCBAweCgBTAHUAYgBDAEEwCwYD
// SIG // VR0PBAQDAgGGMA8GA1UdEwEB/wQFMAMBAf8wHwYDVR0j
// SIG // BBgwFoAU1fZWy4/oolxiaNE9lJBb186aGMQwVgYDVR0f
// SIG // BE8wTTBLoEmgR4ZFaHR0cDovL2NybC5taWNyb3NvZnQu
// SIG // Y29tL3BraS9jcmwvcHJvZHVjdHMvTWljUm9vQ2VyQXV0
// SIG // XzIwMTAtMDYtMjMuY3JsMFoGCCsGAQUFBwEBBE4wTDBK
// SIG // BggrBgEFBQcwAoY+aHR0cDovL3d3dy5taWNyb3NvZnQu
// SIG // Y29tL3BraS9jZXJ0cy9NaWNSb29DZXJBdXRfMjAxMC0w
// SIG // Ni0yMy5jcnQwgaAGA1UdIAEB/wSBlTCBkjCBjwYJKwYB
// SIG // BAGCNy4DMIGBMD0GCCsGAQUFBwIBFjFodHRwOi8vd3d3
// SIG // Lm1pY3Jvc29mdC5jb20vUEtJL2RvY3MvQ1BTL2RlZmF1
// SIG // bHQuaHRtMEAGCCsGAQUFBwICMDQeMiAdAEwAZQBnAGEA
// SIG // bABfAFAAbwBsAGkAYwB5AF8AUwB0AGEAdABlAG0AZQBu
// SIG // AHQALiAdMA0GCSqGSIb3DQEBCwUAA4ICAQAH5ohRDeLG
// SIG // 4Jg/gXEDPZ2joSFvs+umzPUxvs8F4qn++ldtGTCzwsVm
// SIG // yWrf9efweL3HqJ4l4/m87WtUVwgrUYJEEvu5U4zM9GAS
// SIG // inbMQEBBm9xcF/9c+V4XNZgkVkt070IQyK+/f8Z/8jd9
// SIG // Wj8c8pl5SpFSAK84Dxf1L3mBZdmptWvkx872ynoAb0sw
// SIG // RCQiPM/tA6WWj1kpvLb9BOFwnzJKJ/1Vry/+tuWOM7ti
// SIG // X5rbV0Dp8c6ZZpCM/2pif93FSguRJuI57BlKcWOdeyFt
// SIG // w5yjojz6f32WapB4pm3S4Zz5Hfw42JT0xqUKloakvZ4a
// SIG // rgRCg7i1gJsiOCC1JeVk7Pf0v35jWSUPei45V3aicaoG
// SIG // ig+JFrphpxHLmtgOR5qAxdDNp9DvfYPw4TtxCd9ddJgi
// SIG // CGHasFAeb73x4QDf5zEHpJM692VHeOj4qEir995yfmFr
// SIG // b3epgcunCaw5u+zGy9iCtHLNHfS4hQEegPsbiSpUObJb
// SIG // 2sgNVZl6h3M7COaYLeqN4DMuEin1wC9UJyH3yKxO2ii4
// SIG // sanblrKnQqLJzxlBTeCG+SqaoxFmMNO7dDJL32N79ZmK
// SIG // LxvHIa9Zta7cRDyXUHHXodLFVeNp3lfB0d4wwP3M5k37
// SIG // Db9dT+mdHhk4L7zPWAUu7w2gUDXa7wknHNWzfjUeCLra
// SIG // NtvTX4/edIhJEqGCA68wggKXAgEBMIH+oYHUpIHRMIHO
// SIG // MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3Rv
// SIG // bjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWlj
// SIG // cm9zb2Z0IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEmMCQG
// SIG // A1UECxMdVGhhbGVzIFRTUyBFU046MzFDNS0zMEJBLTdD
// SIG // OTExJTAjBgNVBAMTHE1pY3Jvc29mdCBUaW1lLVN0YW1w
// SIG // IFNlcnZpY2WiJQoBATAJBgUrDgMCGgUAAxUAgPXrdH1g
// SIG // 73Y2qb8f7iqKTz8apyOggd4wgdukgdgwgdUxCzAJBgNV
// SIG // BAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYD
// SIG // VQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQg
// SIG // Q29ycG9yYXRpb24xKTAnBgNVBAsTIE1pY3Jvc29mdCBP
// SIG // cGVyYXRpb25zIFB1ZXJ0byBSaWNvMScwJQYDVQQLEx5u
// SIG // Q2lwaGVyIE5UUyBFU046NERFOS0wQzVFLTNFMDkxKzAp
// SIG // BgNVBAMTIk1pY3Jvc29mdCBUaW1lIFNvdXJjZSBNYXN0
// SIG // ZXIgQ2xvY2swDQYJKoZIhvcNAQEFBQACBQDgxK5iMCIY
// SIG // DzIwMTkwNzAyMDAwODAyWhgPMjAxOTA3MDMwMDA4MDJa
// SIG // MHYwPAYKKwYBBAGEWQoEATEuMCwwCgIFAODErmICAQAw
// SIG // CQIBAAIBDwIB/zAHAgEAAgIZ5DAKAgUA4MX/4gIBADA2
// SIG // BgorBgEEAYRZCgQCMSgwJjAMBgorBgEEAYRZCgMBoAow
// SIG // CAIBAAIDFuNgoQowCAIBAAIDB6EgMA0GCSqGSIb3DQEB
// SIG // BQUAA4IBAQAmIle7lDf21qiRI4Sx1tWhv1VK9mE2WTQp
// SIG // gC6Xzi42ab4G+m2n70owcP4jiC2G/K8vvL4jc3LrzulL
// SIG // e/vfS0DgtOSxxCe24HaAP2taX2ByxwpPYqE3VTul48lZ
// SIG // 2sTvJbdS/IpSGzIEms4dYJ+uXSzZnJ/d/WhmbVr2oX0o
// SIG // 9CS8RlgTaAH7yxEXpQN5x1E5+UsepdkSK9hbVZ9jBW5G
// SIG // 9omarW1iH5M8Zs+c4kSStsHSEhPxPRTJr/1xp4jRGrHD
// SIG // MRMoThgnJAYITphHih5Lgi/IJxHS9VBIAtnLfogo8N/P
// SIG // i/Tagbc5S0kmGPjUszFcROry3Q04bAEyky0I3bGM4F4N
// SIG // MYIC9TCCAvECAQEwgZMwfDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAg
// SIG // UENBIDIwMTACEzMAAADNpts4r70tQQAAAAAAAM0wDQYJ
// SIG // YIZIAWUDBAIBBQCgggEyMBoGCSqGSIb3DQEJAzENBgsq
// SIG // hkiG9w0BCRABBDAvBgkqhkiG9w0BCQQxIgQg58uXXqmo
// SIG // NC5job7VWS16t/ZCt/Fu4dGT/U0D13ZZDwMwgeIGCyqG
// SIG // SIb3DQEJEAIMMYHSMIHPMIHMMIGxBBSA9et0fWDvdjap
// SIG // vx/uKopPPxqnIzCBmDCBgKR+MHwxCzAJBgNVBAYTAlVT
// SIG // MRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdS
// SIG // ZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9y
// SIG // YXRpb24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0
// SIG // YW1wIFBDQSAyMDEwAhMzAAAAzabbOK+9LUEAAAAAAADN
// SIG // MBYEFOt7Gs88TspgoFniSSP+c+lX3NxJMA0GCSqGSIb3
// SIG // DQEBCwUABIIBAGNoWxU8Zj/tnhwVr83deXyJt8OwJnON
// SIG // Koi2sSXLqCQsiFoIYFJ+bHew2RUW7diT7KKRUrs/ni+r
// SIG // FyzmB4QzsJysJKjQt9VTXYiEWiZu1M2jEV64zmw5Djcp
// SIG // mvQT1RJJo2nQKuROCJKOa1IYIihaKcPlZjpKKwS3Xg60
// SIG // EyHoxYRSC+deYaV/zQNtEPe6Vy1ess128Hk9DJnwLky1
// SIG // GDPOgoTeJ8gPMlouj6tadIB5j3fgQs4qtO7t6qpXZHKd
// SIG // 47anwVdlYrlVpiqIsYhEVRukrHUUS7MSO6le0QSbVdNQ
// SIG // 6xyxnkXcCFvtGHeW20UQG/T3mv9WvDydCgnw2nLCbbN545Q=
// SIG // End signature block
