"use strict";
var fs = require('fs');
var path = require('path');
var q_1 = require('q');
var findFile_1 = require('./findFile');
var serviceModuleInfo_1 = require('./serviceModuleInfo');
var common_1 = require('./common');
var libraryResourceStrings_1 = require('./libraryResourceStrings');
// Map of Service name -> Full service module info file path.
var knownServicePaths = {};
/**
* Locate and load service module info for the given service.
* @param: service - service module name, resolvable to a .json or .js file.
*
* To locate the service module info, ServiceHub will do the following:
*   * If servicehub.config.json does not exist or it doesn't have the services folder, servicehub will fail to locate the service.
*   * ServiceHub will search for '*\<service>.servicehub.service.json' in the services folder.
*   * If it cannot find it there, it'll use discoveryServices to delegate search to them.
*   * If the service module info is found, ServiceHub will load it by using fs.readFile and JSON.Parse
*/
function getServiceModuleInfo(logger, service, config) {
    if (!logger) {
        throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('logger'));
    }
    if (!service) {
        throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('service'));
    }
    if (!config) {
        throw new common_1.ServiceHubError(common_1.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('config'));
    }
    if (!config.servicesFolderPath) {
        throw new common_1.ServiceHubError(common_1.ErrorKind.ConfigurationError, libraryResourceStrings_1.LibraryResourceStrings.noFolderSpecified);
    }
    // Check known locations first and ensure the file still exist there
    var knownServicePath = knownServicePaths[service];
    if (knownServicePath) {
        logger.info("Service module info for '" + service + "' has been loaded before from '" + knownServicePath + "'. Trying to load it from there again.");
        return q_1.ninvoke(fs, 'access', knownServicePath, fs.R_OK)
            .then(function () { return loadServiceModuleInfo(logger, service, knownServicePath); }, function (err) {
            logger.info("Service module info for '" + service + "' is missing at '" + knownServicePath + "'. Trying to discover the service again.");
            delete knownServicePaths[service];
            return getServiceModuleInfo(logger, service, config);
        });
    }
    // Keep in sync with VsixDiscoveryService, \src\CoreClr\services\ServiceHubVsixServiceDiscovery\Service\DiscoveryService.cs
    var filePattern = '*/' + service + '.servicehub.service.json';
    return findFile_1.default(filePattern, { cwd: config.servicesFolderPath, nodir: true, fileSelector: serviceModuleFileSelector })
        .then(function (filePath) { return filePath || !config.discoveryHelper ? { filePath: filePath } : config.discoveryHelper(service); })
        .then(function (discoveryResult) {
        if (!discoveryResult || !discoveryResult.filePath) {
            var message = discoveryResult && discoveryResult.error ?
                libraryResourceStrings_1.LibraryResourceStrings.cannotFindServiceModuleFileWithReason(filePattern, config.servicesFolderPath, discoveryResult.error.message) :
                libraryResourceStrings_1.LibraryResourceStrings.cannotFindServiceModuleFile(filePattern, config.servicesFolderPath);
            throw new common_1.ServiceHubError(common_1.ErrorKind.ServiceModuleInfoNotFound, message);
        }
        return loadServiceModuleInfo(logger, service, discoveryResult.filePath);
    });
}
exports.getServiceModuleInfo = getServiceModuleInfo;
function serviceModuleFileSelector(files) {
    if (!files || files.length === 0) {
        return null;
    }
    for (var _i = 0, files_1 = files; _i < files_1.length; _i++) {
        var file = files_1[_i];
        if (path.extname(file) === '.json') {
            return file;
        }
    }
    return files[0];
}
function loadServiceModuleInfo(logger, service, filePath) {
    logger.info(libraryResourceStrings_1.LibraryResourceStrings.loadingServiceModule(filePath, service));
    // Use fs.readFile + JSON.parse to load service module instead of require() so:
    // 1. Only JSON format is supported;
    // 2. require() caches the files it loads. We add name and serviceBaseDirectory properties to the object after loading.
    //    The next time when we load the same service module info file, require() will return the cached instance
    //    with the name and serviceBaseDirectory set on it already, and we'll reject it.
    // 3. Service module info load is async and doesn't block the controller.
    // 4. The load happens from the specified path only. Node module discovery is not needed here.
    return common_1.parseFileAsJson(filePath)
        .then(function (serviceModule) {
        if (serviceModule.name) {
            var message = libraryResourceStrings_1.LibraryResourceStrings.serviceInfoInvalid(filePath, service, 'name');
            throw new common_1.ServiceHubError(common_1.ErrorKind.ServiceModuleInfoInvalidPropertyError, message);
        }
        if (serviceModule.serviceBaseDirectory) {
            var message = libraryResourceStrings_1.LibraryResourceStrings.serviceInfoInvalid(filePath, service, 'serviceBaseDirectory');
            throw new common_1.ServiceHubError(common_1.ErrorKind.ServiceModuleInfoInvalidPropertyError, message);
        }
        if (serviceModule.hostId && serviceModule.hostId.indexOf(' ') >= 0) {
            var message = libraryResourceStrings_1.LibraryResourceStrings.serviceInfoInvalidSpaceInHostId(filePath, service, serviceModule.hostId);
            throw new common_1.ServiceHubError(common_1.ErrorKind.ServiceModuleInfoInvalidPropertyError, message);
        }
        serviceModule.name = service;
        serviceModule.serviceBaseDirectory = path.dirname(filePath);
        var result = new serviceModuleInfo_1.ServiceModuleInfo(serviceModule);
        // Cache the service module info file path.
        knownServicePaths[service] = filePath;
        return result;
    })
        .catch(function (err) {
        throw new common_1.ServiceHubError(common_1.ErrorKind.ServiceModuleInfoLoadError, libraryResourceStrings_1.LibraryResourceStrings.cannotResolveServiceModuleInfo(filePath, service, err.message));
    });
}

//# sourceMappingURL=serviceDiscovery.js.map
// SIG // Begin signature block
// SIG // MIIiRgYJKoZIhvcNAQcCoIIiNzCCIjMCAQExDzANBglg
// SIG // hkgBZQMEAgEFADB3BgorBgEEAYI3AgEEoGkwZzAyBgor
// SIG // BgEEAYI3AgEeMCQCAQEEEBDgyQbOONQRoqMAEEvTUJAC
// SIG // AQACAQACAQACAQACAQAwMTANBglghkgBZQMEAgEFAAQg
// SIG // pXIJ2UbK6na9VQo76rqSa9i4CuRcRgOCA7CPflOgeHKg
// SIG // ggtyMIIE+jCCA+KgAwIBAgITMwAAAs58ms59kF7StwAA
// SIG // AAACzjANBgkqhkiG9w0BAQsFADB+MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBT
// SIG // aWduaW5nIFBDQSAyMDEwMB4XDTE5MDUwMjIxMjU0MloX
// SIG // DTIwMDUwMjIxMjU0MlowdDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEeMBwGA1UEAxMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
// SIG // owhbo0atJEL/giHpwS2stH0eY6h2vghqYkLwa9LT5G5k
// SIG // q5QvKepTYRPV5tH0f5xTRTFTuZopGubS+mDvUkm5CF3C
// SIG // 0w4hucK3Mr8ad7l2yeEmZ7Tikgro6rzUApaNX3pEmKmt
// SIG // 2cPaAJ5XnM4DgC6Q5VV6gX3C/NO+h7ss51xKRb662Hjp
// SIG // y4s1p1weopn+gyWVpQwGNgGwTAcSguLN7XpFUl1esVAC
// SIG // GzvS4MIb8qsl1+aXWvgNUMmzSAwlIHDRnmd6HejS5HEM
// SIG // m4Ye2bYmRKb6Bc7I4hsi3pIm304LK4vSAdLH2HElS7VY
// SIG // mofTuiU2MZUNWaz+14kv5R4yRn+ojSNQUQIDAQABo4IB
// SIG // eTCCAXUwHwYDVR0lBBgwFgYKKwYBBAGCNz0GAQYIKwYB
// SIG // BQUHAwMwHQYDVR0OBBYEFHM/DO7M0NXkSmSFJIwE5oKt
// SIG // gMgPMFAGA1UdEQRJMEekRTBDMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEWMBQG
// SIG // A1UEBRMNMjMwODY1KzQ1NDI0MzAfBgNVHSMEGDAWgBTm
// SIG // /F97uyIAWORyTrX0IXQjMubvrDBWBgNVHR8ETzBNMEug
// SIG // SaBHhkVodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NybC9wcm9kdWN0cy9NaWNDb2RTaWdQQ0FfMjAxMC0w
// SIG // Ny0wNi5jcmwwWgYIKwYBBQUHAQEETjBMMEoGCCsGAQUF
// SIG // BzAChj5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NlcnRzL01pY0NvZFNpZ1BDQV8yMDEwLTA3LTA2LmNy
// SIG // dDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IB
// SIG // AQAcPuovja14/IUbUPK1RjGlfqowbC45KppnIgLxztfM
// SIG // +gbG8nz62UhMR8MjrPM2gyl+dwxqSm5kmAqQ0zeU/lTA
// SIG // z3wH6dvaa8gWwR5gCRYCu0z/s1sR7pjXlQpEILf6QP3t
// SIG // PTg+CEhYbIYxUH+Hr8SQisaxZWCC/rTywdBanMHIMQ7L
// SIG // LZbYaEjmkXmjSzIq4wGjaRRvoJ1cQUXLe0BHgaycOCgk
// SIG // TtJMON5OUqZB3C4rqcjlQmp8/THwHeSGwZgVPYQJCNZi
// SIG // j3V24F1nEKDwiazXleVkJ6smBpa4wBYbPFGR/VzTW49o
// SIG // gr51TH3e4IENba6DsZIhnrk2YP7s/xvsbKdGMIIGcDCC
// SIG // BFigAwIBAgIKYQxSTAAAAAAAAzANBgkqhkiG9w0BAQsF
// SIG // ADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMp
// SIG // TWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9y
// SIG // aXR5IDIwMTAwHhcNMTAwNzA2MjA0MDE3WhcNMjUwNzA2
// SIG // MjA1MDE3WjB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMK
// SIG // V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
// SIG // A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYD
// SIG // VQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBDQSAy
// SIG // MDEwMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
// SIG // AQEA6Q5kUHlntcTj/QkATJ6UrPdWaOpE2M/FWE+ppXZ8
// SIG // bUW60zmStKQe+fllguQX0o/9RJwI6GWTzixVhL99COMu
// SIG // K6hBKxi3oktuSUxrFQfe0dLCiR5xlM21f0u0rwjYzIjW
// SIG // axeUOpPOJj/s5v40mFfVHV1J9rIqLtWFu1k/+JC0K4N0
// SIG // yiuzO0bj8EZJwRdmVMkcvR3EVWJXcvhnuSUgNN5dpqWV
// SIG // XqsogM3Vsp7lA7Vj07IUyMHIiiYKWX8H7P8O7YASNUwS
// SIG // pr5SW/Wm2uCLC0h31oVH1RC5xuiq7otqLQVcYMa0Kluc
// SIG // IxxfReMaFB5vN8sZM4BqiU2jamZjeJPVMM+VHwIDAQAB
// SIG // o4IB4zCCAd8wEAYJKwYBBAGCNxUBBAMCAQAwHQYDVR0O
// SIG // BBYEFOb8X3u7IgBY5HJOtfQhdCMy5u+sMBkGCSsGAQQB
// SIG // gjcUAgQMHgoAUwB1AGIAQwBBMAsGA1UdDwQEAwIBhjAP
// SIG // BgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2VsuP
// SIG // 6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeG
// SIG // RWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
// SIG // L3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEwLTA2LTIz
// SIG // LmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKG
// SIG // Pmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2Vy
// SIG // dHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3J0MIGd
// SIG // BgNVHSAEgZUwgZIwgY8GCSsGAQQBgjcuAzCBgTA9Bggr
// SIG // BgEFBQcCARYxaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
// SIG // L1BLSS9kb2NzL0NQUy9kZWZhdWx0Lmh0bTBABggrBgEF
// SIG // BQcCAjA0HjIgHQBMAGUAZwBhAGwAXwBQAG8AbABpAGMA
// SIG // eQBfAFMAdABhAHQAZQBtAGUAbgB0AC4gHTANBgkqhkiG
// SIG // 9w0BAQsFAAOCAgEAGnTvV08pe8QWhXi4UNMi/AmdrIKX
// SIG // +DT/KiyXlRLl5L/Pv5PI4zSp24G43B4AvtI1b6/lf3mV
// SIG // d+UC1PHr2M1OHhthosJaIxrwjKhiUUVnCOM/PB6T+DCF
// SIG // F8g5QKbXDrMhKeWloWmMIpPMdJjnoUdD8lOswA8waX/+
// SIG // 0iUgbW9h098H1dlyACxphnY9UdumOUjJN2FtB91TGcun
// SIG // 1mHCv+KDqw/ga5uV1n0oUbCJSlGkmmzItx9KGg5pqdfc
// SIG // wX7RSXCqtq27ckdjF/qm1qKmhuyoEESbY7ayaYkGx0aG
// SIG // ehg/6MUdIdV7+QIjLcVBy78dTMgW77Gcf/wiS0mKbhXj
// SIG // pn92W9FTeZGFndXS2z1zNfM8rlSyUkdqwKoTldKOEdqZ
// SIG // Z14yjPs3hdHcdYWch8ZaV4XCv90Nj4ybLeu07s8n07Ve
// SIG // afqkFgQBpyRnc89NT7beBVaXevfpUk30dwVPhcbYC/GO
// SIG // 7UIJ0Q124yNWeCImNr7KsYxuqh3khdpHM2KPpMmRM19x
// SIG // HkCvmGXJIuhCISWKHC1g2TeJQYkqFg/XYTyUaGBS79ZH
// SIG // maCAQO4VgXc+nOBTGBpQHTiVmx5mMxMnORd4hzbOTsNf
// SIG // svU9R1O24OXbC2E9KteSLM43Wj5AQjGkHxAIwlacvyRd
// SIG // UQKdannSF9PawZSOB3slcUSrBmrm1MbfI5qWdcUxghYs
// SIG // MIIWKAIBATCBlTB+MQswCQYDVQQGEwJVUzETMBEGA1UE
// SIG // CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEe
// SIG // MBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgw
// SIG // JgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBD
// SIG // QSAyMDEwAhMzAAACznyazn2QXtK3AAAAAALOMA0GCWCG
// SIG // SAFlAwQCAQUAoIGuMBkGCSqGSIb3DQEJAzEMBgorBgEE
// SIG // AYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3
// SIG // AgEVMC8GCSqGSIb3DQEJBDEiBCA3IILm3g9bUmB6c8aG
// SIG // QrEGEszXIuxr9ABW+OzCgTR4fTBCBgorBgEEAYI3AgEM
// SIG // MTQwMqAUgBIATQBpAGMAcgBvAHMAbwBmAHShGoAYaHR0
// SIG // cDovL3d3dy5taWNyb3NvZnQuY29tMA0GCSqGSIb3DQEB
// SIG // AQUABIIBAEz1B84kXV8r/rLGPllASkpaZjgBwMjMxI6O
// SIG // RqbR7c0emiUR3a1YVLKBcSpIZ8MDIrHFwP1cH6wAmtIw
// SIG // w2W2W0915XAqj3ue17oajGXxfPwiQ+MYqy2vVtqCtNrN
// SIG // G0/VJmEj1pmqmmywFU78y4C0NUlnhk7rnxhWBVnNatcm
// SIG // EsUtcc6x2Sg9XSYu3Bt50fgTq6PNX45JG0JlRoR97rAx
// SIG // T0eJa+k1kAuRMWarpW2kMHOLc6CxhP5IfZoeahOE/scH
// SIG // g+E6LN0YSpGUr6SXmVt2F6HUEB2Q9y5lYXLKfx1shqtm
// SIG // AJFhvAfYPNtp35+uFMOL8P7hkG7LyWC4NdgEWAcswlih
// SIG // ghO2MIITsgYKKwYBBAGCNwMDATGCE6IwghOeBgkqhkiG
// SIG // 9w0BBwKgghOPMIITiwIBAzEPMA0GCWCGSAFlAwQCAQUA
// SIG // MIIBWAYLKoZIhvcNAQkQAQSgggFHBIIBQzCCAT8CAQEG
// SIG // CisGAQQBhFkKAwEwMTANBglghkgBZQMEAgEFAAQgD4T4
// SIG // GxyY6Xr4TxLlwk63BOn9Xekza6xSf8XJhnLXfW4CBl0a
// SIG // LwtmpBgTMjAxOTA3MDEyMTEyNTQuMzE2WjAHAgEBgAIB
// SIG // 9KCB1KSB0TCBzjELMAkGA1UEBhMCVVMxEzARBgNVBAgT
// SIG // Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAc
// SIG // BgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcG
// SIG // A1UECxMgTWljcm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRv
// SIG // IFJpY28xJjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOjdE
// SIG // MkUtMzc4Mi1CMEY3MSUwIwYDVQQDExxNaWNyb3NvZnQg
// SIG // VGltZS1TdGFtcCBTZXJ2aWNloIIPHjCCBnEwggRZoAMC
// SIG // AQICCmEJgSoAAAAAAAIwDQYJKoZIhvcNAQELBQAwgYgx
// SIG // CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9u
// SIG // MRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNy
// SIG // b3NvZnQgQ29ycG9yYXRpb24xMjAwBgNVBAMTKU1pY3Jv
// SIG // c29mdCBSb290IENlcnRpZmljYXRlIEF1dGhvcml0eSAy
// SIG // MDEwMB4XDTEwMDcwMTIxMzY1NVoXDTI1MDcwMTIxNDY1
// SIG // NVowfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UEAxMd
// SIG // TWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAwggEi
// SIG // MA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCpHQ28
// SIG // dxGKOiDs/BOX9fp/aZRrdFQQ1aUKAIKF++18aEssX8XD
// SIG // 5WHCdrc+Zitb8BVTJwQxH0EbGpUdzgkTjnxhMFmxMEQP
// SIG // 8WCIhFRDDNdNuDgIs0Ldk6zWczBXJoKjRQ3Q6vVHgc2/
// SIG // JGAyWGBG8lhHhjKEHnRhZ5FfgVSxz5NMksHEpl3RYRNu
// SIG // KMYa+YaAu99h/EbBJx0kZxJyGiGKr0tkiVBisV39dx89
// SIG // 8Fd1rL2KQk1AUdEPnAY+Z3/1ZsADlkR+79BL/W7lmsqx
// SIG // qPJ6Kgox8NpOBpG2iAg16HgcsOmZzTznL0S6p/TcZL2k
// SIG // AcEgCZN4zfy8wMlEXV4WnAEFTyJNAgMBAAGjggHmMIIB
// SIG // 4jAQBgkrBgEEAYI3FQEEAwIBADAdBgNVHQ4EFgQU1WM6
// SIG // XIoxkPNDe3xGG8UzaFqFbVUwGQYJKwYBBAGCNxQCBAwe
// SIG // CgBTAHUAYgBDAEEwCwYDVR0PBAQDAgGGMA8GA1UdEwEB
// SIG // /wQFMAMBAf8wHwYDVR0jBBgwFoAU1fZWy4/oolxiaNE9
// SIG // lJBb186aGMQwVgYDVR0fBE8wTTBLoEmgR4ZFaHR0cDov
// SIG // L2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwvcHJvZHVj
// SIG // dHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3JsMFoG
// SIG // CCsGAQUFBwEBBE4wTDBKBggrBgEFBQcwAoY+aHR0cDov
// SIG // L3d3dy5taWNyb3NvZnQuY29tL3BraS9jZXJ0cy9NaWNS
// SIG // b29DZXJBdXRfMjAxMC0wNi0yMy5jcnQwgaAGA1UdIAEB
// SIG // /wSBlTCBkjCBjwYJKwYBBAGCNy4DMIGBMD0GCCsGAQUF
// SIG // BwIBFjFodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vUEtJ
// SIG // L2RvY3MvQ1BTL2RlZmF1bHQuaHRtMEAGCCsGAQUFBwIC
// SIG // MDQeMiAdAEwAZQBnAGEAbABfAFAAbwBsAGkAYwB5AF8A
// SIG // UwB0AGEAdABlAG0AZQBuAHQALiAdMA0GCSqGSIb3DQEB
// SIG // CwUAA4ICAQAH5ohRDeLG4Jg/gXEDPZ2joSFvs+umzPUx
// SIG // vs8F4qn++ldtGTCzwsVmyWrf9efweL3HqJ4l4/m87WtU
// SIG // VwgrUYJEEvu5U4zM9GASinbMQEBBm9xcF/9c+V4XNZgk
// SIG // Vkt070IQyK+/f8Z/8jd9Wj8c8pl5SpFSAK84Dxf1L3mB
// SIG // ZdmptWvkx872ynoAb0swRCQiPM/tA6WWj1kpvLb9BOFw
// SIG // nzJKJ/1Vry/+tuWOM7tiX5rbV0Dp8c6ZZpCM/2pif93F
// SIG // SguRJuI57BlKcWOdeyFtw5yjojz6f32WapB4pm3S4Zz5
// SIG // Hfw42JT0xqUKloakvZ4argRCg7i1gJsiOCC1JeVk7Pf0
// SIG // v35jWSUPei45V3aicaoGig+JFrphpxHLmtgOR5qAxdDN
// SIG // p9DvfYPw4TtxCd9ddJgiCGHasFAeb73x4QDf5zEHpJM6
// SIG // 92VHeOj4qEir995yfmFrb3epgcunCaw5u+zGy9iCtHLN
// SIG // HfS4hQEegPsbiSpUObJb2sgNVZl6h3M7COaYLeqN4DMu
// SIG // Ein1wC9UJyH3yKxO2ii4sanblrKnQqLJzxlBTeCG+Sqa
// SIG // oxFmMNO7dDJL32N79ZmKLxvHIa9Zta7cRDyXUHHXodLF
// SIG // VeNp3lfB0d4wwP3M5k37Db9dT+mdHhk4L7zPWAUu7w2g
// SIG // UDXa7wknHNWzfjUeCLraNtvTX4/edIhJEjCCBPUwggPd
// SIG // oAMCAQICEzMAAADPTBCl2yy3BVIAAAAAAM8wDQYJKoZI
// SIG // hvcNAQELBQAwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgT
// SIG // Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAc
// SIG // BgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQG
// SIG // A1UEAxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIw
// SIG // MTAwHhcNMTgwODIzMjAyNjI3WhcNMTkxMTIzMjAyNjI3
// SIG // WjCBzjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMg
// SIG // TWljcm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28x
// SIG // JjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOjdEMkUtMzc4
// SIG // Mi1CMEY3MSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1T
// SIG // dGFtcCBTZXJ2aWNlMIIBIjANBgkqhkiG9w0BAQEFAAOC
// SIG // AQ8AMIIBCgKCAQEAsx8ZWqwk9hFhmdV0An6Q3U8IMj1T
// SIG // Zh5POuzqNhOwAmX80bNjzhbLVtnTlkiVh05GUJAZS+ni
// SIG // ilaMuQNsxGbv0TehQxRtEOw8CuPG/hfImNuDCzJwr5Ma
// SIG // 0MZZ58d8Wzf6gbMuae0Cwp/iURroivmj0lVjOFS0+KHk
// SIG // vPMykOmoO9/u2t3TVdJ4DZo4E1Rk2Pl1wIrfyo2Lz1vH
// SIG // Zb9OFzyRZv3MvCgu1vCRieUjujWqzuaFCuNlCsceudGz
// SIG // LeHCtvRBqsWoVEj8otosZHqA+U0bnDYl3O2MuLbmXoix
// SIG // qztoYZotEf1HYADJnL2R0XGCkhb3WGENQTn0kgXA4x3L
// SIG // j2AcMQIDAQABo4IBGzCCARcwHQYDVR0OBBYEFOFibOMi
// SIG // tjpAsmYb4tD7E962x8vwMB8GA1UdIwQYMBaAFNVjOlyK
// SIG // MZDzQ3t8RhvFM2hahW1VMFYGA1UdHwRPME0wS6BJoEeG
// SIG // RWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
// SIG // L3Byb2R1Y3RzL01pY1RpbVN0YVBDQV8yMDEwLTA3LTAx
// SIG // LmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKG
// SIG // Pmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2Vy
// SIG // dHMvTWljVGltU3RhUENBXzIwMTAtMDctMDEuY3J0MAwG
// SIG // A1UdEwEB/wQCMAAwEwYDVR0lBAwwCgYIKwYBBQUHAwgw
// SIG // DQYJKoZIhvcNAQELBQADggEBABESPeCskOJ/JDbE8jBb
// SIG // UZScpH7ouj3W0i6l/zCr9f9DwzbcqbK4FLLcGLdHJpCf
// SIG // lrhHME5KvVqyWCf46KgIrWoX9iEboMNlo/0XubEc4qlk
// SIG // hcyDWshXnAcN3fqGT4FJesRq3pA/GzozTOynwiVtJI3z
// SIG // OpMyFxyV3gfGnkix9JRQJmbF+XAiGoS6iP5RkxU7jHuO
// SIG // c/r0FeSY4CMRCru584BBePPeoRO20CvMKsGPXrf3e33i
// SIG // JjNLci1q5ShirPt7mDp8+sD691hj7Hf4hED0eB6VjeGq
// SIG // 32irqssMkeAG6s97UjZtZalxZ+6/yQBWL57myuyi0Dr8
// SIG // 9UJkOA/Q/AkZ2xShggOsMIIClAIBATCB/qGB1KSB0TCB
// SIG // zjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0
// SIG // b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1p
// SIG // Y3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMgTWlj
// SIG // cm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28xJjAk
// SIG // BgNVBAsTHVRoYWxlcyBUU1MgRVNOOjdEMkUtMzc4Mi1C
// SIG // MEY3MSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFt
// SIG // cCBTZXJ2aWNloiUKAQEwCQYFKw4DAhoFAAMVAIk+0OTQ
// SIG // MsOFXV0ilrLeLdGvuJfdoIHeMIHbpIHYMIHVMQswCQYD
// SIG // VQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4G
// SIG // A1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
// SIG // IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQg
// SIG // T3BlcmF0aW9ucyBQdWVydG8gUmljbzEnMCUGA1UECxMe
// SIG // bkNpcGhlciBOVFMgRVNOOjRERTktMEM1RS0zRTA5MSsw
// SIG // KQYDVQQDEyJNaWNyb3NvZnQgVGltZSBTb3VyY2UgTWFz
// SIG // dGVyIENsb2NrMA0GCSqGSIb3DQEBBQUAAgUA4MStpjAi
// SIG // GA8yMDE5MDcwMTE2MDQ1NFoYDzIwMTkwNzAyMTYwNDU0
// SIG // WjBzMDkGCisGAQQBhFkKBAExKzApMAoCBQDgxK2mAgEA
// SIG // MAYCAQACASowBwIBAAICGdwwCgIFAODF/yYCAQAwNgYK
// SIG // KwYBBAGEWQoEAjEoMCYwDAYKKwYBBAGEWQoDAaAKMAgC
// SIG // AQACAxbjYKEKMAgCAQACAwehIDANBgkqhkiG9w0BAQUF
// SIG // AAOCAQEARiOg9MuKX+idrFgTEWjWWcvueVQIN0qewIRV
// SIG // 6P0yLV4lzwqx9Hc2k5mDMsfC2oJMQxQQ2HQ68a1vG2ka
// SIG // d/oDPz7O6HVGkQ+fVnwxwjk7Dxsho+kdfkS4ue04AUdd
// SIG // knx+Zd+ZJEE1ASmG25+nnzZ/pEhUrsvWep3KlakS/KHu
// SIG // N0efVHwuccZ5khk9ry1zCKDioclYTfJoY1BXwEC5fm5Y
// SIG // ziec7GkKG6Zp6+ZJEHeR9FY6b6owCp7bFpHgKKET9RhA
// SIG // 9Lle1bTQmB2wpSb4SsipJlXJCWnL7b2EHULoIB8qTg0r
// SIG // 7JtACsOXwHVWWBKXOb4HmXBSJ+cF2ocFnzaeHmDKIDGC
// SIG // AvUwggLxAgEBMIGTMHwxCzAJBgNVBAYTAlVTMRMwEQYD
// SIG // VQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25k
// SIG // MR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24x
// SIG // JjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1lLVN0YW1wIFBD
// SIG // QSAyMDEwAhMzAAAAz0wQpdsstwVSAAAAAADPMA0GCWCG
// SIG // SAFlAwQCAQUAoIIBMjAaBgkqhkiG9w0BCQMxDQYLKoZI
// SIG // hvcNAQkQAQQwLwYJKoZIhvcNAQkEMSIEIKd0s0og0IFX
// SIG // V/BJ/Tac1tzlGwdMOasqEWsxJRebPfSHMIHiBgsqhkiG
// SIG // 9w0BCRACDDGB0jCBzzCBzDCBsQQUiT7Q5NAyw4VdXSKW
// SIG // st4t0a+4l90wgZgwgYCkfjB8MQswCQYDVQQGEwJVUzET
// SIG // MBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVk
// SIG // bW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0
// SIG // aW9uMSYwJAYDVQQDEx1NaWNyb3NvZnQgVGltZS1TdGFt
// SIG // cCBQQ0EgMjAxMAITMwAAAM9MEKXbLLcFUgAAAAAAzzAW
// SIG // BBTevvnEHPa8uyoRe/wXNUWZe0J9SDANBgkqhkiG9w0B
// SIG // AQsFAASCAQAycLQny2Iy3gduIAXHoksAOeCNh+84VFP3
// SIG // CNW+KxwLau1iTlDKt458wTTXZMTck4tL8kuqQ4E0IGq3
// SIG // VIPRZutwd7IUMQ+xQbQEY6tTdcwD7DQ25imsfLH4zzEm
// SIG // XZAb2MjU5Cflu4m8CKPUQAjkKepOl+0ovQsyGlJxvlLy
// SIG // NDEiK0u3VB9I0jr3rezeNHEBrltb0lciaNKEU5WYxP78
// SIG // NCEu/Oq40kgvtTYbL4KlAZnbWvX4uibakXPeedUgKpBa
// SIG // vZhAoplTCpq8Gl3HnwQpy2vvIjGuxV12k8sD/3R5RNnl
// SIG // RXqOg4pZsXejMfJR/BLbkF7rJiV//2Qtp4G/KIi/UpYY
// SIG // End signature block
