"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var path = require('path');
var fs = require('fs');
var e = require('events');
var q = require('q');
var c = require('./common');
var libraryResourceStrings_1 = require('./libraryResourceStrings');
/**
 * Service Manager
 *
 * Events:
 *     servicesStarted(serviceName, stream) - at least one client has open connection to a service running by this service manager.
 *     servicesEnded - there are no more open connections to any service running by this service manager.
 */
var ServiceManager = (function (_super) {
    __extends(ServiceManager, _super);
    function ServiceManager(logger) {
        _super.call(this);
        this.endPoints = [];
        this.streams = [];
        this.logger = logger;
    }
    ServiceManager.prototype.startService = function (serviceModuleInfo) {
        if (serviceModuleInfo == null) {
            throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('serviceModuleInfo'));
        }
        this.logger.info(libraryResourceStrings_1.LibraryResourceStrings.startingService(serviceModuleInfo.name));
        return this.startServiceObject(serviceModuleInfo);
    };
    ServiceManager.loadFunction = function (moduleName, constructor) {
        if (!moduleName) {
            throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('moduleName'));
        }
        if (!constructor) {
            throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('constructor'));
        }
        var checkFile = path.isAbsolute(moduleName) ? q.ninvoke(fs, 'access', moduleName, fs.F_OK | fs.R_OK) : q.resolve(null);
        return checkFile.then(function () {
            var nodeModule = require(moduleName);
            var nodeModuleExport = nodeModule[constructor];
            if (typeof nodeModuleExport !== 'function') {
                var message = libraryResourceStrings_1.LibraryResourceStrings.variableMustBeFunction(constructor, moduleName, typeof nodeModuleExport);
                throw new c.ServiceHubError(c.ErrorKind.InvalidOperation, message);
            }
            return function () {
                var params = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    params[_i - 0] = arguments[_i];
                }
                var result = Object.create(nodeModuleExport.prototype);
                result.constructor.apply(result, params);
                return result;
            };
        });
    };
    ServiceManager.prototype.startServiceObject = function (serviceModuleInfo) {
        var _this = this;
        var name = serviceModuleInfo.name;
        var serviceEntryPoint = (serviceModuleInfo.entryPoint);
        if (!serviceEntryPoint.scriptPath) {
            var message = libraryResourceStrings_1.LibraryResourceStrings.serviceEntryPointFileNameNotFound(name);
            this.logger.error(message);
            throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, message);
        }
        if (!serviceEntryPoint.constructorFunction) {
            var message = libraryResourceStrings_1.LibraryResourceStrings.serviceEntryPointConstructorNotFound(name);
            this.logger.error(message);
            throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, message);
        }
        var moduleName = ServiceManager.getFullPath(serviceModuleInfo, serviceEntryPoint.scriptPath);
        var constructor = serviceEntryPoint.constructorFunction;
        var serviceOptions = {
            name: name,
            logger: this.logger,
            noStreamTrace: true,
        };
        return ServiceManager.loadFunction(moduleName, constructor)
            .catch(function (err) {
            var message = libraryResourceStrings_1.LibraryResourceStrings.loadServiceModuleError(name, err.message);
            _this.logger.error(message);
            return q.reject(new Error(message));
        })
            .then(function (serviceModuleFactory) { return c.serverManager.startService(serviceOptions, function (stream, logger) {
            logger.info(libraryResourceStrings_1.LibraryResourceStrings.connectedToService(name));
            // Keep a reference to the service stream so it is not garbage collected.
            if (_this.streams.push(stream) === 1) {
                _this.emit(ServiceManager.ServicesStartedEvent, name, stream);
            }
            var serviceModuleInstance = null;
            stream.once('end', function () {
                logger.info("A stream to " + name + " ended.");
                var disposal = q.resolve(null);
                if (serviceModuleInstance && typeof serviceModuleInstance.disposeAsync === 'function') {
                    var instanceName_1 = serviceModuleInstance.constructor.name ? serviceModuleInstance.constructor.name + ' ' : '';
                    disposal = disposal
                        .then(function () { return serviceModuleInstance.disposeAsync(); })
                        .catch(function (reason) {
                        logger.error(libraryResourceStrings_1.LibraryResourceStrings.disposeServiceError(instanceName_1, name, reason.stack || reason.message || reason));
                        return null;
                    });
                }
                disposal.finally(function () {
                    var index = _this.streams.indexOf(stream);
                    if (index >= 0) {
                        _this.streams.splice(index, 1);
                    }
                    if (_this.streams.length === 0) {
                        _this.emit(ServiceManager.ServicesEndedEvent);
                    }
                }).done();
            });
            try {
                var svcs = {
                    logger: _this.logger,
                };
                serviceModuleInstance = serviceModuleFactory.call(_this, stream, svcs);
            }
            catch (err) {
                logger.error(libraryResourceStrings_1.LibraryResourceStrings.createServiceInstanceError(name, err.stack || err.message || err));
                stream.end();
                return;
            }
        }); });
    };
    ServiceManager.getFullPath = function (smi, fileNameOrPath) {
        if (!fileNameOrPath) {
            throw new c.ServiceHubError(c.ErrorKind.InvalidArgument, libraryResourceStrings_1.LibraryResourceStrings.variableIsNotDefined('fileNameOrPath'));
        }
        var result;
        if (path.isAbsolute(fileNameOrPath)) {
            result = fileNameOrPath;
        }
        else {
            result = smi.serviceBaseDirectory ? path.join(smi.serviceBaseDirectory, fileNameOrPath) : fileNameOrPath;
        }
        return result.replace(/\\/g, '/');
    };
    ServiceManager.ServicesStartedEvent = 'servicesStarted';
    ServiceManager.ServicesEndedEvent = 'servicesEnded';
    return ServiceManager;
}(e.EventEmitter));
exports.ServiceManager = ServiceManager;

//# sourceMappingURL=serviceManager.js.map
// SIG // Begin signature block
// SIG // MIIiRwYJKoZIhvcNAQcCoIIiODCCIjQCAQExDzANBglg
// SIG // hkgBZQMEAgEFADB3BgorBgEEAYI3AgEEoGkwZzAyBgor
// SIG // BgEEAYI3AgEeMCQCAQEEEBDgyQbOONQRoqMAEEvTUJAC
// SIG // AQACAQACAQACAQACAQAwMTANBglghkgBZQMEAgEFAAQg
// SIG // 8L+NeZ53aErkolnHkQijPES2qtoToL1OhbZgh74EN2qg
// SIG // ggtyMIIE+jCCA+KgAwIBAgITMwAAAs58ms59kF7StwAA
// SIG // AAACzjANBgkqhkiG9w0BAQsFADB+MQswCQYDVQQGEwJV
// SIG // UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMH
// SIG // UmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBv
// SIG // cmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBT
// SIG // aWduaW5nIFBDQSAyMDEwMB4XDTE5MDUwMjIxMjU0MloX
// SIG // DTIwMDUwMjIxMjU0MlowdDELMAkGA1UEBhMCVVMxEzAR
// SIG // BgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1v
// SIG // bmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlv
// SIG // bjEeMBwGA1UEAxMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
// SIG // owhbo0atJEL/giHpwS2stH0eY6h2vghqYkLwa9LT5G5k
// SIG // q5QvKepTYRPV5tH0f5xTRTFTuZopGubS+mDvUkm5CF3C
// SIG // 0w4hucK3Mr8ad7l2yeEmZ7Tikgro6rzUApaNX3pEmKmt
// SIG // 2cPaAJ5XnM4DgC6Q5VV6gX3C/NO+h7ss51xKRb662Hjp
// SIG // y4s1p1weopn+gyWVpQwGNgGwTAcSguLN7XpFUl1esVAC
// SIG // GzvS4MIb8qsl1+aXWvgNUMmzSAwlIHDRnmd6HejS5HEM
// SIG // m4Ye2bYmRKb6Bc7I4hsi3pIm304LK4vSAdLH2HElS7VY
// SIG // mofTuiU2MZUNWaz+14kv5R4yRn+ojSNQUQIDAQABo4IB
// SIG // eTCCAXUwHwYDVR0lBBgwFgYKKwYBBAGCNz0GAQYIKwYB
// SIG // BQUHAwMwHQYDVR0OBBYEFHM/DO7M0NXkSmSFJIwE5oKt
// SIG // gMgPMFAGA1UdEQRJMEekRTBDMSkwJwYDVQQLEyBNaWNy
// SIG // b3NvZnQgT3BlcmF0aW9ucyBQdWVydG8gUmljbzEWMBQG
// SIG // A1UEBRMNMjMwODY1KzQ1NDI0MzAfBgNVHSMEGDAWgBTm
// SIG // /F97uyIAWORyTrX0IXQjMubvrDBWBgNVHR8ETzBNMEug
// SIG // SaBHhkVodHRwOi8vY3JsLm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NybC9wcm9kdWN0cy9NaWNDb2RTaWdQQ0FfMjAxMC0w
// SIG // Ny0wNi5jcmwwWgYIKwYBBQUHAQEETjBMMEoGCCsGAQUF
// SIG // BzAChj5odHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
// SIG // L2NlcnRzL01pY0NvZFNpZ1BDQV8yMDEwLTA3LTA2LmNy
// SIG // dDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4IB
// SIG // AQAcPuovja14/IUbUPK1RjGlfqowbC45KppnIgLxztfM
// SIG // +gbG8nz62UhMR8MjrPM2gyl+dwxqSm5kmAqQ0zeU/lTA
// SIG // z3wH6dvaa8gWwR5gCRYCu0z/s1sR7pjXlQpEILf6QP3t
// SIG // PTg+CEhYbIYxUH+Hr8SQisaxZWCC/rTywdBanMHIMQ7L
// SIG // LZbYaEjmkXmjSzIq4wGjaRRvoJ1cQUXLe0BHgaycOCgk
// SIG // TtJMON5OUqZB3C4rqcjlQmp8/THwHeSGwZgVPYQJCNZi
// SIG // j3V24F1nEKDwiazXleVkJ6smBpa4wBYbPFGR/VzTW49o
// SIG // gr51TH3e4IENba6DsZIhnrk2YP7s/xvsbKdGMIIGcDCC
// SIG // BFigAwIBAgIKYQxSTAAAAAAAAzANBgkqhkiG9w0BAQsF
// SIG // ADCBiDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
// SIG // bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoT
// SIG // FU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEyMDAGA1UEAxMp
// SIG // TWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9y
// SIG // aXR5IDIwMTAwHhcNMTAwNzA2MjA0MDE3WhcNMjUwNzA2
// SIG // MjA1MDE3WjB+MQswCQYDVQQGEwJVUzETMBEGA1UECBMK
// SIG // V2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwG
// SIG // A1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYD
// SIG // VQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBDQSAy
// SIG // MDEwMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
// SIG // AQEA6Q5kUHlntcTj/QkATJ6UrPdWaOpE2M/FWE+ppXZ8
// SIG // bUW60zmStKQe+fllguQX0o/9RJwI6GWTzixVhL99COMu
// SIG // K6hBKxi3oktuSUxrFQfe0dLCiR5xlM21f0u0rwjYzIjW
// SIG // axeUOpPOJj/s5v40mFfVHV1J9rIqLtWFu1k/+JC0K4N0
// SIG // yiuzO0bj8EZJwRdmVMkcvR3EVWJXcvhnuSUgNN5dpqWV
// SIG // XqsogM3Vsp7lA7Vj07IUyMHIiiYKWX8H7P8O7YASNUwS
// SIG // pr5SW/Wm2uCLC0h31oVH1RC5xuiq7otqLQVcYMa0Kluc
// SIG // IxxfReMaFB5vN8sZM4BqiU2jamZjeJPVMM+VHwIDAQAB
// SIG // o4IB4zCCAd8wEAYJKwYBBAGCNxUBBAMCAQAwHQYDVR0O
// SIG // BBYEFOb8X3u7IgBY5HJOtfQhdCMy5u+sMBkGCSsGAQQB
// SIG // gjcUAgQMHgoAUwB1AGIAQwBBMAsGA1UdDwQEAwIBhjAP
// SIG // BgNVHRMBAf8EBTADAQH/MB8GA1UdIwQYMBaAFNX2VsuP
// SIG // 6KJcYmjRPZSQW9fOmhjEMFYGA1UdHwRPME0wS6BJoEeG
// SIG // RWh0dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3Js
// SIG // L3Byb2R1Y3RzL01pY1Jvb0NlckF1dF8yMDEwLTA2LTIz
// SIG // LmNybDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKG
// SIG // Pmh0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2Vy
// SIG // dHMvTWljUm9vQ2VyQXV0XzIwMTAtMDYtMjMuY3J0MIGd
// SIG // BgNVHSAEgZUwgZIwgY8GCSsGAQQBgjcuAzCBgTA9Bggr
// SIG // BgEFBQcCARYxaHR0cDovL3d3dy5taWNyb3NvZnQuY29t
// SIG // L1BLSS9kb2NzL0NQUy9kZWZhdWx0Lmh0bTBABggrBgEF
// SIG // BQcCAjA0HjIgHQBMAGUAZwBhAGwAXwBQAG8AbABpAGMA
// SIG // eQBfAFMAdABhAHQAZQBtAGUAbgB0AC4gHTANBgkqhkiG
// SIG // 9w0BAQsFAAOCAgEAGnTvV08pe8QWhXi4UNMi/AmdrIKX
// SIG // +DT/KiyXlRLl5L/Pv5PI4zSp24G43B4AvtI1b6/lf3mV
// SIG // d+UC1PHr2M1OHhthosJaIxrwjKhiUUVnCOM/PB6T+DCF
// SIG // F8g5QKbXDrMhKeWloWmMIpPMdJjnoUdD8lOswA8waX/+
// SIG // 0iUgbW9h098H1dlyACxphnY9UdumOUjJN2FtB91TGcun
// SIG // 1mHCv+KDqw/ga5uV1n0oUbCJSlGkmmzItx9KGg5pqdfc
// SIG // wX7RSXCqtq27ckdjF/qm1qKmhuyoEESbY7ayaYkGx0aG
// SIG // ehg/6MUdIdV7+QIjLcVBy78dTMgW77Gcf/wiS0mKbhXj
// SIG // pn92W9FTeZGFndXS2z1zNfM8rlSyUkdqwKoTldKOEdqZ
// SIG // Z14yjPs3hdHcdYWch8ZaV4XCv90Nj4ybLeu07s8n07Ve
// SIG // afqkFgQBpyRnc89NT7beBVaXevfpUk30dwVPhcbYC/GO
// SIG // 7UIJ0Q124yNWeCImNr7KsYxuqh3khdpHM2KPpMmRM19x
// SIG // HkCvmGXJIuhCISWKHC1g2TeJQYkqFg/XYTyUaGBS79ZH
// SIG // maCAQO4VgXc+nOBTGBpQHTiVmx5mMxMnORd4hzbOTsNf
// SIG // svU9R1O24OXbC2E9KteSLM43Wj5AQjGkHxAIwlacvyRd
// SIG // UQKdannSF9PawZSOB3slcUSrBmrm1MbfI5qWdcUxghYt
// SIG // MIIWKQIBATCBlTB+MQswCQYDVQQGEwJVUzETMBEGA1UE
// SIG // CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEe
// SIG // MBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgw
// SIG // JgYDVQQDEx9NaWNyb3NvZnQgQ29kZSBTaWduaW5nIFBD
// SIG // QSAyMDEwAhMzAAACznyazn2QXtK3AAAAAALOMA0GCWCG
// SIG // SAFlAwQCAQUAoIGuMBkGCSqGSIb3DQEJAzEMBgorBgEE
// SIG // AYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3
// SIG // AgEVMC8GCSqGSIb3DQEJBDEiBCANgG+77xKyj2weeoCp
// SIG // jpSkSjs8KUzbvUzh/a1cHT8uATBCBgorBgEEAYI3AgEM
// SIG // MTQwMqAUgBIATQBpAGMAcgBvAHMAbwBmAHShGoAYaHR0
// SIG // cDovL3d3dy5taWNyb3NvZnQuY29tMA0GCSqGSIb3DQEB
// SIG // AQUABIIBAETJwYyYB6sxsHwdNt8Wp8NkUOPA0ezh8iAR
// SIG // XETUKoKxhHxbDldda4llr8Izx7pkoaZCcACCa98VVq3G
// SIG // SdYYNKCd+z/tLruZ54YDNhRfdA+9oj74IzKCVB857L4L
// SIG // N0O9+xiaGIniQjLhK+81N6V6Vni3kkP1jEEEPqPvV0aJ
// SIG // cIJ+2b0Cxb7eUdKe+Qojja5AAQIN6Xnst0R7XEtAW4rj
// SIG // JccklrTIN6zjIQ3AzARI1GKzE+Rccp1CjYqHDayM2G5/
// SIG // mT5yPlHTTLPgOTAyQPJmv8+XbzHUNh6JmqqNcFD7N1DV
// SIG // CmA5UjrpGjMknA9gdv7A7ljiT5s4GzVzcO4fGgHEQLeh
// SIG // ghO3MIITswYKKwYBBAGCNwMDATGCE6MwghOfBgkqhkiG
// SIG // 9w0BBwKgghOQMIITjAIBAzEPMA0GCWCGSAFlAwQCAQUA
// SIG // MIIBWAYLKoZIhvcNAQkQAQSgggFHBIIBQzCCAT8CAQEG
// SIG // CisGAQQBhFkKAwEwMTANBglghkgBZQMEAgEFAAQg73tw
// SIG // Xo3esbL3AMDl7bR3PR//9UEzfJK34KrWJ5qhu+UCBl0W
// SIG // Jo0lqhgTMjAxOTA3MDEyMTEzMjkuMDczWjAHAgEBgAIB
// SIG // 9KCB1KSB0TCBzjELMAkGA1UEBhMCVVMxEzARBgNVBAgT
// SIG // Cldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAc
// SIG // BgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcG
// SIG // A1UECxMgTWljcm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRv
// SIG // IFJpY28xJjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOjU4
// SIG // NDctRjc2MS00RjcwMSUwIwYDVQQDExxNaWNyb3NvZnQg
// SIG // VGltZS1TdGFtcCBTZXJ2aWNloIIPHzCCBPUwggPdoAMC
// SIG // AQICEzMAAADUTxnD2ITLRWMAAAAAANQwDQYJKoZIhvcN
// SIG // AQELBQAwfDELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldh
// SIG // c2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNV
// SIG // BAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEmMCQGA1UE
// SIG // AxMdTWljcm9zb2Z0IFRpbWUtU3RhbXAgUENBIDIwMTAw
// SIG // HhcNMTgwODIzMjAyNjQwWhcNMTkxMTIzMjAyNjQwWjCB
// SIG // zjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0
// SIG // b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1p
// SIG // Y3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMgTWlj
// SIG // cm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28xJjAk
// SIG // BgNVBAsTHVRoYWxlcyBUU1MgRVNOOjU4NDctRjc2MS00
// SIG // RjcwMSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFt
// SIG // cCBTZXJ2aWNlMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
// SIG // MIIBCgKCAQEAv0i7sMQTtT74OGpouWATfTY+WnmWenhQ
// SIG // yL9pJAFK/a7LZOF32Fmgf00gimwjJckiYY72FBQx0UqD
// SIG // b3aTsusoplv47VDf27/0klDF49EJ8gBIwhEWLYTHtFNR
// SIG // g98M5wOTzVKfuhjWXK1nzPsW5/Qx6NGoQjfKDhPjMEsR
// SIG // pEzYPH8v0ef742MSvrRI4ydNswRZRX0mcdrx2hRFmIRo
// SIG // Ku4m4jadyHxkwdYBWE4mA4V1vZfS0MxFHcjbCPaW1Ufu
// SIG // Hw4NnbHsDwRv8L0QOESupwbQA7wledA0Xj+fQchPb69P
// SIG // 0lk17eyRGKneUgB+fHAL0JNbY+qqMuwGh1R0Uda3yQEb
// SIG // JQIDAQABo4IBGzCCARcwHQYDVR0OBBYEFP5k/7/EzmU2
// SIG // uNamuNJbShVglJcHMB8GA1UdIwQYMBaAFNVjOlyKMZDz
// SIG // Q3t8RhvFM2hahW1VMFYGA1UdHwRPME0wS6BJoEeGRWh0
// SIG // dHA6Ly9jcmwubWljcm9zb2Z0LmNvbS9wa2kvY3JsL3By
// SIG // b2R1Y3RzL01pY1RpbVN0YVBDQV8yMDEwLTA3LTAxLmNy
// SIG // bDBaBggrBgEFBQcBAQROMEwwSgYIKwYBBQUHMAKGPmh0
// SIG // dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2kvY2VydHMv
// SIG // TWljVGltU3RhUENBXzIwMTAtMDctMDEuY3J0MAwGA1Ud
// SIG // EwEB/wQCMAAwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDQYJ
// SIG // KoZIhvcNAQELBQADggEBAEnI5BKJf8YlLqfOo0xeUA8d
// SIG // kzKYWUUpF1roTtBH5fmvsXLsVQx84IT8WKvAcqU2/2dn
// SIG // rMP/YFtz3qSrucNPZXIdjhEOWME9ELHrBKEWzHRmfm/D
// SIG // ZuqorEPnQfaNboPXbJuyfxWc6tJn3h18adwiTJW7AIde
// SIG // Gf2e+D1v2qehOcrAtOC2l+rNRzCULRvKRgTS6o77geVG
// SIG // 4V97yLKxLMF5ZdI89jN+q+8EHIsIB+ggteUYAGOX+WXZ
// SIG // xR4/Ib/odk3ze9AD2FR+X1WlF1EzMrg9QUm4Aszmkld5
// SIG // wiR9HCbn7ji5EsAA8H6Irp8csHrRmXUl2WeVvysXxNja
// SIG // kP23cgQvEn8wggZxMIIEWaADAgECAgphCYEqAAAAAAAC
// SIG // MA0GCSqGSIb3DQEBCwUAMIGIMQswCQYDVQQGEwJVUzET
// SIG // MBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVk
// SIG // bW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0
// SIG // aW9uMTIwMAYDVQQDEylNaWNyb3NvZnQgUm9vdCBDZXJ0
// SIG // aWZpY2F0ZSBBdXRob3JpdHkgMjAxMDAeFw0xMDA3MDEy
// SIG // MTM2NTVaFw0yNTA3MDEyMTQ2NTVaMHwxCzAJBgNVBAYT
// SIG // AlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQH
// SIG // EwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29y
// SIG // cG9yYXRpb24xJjAkBgNVBAMTHU1pY3Jvc29mdCBUaW1l
// SIG // LVN0YW1wIFBDQSAyMDEwMIIBIjANBgkqhkiG9w0BAQEF
// SIG // AAOCAQ8AMIIBCgKCAQEAqR0NvHcRijog7PwTl/X6f2mU
// SIG // a3RUENWlCgCChfvtfGhLLF/Fw+Vhwna3PmYrW/AVUycE
// SIG // MR9BGxqVHc4JE458YTBZsTBED/FgiIRUQwzXTbg4CLNC
// SIG // 3ZOs1nMwVyaCo0UN0Or1R4HNvyRgMlhgRvJYR4YyhB50
// SIG // YWeRX4FUsc+TTJLBxKZd0WETbijGGvmGgLvfYfxGwScd
// SIG // JGcSchohiq9LZIlQYrFd/XcfPfBXday9ikJNQFHRD5wG
// SIG // Pmd/9WbAA5ZEfu/QS/1u5ZrKsajyeioKMfDaTgaRtogI
// SIG // Neh4HLDpmc085y9Euqf03GS9pAHBIAmTeM38vMDJRF1e
// SIG // FpwBBU8iTQIDAQABo4IB5jCCAeIwEAYJKwYBBAGCNxUB
// SIG // BAMCAQAwHQYDVR0OBBYEFNVjOlyKMZDzQ3t8RhvFM2ha
// SIG // hW1VMBkGCSsGAQQBgjcUAgQMHgoAUwB1AGIAQwBBMAsG
// SIG // A1UdDwQEAwIBhjAPBgNVHRMBAf8EBTADAQH/MB8GA1Ud
// SIG // IwQYMBaAFNX2VsuP6KJcYmjRPZSQW9fOmhjEMFYGA1Ud
// SIG // HwRPME0wS6BJoEeGRWh0dHA6Ly9jcmwubWljcm9zb2Z0
// SIG // LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL01pY1Jvb0NlckF1
// SIG // dF8yMDEwLTA2LTIzLmNybDBaBggrBgEFBQcBAQROMEww
// SIG // SgYIKwYBBQUHMAKGPmh0dHA6Ly93d3cubWljcm9zb2Z0
// SIG // LmNvbS9wa2kvY2VydHMvTWljUm9vQ2VyQXV0XzIwMTAt
// SIG // MDYtMjMuY3J0MIGgBgNVHSABAf8EgZUwgZIwgY8GCSsG
// SIG // AQQBgjcuAzCBgTA9BggrBgEFBQcCARYxaHR0cDovL3d3
// SIG // dy5taWNyb3NvZnQuY29tL1BLSS9kb2NzL0NQUy9kZWZh
// SIG // dWx0Lmh0bTBABggrBgEFBQcCAjA0HjIgHQBMAGUAZwBh
// SIG // AGwAXwBQAG8AbABpAGMAeQBfAFMAdABhAHQAZQBtAGUA
// SIG // bgB0AC4gHTANBgkqhkiG9w0BAQsFAAOCAgEAB+aIUQ3i
// SIG // xuCYP4FxAz2do6Ehb7Prpsz1Mb7PBeKp/vpXbRkws8LF
// SIG // Zslq3/Xn8Hi9x6ieJeP5vO1rVFcIK1GCRBL7uVOMzPRg
// SIG // Eop2zEBAQZvcXBf/XPleFzWYJFZLdO9CEMivv3/Gf/I3
// SIG // fVo/HPKZeUqRUgCvOA8X9S95gWXZqbVr5MfO9sp6AG9L
// SIG // MEQkIjzP7QOllo9ZKby2/QThcJ8ySif9Va8v/rbljjO7
// SIG // Yl+a21dA6fHOmWaQjP9qYn/dxUoLkSbiOewZSnFjnXsh
// SIG // bcOco6I8+n99lmqQeKZt0uGc+R38ONiU9MalCpaGpL2e
// SIG // Gq4EQoO4tYCbIjggtSXlZOz39L9+Y1klD3ouOVd2onGq
// SIG // BooPiRa6YacRy5rYDkeagMXQzafQ732D8OE7cQnfXXSY
// SIG // Ighh2rBQHm+98eEA3+cxB6STOvdlR3jo+KhIq/fecn5h
// SIG // a293qYHLpwmsObvsxsvYgrRyzR30uIUBHoD7G4kqVDmy
// SIG // W9rIDVWZeodzOwjmmC3qjeAzLhIp9cAvVCch98isTtoo
// SIG // uLGp25ayp0Kiyc8ZQU3ghvkqmqMRZjDTu3QyS99je/WZ
// SIG // ii8bxyGvWbWu3EQ8l1Bx16HSxVXjad5XwdHeMMD9zOZN
// SIG // +w2/XU/pnR4ZOC+8z1gFLu8NoFA12u8JJxzVs341Hgi6
// SIG // 2jbb01+P3nSISRKhggOtMIIClQIBATCB/qGB1KSB0TCB
// SIG // zjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0
// SIG // b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1p
// SIG // Y3Jvc29mdCBDb3Jwb3JhdGlvbjEpMCcGA1UECxMgTWlj
// SIG // cm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJpY28xJjAk
// SIG // BgNVBAsTHVRoYWxlcyBUU1MgRVNOOjU4NDctRjc2MS00
// SIG // RjcwMSUwIwYDVQQDExxNaWNyb3NvZnQgVGltZS1TdGFt
// SIG // cCBTZXJ2aWNloiUKAQEwCQYFKw4DAhoFAAMVAO0ICv9f
// SIG // 7xxmgMPCjIj5XADfKo9DoIHeMIHbpIHYMIHVMQswCQYD
// SIG // VQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4G
// SIG // A1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0
// SIG // IENvcnBvcmF0aW9uMSkwJwYDVQQLEyBNaWNyb3NvZnQg
// SIG // T3BlcmF0aW9ucyBQdWVydG8gUmljbzEnMCUGA1UECxMe
// SIG // bkNpcGhlciBOVFMgRVNOOjRERTktMEM1RS0zRTA5MSsw
// SIG // KQYDVQQDEyJNaWNyb3NvZnQgVGltZSBTb3VyY2UgTWFz
// SIG // dGVyIENsb2NrMA0GCSqGSIb3DQEBBQUAAgUA4MQRFjAi
// SIG // GA8yMDE5MDcwMTEyNTY1NFoYDzIwMTkwNzAyMTI1NjU0
// SIG // WjB0MDoGCisGAQQBhFkKBAExLDAqMAoCBQDgxBEWAgEA
// SIG // MAcCAQACAgO0MAcCAQACAhnjMAoCBQDgxWKWAgEAMDYG
// SIG // CisGAQQBhFkKBAIxKDAmMAwGCisGAQQBhFkKAwGgCjAI
// SIG // AgEAAgMW42ChCjAIAgEAAgMHoSAwDQYJKoZIhvcNAQEF
// SIG // BQADggEBAIv/F6Td2a3/FyBtCTMs+tMJOMAs/IU9G7VD
// SIG // DCGetUAFKeu77w1CO6NpfR0ZPL8x4QNy6ABeKMArCBQW
// SIG // 2yIXJ6UlO9N3GwVawH0cNLzhDFx4v2f9pVRV1pMNyfPl
// SIG // D8/vYTCLthavLHse2rrfrWGQVMiKrrrQbCXiaeDz1qvU
// SIG // qBYUK8RDXiQTUuy+XhrvcUGJCbGh+p/F0eNamclzddKr
// SIG // YL7XY8SXHcWeSUan4h/mHYl22YY2goAwHHoeqOwrVGri
// SIG // Q6am7ZfDkq1ofpp2ohVPsIdf/Rwg8XwYJDNMHe0HeVUd
// SIG // ttvDIK8/Nkqy2wAOjdjhIKrM8jyQPdP01ZZjLnCPgNQx
// SIG // ggL1MIIC8QIBATCBkzB8MQswCQYDVQQGEwJVUzETMBEG
// SIG // A1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
// SIG // ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9u
// SIG // MSYwJAYDVQQDEx1NaWNyb3NvZnQgVGltZS1TdGFtcCBQ
// SIG // Q0EgMjAxMAITMwAAANRPGcPYhMtFYwAAAAAA1DANBglg
// SIG // hkgBZQMEAgEFAKCCATIwGgYJKoZIhvcNAQkDMQ0GCyqG
// SIG // SIb3DQEJEAEEMC8GCSqGSIb3DQEJBDEiBCA0XtbWHkli
// SIG // faCiKhl7wWlIboBriTJ47q/zF8jPGHfKMTCB4gYLKoZI
// SIG // hvcNAQkQAgwxgdIwgc8wgcwwgbEEFO0ICv9f7xxmgMPC
// SIG // jIj5XADfKo9DMIGYMIGApH4wfDELMAkGA1UEBhMCVVMx
// SIG // EzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1Jl
// SIG // ZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3Jh
// SIG // dGlvbjEmMCQGA1UEAxMdTWljcm9zb2Z0IFRpbWUtU3Rh
// SIG // bXAgUENBIDIwMTACEzMAAADUTxnD2ITLRWMAAAAAANQw
// SIG // FgQUhPIorMqr0/MceQxGbyYQdSQKY0YwDQYJKoZIhvcN
// SIG // AQELBQAEggEAftBYDT8Tr0ZctDOLyokkh3vsmEwaBDwO
// SIG // jXMX7PUv9UsyHw6qao8fajhnnrE9JMgE8vMeP+Qc0AWx
// SIG // DsV9cH7tjuxXMhPiW/r518d7IqnRKQa/1o/ePVkPenC9
// SIG // DtYPMcfUDZFYHlAugODLcRh3XEXVFWBgsraO+JmhlabE
// SIG // n18WG3pTETS7nDGL6KaRKxLmzolVNCZSEMJ9q6hOthUc
// SIG // //6NwaqCgu+GWXW41olHbBuVVZfHpsVpCW9EbZn3A0Vf
// SIG // MEEYqcZu937DOM5bJdXIjOUjJQV/O5Od+h8nxbZlqfv1
// SIG // 9BH/MXx8iz+3NPSg42tfFvKF1JgtiozI4X0bq9T7csPNWA==
// SIG // End signature block
